<?php

namespace Tangible\LifterElements;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

/**
 * Handle the logic specific to the elementor editor
 */
class Editor {

  /**
   * Constructor
   */
  public function __construct() {
    
    $this->enqueue_builder_dependencies();  
    $this->courses_actions(); 
  }

    /**
   * Enqueue scripts and style needed by the widgets in the
   * builder mode
   */
  private function enqueue_builder_dependencies() {

    /**
     * We need to manually enqueue media element's dependencies in the builder
     * 
     * @see https://github.com/elementor/elementor/issues/3670  
     */
    add_action( 'elementor/frontend/after_register_styles', function() {
      wp_register_style( 'le-legacy-wp-mediaelement', includes_url() .'/js/mediaelement/mediaelementplayer-legacy.min.css' );
      wp_register_style( 'le-wp-mediaelement', includes_url() .'/js/mediaelement/wp-mediaelement.min.css' );
    });

    /**
     * Css specific to the builder
     */
    add_action( 'elementor/frontend/after_register_styles', function() {
      wp_register_style( 'le-builder-style', LifterElements_URL . '/assets/build/builder.min.css', [], LifterElements_VER );
    });

    /**
     * Enqueue all the styles
     */
    add_action( 'elementor/preview/enqueue_styles', function() {
      wp_enqueue_style( 'llms-assignments' );
      wp_enqueue_style( 'le-legacy-wp-mediaelement' );
      wp_enqueue_style( 'le-wp-mediaelement' );
      wp_enqueue_style( 'le-builder-style' );
    });
  }

  /**
   * Action needed when editing a course
   */
  private function courses_actions() {
    
    add_action( 'elementor/preview/init', function() {
      
      $id = utils\get_edited_post_id();
      
      /**
       * Add the class to the body if needed by specific post type  
       */
      add_filter('body_class', function( $classes ) use( $id ) {

        if( lift\is_course( $id ) ) $classes[] = 'course';
        if( lift\is_quiz( $id ) ) $classes[] = 'single-llms_quiz';

        return $classes;
      }, 10 );

    }, 99, 1 );
  }

}
