<?php

namespace Tangible\LifterElements;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Admin\CustomSettings;
use Tangible\LifterElements\Admin\Documentation;
use Tangible\ElementLogic as element_logic;

/**
 * @see https://docs.tangibletasks.com/modules/tangible-plugin-framework
 */
class Framework {

  private $conditional_ui_group;

  /**
   * Constructor
   *
   * @param      object  $plugin  Plugin instance (needed for the documentation)
   */
	public function __construct( $plugin, $callback ) {

    add_action( 'plugins_loaded', function() use( $plugin, $callback ) {

      $framework = tangible();

      $plugin->api = $framework
        ->register_plugin([
    			'name' 				     => 'lifter-elements',
    			'title' 			     => 'Lifter Elements',
    			'setting_prefix'   => 'lifterelements',
    			'file_path' 		   => LifterElements_FILE,
    			'item_id'          => '5123',
          'url'              => plugins_url( '/', LifterElements_FILE ),
    			'version'          => LifterElements_VER,
    		])
        ->register_settings([
    			'js' => plugins_url( 'assets/build/settings.min.js', LifterElements_FILE ),

          // Settings page title
          'title_callback' => function($plugin, $tabs, $active_tab) {
          ?>
            <div class="lifter-elements-logo">
              <img class="lifter-elements-logo-image"
                  alt="Lifter Elements"
                  src="<?= $plugin['url'] ?>assets/images/lifter-elements-logo.png"
              />
              <?= $plugin['title'] ?>
            </div>
          <?php
          },
          'tabs' => [
            'doc' => [
              'title'    => __( 'Documentation', 'lifter-elements' ),
              'callback' => function() use ($plugin) {
                $doc = new Documentation($plugin);
                $doc->render();
              },
            ],
            'settings' => [
              'title'   => __( 'Settings', 'lifter-elements' ),
              'css'     => plugins_url( 'assets/lib/select2.min.css', LifterElements_FILE ),
              'js'      => plugins_url( 'assets/lib/select2.min.js', LifterElements_FILE ),
              'callback'=> function($plugin_config, $settings, $settings_key) use ($plugin) {
                $redirect = new CustomSettings( $plugin, $plugin_config, $settings, $settings_key );
                $redirect->render();
              },
            ],
          ],
        ])
        ->register_dependencies([
          // No need to check for Elementor (@see includes/Elementor.php init())
          'lifterlms/lifterlms.php' => [
            'title' => 'LifterLMS',
            'fallback_check' => function() { return class_exists( 'LifterLMS' ); }
          ],
        ]);


      $this->conditional_ui_group = $plugin->api->register_plugin_group( 'elementor_conditional_ui' );
      element_logic\init( $this->conditional_ui_group );
      $callback();
            
    }, 6); // Before plugin instance is used at priority 7 in ./Plugin.php
  }

}
