<?php
namespace Tangible\LifterElements\Admin;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;

/**
 * Set conditions for allowing Dripped Lesson View
 */
class CustomSettings {

  /**
   * Constructor
   *
   * @param      object  $plugin         The plugin instance
   * @param      array   $plugin_config  All data passed to plugin registration
   * @param      array   $settings       Plugin settings data
   * @param      string  $settings_key   Plugin settings key
   *
   */
  public function __construct( $plugin, $plugin_config, $settings, $settings_key ) {

    if( !is_object( $plugin ) ) return;

    $this -> plugin = $plugin;
    $this -> plugin_config = $plugin_config;
    $this -> settings = $settings;
    $this -> settings_key = $settings_key;
  }

  /**
   * Render the HTML
   */
  public function render() {

    $settings = $this -> settings;
    $settings_key = $this -> settings_key;
    $course_data = $this -> course_cats_data( $settings, $settings_key, 'lessons' );
    $all_lessons_data = $this -> all_lessons_data( $settings, $settings_key, 'lessons' );

    // Whether to show or hide Course Categories select box depending on all_lessons_data setting
    $data_expand = ( $all_lessons_data['field_value'] == 'true' ) ? ' data-expand="false"' : ' data-expand="true"' ;
    ?>
      <main>
				<div class="lifter-elements-dripped-lesson-view">
					<div class="section">
            <div class="section-title">
              <b><?= __( 'Dripped Lesson View Allowed', 'lifter-elements' ) ?></b>
            </div>

            <div class="part">
              <div class="part-title"><b><?= __( 'Allow Dripped Lesson View ', 'lifter-elements' ) ?></b><?= __( 'for all Courses', 'lifter-elements' ) ?></div>
              <p>
                <div id="drip_view_all_lessons">
                <?php
                  $this  -> plugin_config ['framework'] -> render_setting_field([
                    'type'  => 'checkbox',
                    'name'  => $all_lessons_data['full_name'],
                    'value' => $all_lessons_data['field_value'],
                    'label' => __( 'If checked, view of ALL Lessons with Drip Date in future will be allowed', 'lifter-elements' )
                  ]);
                ?>
                </div>
              </p>
            </div>

						<div class="part drip-view-course-categories" <?= $data_expand ?> >
							<div class="part-title"><b><?= __( 'Choose Specific Course Categories ', 'lifter-elements' ) ?></b><?= __( 'in which Dripped Lessons view will be allowed', 'lifter-elements' ) ?></div>
              <p>
                <?= $course_data['empty_option_input'] ?>
                <select name="<?= $course_data['full_name'] ?>"  multiple>
                  <?= $course_data['options'] ?>
                </select>
              </p>
						</div>
					</div>
        <?php
				submit_button();
        ?>
				</div>
      </main>
    <?php
  }

  /**
   * Return all dripped lessons view allowed data
   *
   * @param $settings
   * @param $settings_key
   * @param $settings_cat_name
   * @return array
   *
   */
  public function all_lessons_data( $settings, $settings_key, $settings_cat_name ) {

    $field_name = 'drip_view_all_lessons';
    $field_value = isset($settings[$settings_cat_name][ $field_name ])
      ? $settings[$settings_cat_name][ $field_name ]
      : ''; // Default value

    $full_field_name = "{$settings_key}[{$settings_cat_name}][{$field_name}]";

    $data = [
      'field_value' => $field_value,
      'full_name' => $full_field_name
      ];

    return $data;
  }

  /**
   * Return specific course categories for dripped lessons view allowed data
   *
   * @param $settings
   * @param $settings_key
   * @param $settings_cat_name
   * @return array
   *
   */
  public function course_cats_data( $settings, $settings_key, $settings_cat_name ) {

    // course categories select field; options: course categories
    $field_name = 'drip_view_course_categories';
    $field_value = ( isset($settings[$settings_cat_name][ $field_name ]) && is_array($settings[$settings_cat_name][ $field_name ]) )
      ? $settings[$settings_cat_name][ $field_name ] : [];
    $full_field_name = "{$settings_key}[{$settings_cat_name}][{$field_name}][]";

    // check if 'all_lessons_data' is set to true - if yes then we don't need drip_view_categories selected
    $all_lessons_data = $this -> all_lessons_data( $settings, $settings_key, 'lessons' );
    if( $all_lessons_data['field_value'] === 'true' ) $field_value = [];

    $options = '';
    $taxonomy = 'course_cat';
    $course_categories =  get_terms([
      'taxonomy' => $taxonomy,
      'hide_empty' => false,
    ]);
    foreach( $course_categories as $k => $c ){
      if (!is_object($c)) continue;
      $selected = in_array( $c->term_id, $field_value ) ? ' selected="selected" ' : '';
      $options .= '<option value="'.$c->term_id.'" '.$selected.'>'.$c->name.'</option>';
    }

    $data = [
      'options' => $options,
      'full_name' => $full_field_name,

      // Pass empty value when nothing is selected
      // @see https://stackoverflow.com/questions/1132303/how-to-post-empty-select-multiple-html-elements-if-empty#1132343
      'empty_option_input' =>
        '<input type="hidden" name="' . "{$settings_key}[{$settings_cat_name}][{$field_name}]" . '" value="" />'
      ,
    ];

    return $data;
  }
}
