<?php

namespace Tangible\LifterElements\Admin;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;

/**
 * Render the shortcode/module/dynamic tags documentation
 */
class Documentation {

    
  /**
   * Constructor
   *
   * @param      object  $plugin  The plugin instance
   */
  public function __construct( $plugin ) {

    if( !is_object($plugin) ) return;

    $this->plugin = $plugin; 
  }


  /**
   * Render the HTML
   */
  public function render() {

    ?>
    
    <div class="lifter-elements-doc">
      <aside>
        <p>
          <input class="search-box" type="text" name="search_shortcode" placeholder="Search shortcodes..">
        </p>
      </aside>
      <main>
      <?php
        $this->render_widgets();
        $this->render_dynamic_tags();
        $this->render_shortcodes();
      ?>
      </main>
    </div>

    <?php
  }  


  /**
   * Render widget documentation
   */
  protected function render_widgets() {

    $widgets = $this->plugin->widgets; ?>

    <div class="area">
      <h3 class="area-title"><?= __( 'LifterLMS Widgets', 'lifter-elements' ) ?></h3>
    </div>

    <?php foreach( $widgets as $category_slug => $category_widgets ): ?>

      <div class="section">
        <div class="section-title">
          <b><?= ucfirst($category_slug) ?></b>
        </div>

        <?php foreach( $category_widgets as $widget_slug => $widget ): ?>

          <?php 
            $dependencies = "";
            
            if( isset($widget['dependencies']) ){
                
              foreach( $widget['dependencies'] as $dependency ){
                if( $dependencies !== "") $dependencies = $dependencies . ', ';
                $dependencies .= utils\kebabcase_to_text($dependency);
              }
              $dependencies = '(' . $dependencies . ')';
            } 
          ?>

          <div class="part">
            <div class="part-title"><b><?= $widget['label'] ?></b> <?= $dependencies ?></div>
            <?= isset($widget['description']) ? $widget['description'] : '' ?>
          </div>
        <?php endforeach; ?> 

      </div>     
    <?php endforeach; 

  } 


  /**
   * Render dynamic tag documentation
   */
  protected function render_dynamic_tags() {

    $dynamic_tags = $this->plugin->dynamic_tags; ?>

    <div class="area">
      <h3 class="area-title"><?= __( 'LifterLMS Dynamic Tags', 'lifter-elements' ) ?></h3>
    </div>

    <?php foreach( $dynamic_tags as $category_slug => $category ): ?>

      <div class="section" data-expand="false">
        <div class="section-title">
          <b><?= $category['label'] ?></b>
          <span class="expand-toggle"></span>
        </div>
        
        <?php foreach( $category['tags'] as $tag_slug => $tag ): ?>
          <div class="part">
            <?= $tag['label'] ?><br>
          </div>
        <?php endforeach; ?> 
      </div>
    <?php endforeach; 
  } 


  /**
   * Render dynamic tag documentation
   */
  protected function render_shortcodes() {

    $shortcodes = $this->plugin->shortcodes; ?>
    
    <div class="area">
      <h3 class="area-title"><?= __( 'LifterLMS Shortcodes', 'lifter-elements' ) ?></h3>
    </div>

    <?php foreach( $shortcodes as $category_slug => $category ): ?>

      <div class="section" data-expand="false">
        <div class="section-title">
          <b><?= $category['label'] ?></b>
          <span class="expand-toggle"></span>
        </div>
          
        <?php foreach( $category['sections'] as $section_slug => $section ): ?>
          <div class="part">
            <div class="part-title">
              <b><?= utils\kebabcase_to_text( $section_slug ) ?></b>
            </div>
            <?php foreach( $section as $shortcode ): ?>
              <div class="item-name">
                  [<?= $shortcode['tag'] ?>]
              </div>
            <?php endforeach; ?> 
          </div>
        <?php endforeach; ?> 
      </div>
    <?php endforeach; 
  } 
            
}
