<?php

namespace Tangible\LifterElements\DynamicTags\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;

/**
 * @see https://developers.elementor.com/dynamic-tags/
 * @see https://lifterlms.com/docs/lifterlms-database-description/#course-data
 */
class Information extends \Elementor\Core\DynamicTags\Tag {
	

	/**
	 * Slug of the dynamic tag
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'course-information';
	}


	/**
	 * Title of the dynamic tag
	 *  
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Info', 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/dynamic-tags/
	 * 
	 * @return string
	 */
	public function get_group() {
		return 'le-course';
	}


	/**
	 * @see https://developers.elementor.com/dynamic-tags/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( \Elementor\Modules\DynamicTags\Module::TEXT_CATEGORY );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {

		$this->add_control(
			'information_field',
			[
				'label'   => __( 'Field', 'lifter-elements' ),
			  'type'    => \Elementor\Controls_Manager::SELECT,
			  'default' => 'start_date',
				'options' => [
					'capacity' 						      => __( 'Capacity', 'lifter-elements' ),
					'capacity_message' 				  => __( 'Capacity message', 'lifter-elements' ),
					'content_restricted_message'=> __( 'Content restricted message', 'lifter-elements' ),
					'course_closed_message' 		=> __( 'Course closed message', 'lifter-elements' ),
					'course_opens_message' 			=> __( 'Course opens message', 'lifter-elements' ),
					'enable_capacity' 				  => __( 'Enable capacity', 'lifter-elements' ),
					'enrollment_closed_message' => __( 'Enrollment closed message', 'lifter-elements' ),
					'enrollment_end_date' 			=> __( 'Enrollment end date', 'lifter-elements' ),
					'enrollment_opens_message'  => __( 'Enrollment opens message', 'lifter-elements' ),
					'enrollment_period' 			  => __( 'Enrollment period', 'lifter-elements' ),
					'enrollment_start_date' 		=> __( 'Enrollment start date', 'lifter-elements' ),
					'end_date' 						      => __( 'End date', 'lifter-elements' ),
					'has_prerequisite' 				  => __( 'Has prerequisite', 'lifter-elements' ),
					'prerequisite' 					    => __( 'Prerequisite', 'lifter-elements' ),
					'prerequisite_track' 			  => __( 'Prerequisite track', 'lifter-elements' ),
					'start_date' 					      => __( 'Start date', 'lifter-elements' ),
					'length' 						        => __( 'Length', 'lifter-elements' ),
					'time_period' 					    => __( 'Time period', 'lifter-elements' ),
				]
			]
		);

		// If the control is a date, we add a format field
		$this->add_control(
			'information_date_format', [
				'label'   => __( 'Date format', 'lifter-elements' ),
				'type'    => \Elementor\Controls_Manager::SELECT,
				'default' => 'Y-m-d',
				'options' => [
					'Y-m-d'  	=> __( 'Y-m-d', 'lifter-elements' ),
					'F j, Y'  => __( 'F j, Y', 'lifter-elements' ),
					'm.d.y' 	=> __( 'm.d.y', 'lifter-elements' ),
					'j, n, Y' => __( 'j, n, Y', 'lifter-elements' ),
					'D M j G' => __( 'D M j G', 'lifter-elements' ),
					'custom' 	=> __( 'Custom format', 'lifter-elements' ),	
				],
				'conditions' => [
					'terms' => [
						[
							'name' 		=> 'information_field',
							'operator'=> 'in',
							'value' 	=> [
								'enrollment_end_date',
								'enrollment_start_date',
								'start_date',
								'end_date',
							],
						],
					]
				]
			]
		);

		// If the user choose a custom format for a date
		$this->add_control(
			'information_date_custom_format', [
				'label' 		=> __( 'Custom date format', 'lifter-elements' ),
				'type' 			=> \Elementor\Controls_Manager::TEXT,
				'conditions'=> [
					'terms' => [
						[
							'name' 		=> 'information_field',
							'operator'=> 'in',
							'value' 	=> [
								'enrollment_end_date',
								'enrollment_start_date',
								'start_date',
								'end_date',
							],
						],
						[
							'name' 		=> 'information_date_format',
							'operator'=> 'in',
							'value' 	=> ['custom'],
						],
					]
				]
			]
		);
	}


	/**
	 * Display the selected key after the name of the dynamic tag
	 * 
	 * @return string
	 */
	public function get_panel_template_setting_key() {
		return 'information_field';
	}


	/**
	 * Check if current key value is a date 
	 *
	 * @param string $key
	 * 
	 * @return boolean
	 */
	protected function is_in_date_field( $key ) {
		
		$date_keys = array(
			'enrollment_end_date',
			'enrollment_start_date',
			'end_date',
			'start_date',
		);

		return in_array( $key, $date_keys );
	}


	/**
	 * Output result
	 * 
	 * @return string
	 */
	public function render() {

		$key = $this->get_settings( 'information_field' );	
		$type = $this->is_in_date_field( $key ) ? 'date' : 'string';
			
		if( $type === 'date' && $this->get_settings( 'information_date_format' ) === 'custom' ) {
			$format = $this->get_settings( 'information_date_custom_format' );
		}
		else if( $type === 'date' ) {
			$format = $this->get_settings( 'information_date_format' );
		}
		else {
			$format = 'Y-m-d';
		}

		$course_id =  (int) lift\course_id();
		
		echo do_shortcode( '[lifterlms_course_info id="' . $course_id . '" key="' . $key . '" type="' . $type . '" date_format="' . $format . '"]' );
	}
}
