<?php

namespace Tangible\LifterElements\DynamicTags\Lessons;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;

/**
 * @see https://developers.elementor.com/dynamic-tags/
 */
class DripDays extends \Elementor\Core\DynamicTags\Tag {


	/**
	 * Slug of the dynamic tag
	 *
	 * @return string
	 */
	public function get_name () {
		return 'lesson-drip-days';
	}


	/**
	 * Title of the dynamic tag
	 *
	 * @return string
	 */
	public function get_title () {
      return __( 'Lesson Drip Days', 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/dynamic-tags/
	 *
	 * @return string
	 */
	public function get_group () {
      return 'le-lesson';
	}


	/**
	 * @see https://developers.elementor.com/dynamic-tags/
	 *
	 * @return array
	 */
	public function get_categories () {
      return array( \Elementor\Modules\DynamicTags\Module::TEXT_CATEGORY );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls () {
		
    $this->add_control(
	    'round_day',
			[
				'label'       => __( 'Round Day', 'lifter-elements' ),
				'type'        => \Elementor\Controls_Manager::SWITCHER,
				'label_on'    => __( 'Yes', 'lifter-elements' ),
				'label_off'   => __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default'     => 'no',
			]
		);

		$this->add_control(
	    'current_next',
		 	[
				'label' 	=> __( 'For which lesson', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'current',
				'options'	=> [
					'current' => __( 'Current', 'lifter-elements' ),
					'next'		=> __( 'Next', 'lifter-elements' )
				]
		 	]
		);
	}


	/**
	 * Output result
	 *
	 * @return string
	 */
	public function render () {

    $which_lesson = $this->get_settings( 'current_next' ) ? : 'current';

		// @see betterlifter/lesson/drip
		$html = lift\lesson_timer_html([
			'id' =>  $this->get_id(),
			'show' => 'days',
			'round' => $this->get_settings( 'round_day' ) ? : 'no',
			'which_lesson' => $which_lesson
		]);

		echo $html;
	}
}
