<?php

namespace Tangible\LifterElements\Extensions;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;

/**
 * Logic related to the private area extensions
 * 
 * @see https://lifterlms.com/product/private-areas/
 */
new class{

  /**
   * Constructs a new instance.
   */
  public function __construct() {
    $this->replace_course_content();
  }


  /**
   * By default, the private area content use the course template. This function
   * remove the course template on private area page
   */
  private function replace_course_content() {

    /**
     * Remove the template if we are not on private area (for some reasons, elementor
     * will apply the course template otherwise)
     * 
     * @see https://github.com/elementor/elementor/issues/4839
     * @see elementor-pro/modules/theme-builder/classes/get_location_templates()
     */
    add_filter( 'elementor/theme/get_location_templates/template_id', function( $template_id ) {

      $document = \ElementorPro\Modules\ThemeBuilder\Module::instance()->get_document( $template_id );
      if( empty($document) ) return $template_id;

      if( in_array( $document->get_location(), ['footer', 'header'] ) ) return $template_id;

      if( \is_llms_private_area() ) return '';

      return $template_id;
    }, 10, 2 );

  }

};
