<?php

namespace Tangible\LifterElements\Groups\ElementorConditionalUI;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;

/**
 * Defined fields
 */
add_filter( 'le-conditional-fields', function( $fields ) {
  $fields []= [ 
    'name'      => 'completion_status', 
    'label'     => __( 'Completion status', 'lifter-elements' ),
    'values'  => [
      [
        'type'           => 'select',
        'before_operand' => true,
        'options'        => [
          'current_lesson' => [
            'label'   => __( 'Current lesson', 'lifter-elements' ),
            'name'    => 'current_lesson',
          ],
          'current_course' => [
            'label'   => __( 'Current course', 'lifter-elements' ),
            'name'    => 'current_course',
          ],
          'specific_course' => [
            'label'   => __( 'Specific course', 'lifter-elements' ),
            'name'    => 'specific_course',
          ],
          'specific_lesson' => [
            'label'   => __( 'Specific lesson', 'lifter-elements' ),
            'name'    => 'specific_lesson',
          ],
        ],
      ],
    ],
    'operands'  => [
      [ 
        'name'  => 'is', 
        'label' => __( 'is complete', 'lifter-elements' ),  
      ],
      [ 
        'name'  => 'is_not', 
        'label' => __( 'is incomplete', 'lifter-elements' )
      ],
    ],
    'values_2'    => [
      [ 
        'type'           => 'select',
        'name'           => 'course_id', 
        'options'        => course_options(),
        'before_operand' => true,
        'visibility'  => [
          [
            'field'   => 'values',
            'value'   => ['specific_course'],
          ]
        ] 
      ],
    ],
    'values_3'    => [
      [ 
        'type'           => 'select',
        'name'           => 'lesson_id', 
        'options'        => lessons_options(),
        'before_operand' => true,
        'visibility'  => [
          [
            'field'   => 'values',
            'value'   => ['specific_lesson'],
          ]
        ]
      ],
    ],
  ]; 
  return $fields;
}, 10, 1 );


/**
 * Evaluate visibility
 */
add_filter( 'le-conditional-widget-visibility', function( $visibility, $rule ) {
  if( !isset($rule['field']) || $rule['field'] !== 'completion_status' ) return $visibility;

  $operand = $rule['operand'];

  switch( $rule['value'] ) {
    case 'current_lesson':
      $visibility = $operand === 'is' ? lift\has_complete_lesson() : !lift\has_complete_lesson();
      break;

    case 'current_course':
      $visibility = $operand === 'is' ? lift\has_complete_course() : !lift\has_complete_course();      
      break;

    case 'specific_course':
      $course_id = (int) $rule['value_2'];
      $visibility = $operand === 'is' ? lift\has_complete_course( $course_id ) : !lift\has_complete_course( $course_id );      
      break;

    case 'specific_lesson':
      $lesson_id = (int) $rule['value_3'];
      $visibility = $operand === 'is' ? lift\has_complete_lesson( $lesson_id ) : !lift\has_complete_lesson( $lesson_id );
      break;
  }

  return $visibility;
}, 10, 2 );
