<?php

namespace Tangible\WordpressVideo;

/**
 * Parent class for all the other
 * 
 * Chids will need to have a set_output() and a
 * apply_params method defined
 */
class Base {

  /**
  * Url to the video
  *
  * @var        string
  */
  public $url = '';


  /**
  * HTML for the video output
  *
  * @var        string   HTML
  */
  public $output = '';


  /**
  * False if not set
  * Otherwise, integer for width in pixel 
  *
  * @var        mixed
  */
  protected $width = false;


  /**
  * False if not set
  * Otherwise, integer for width in pixel 
  *
  * @var        mixed
  */
  protected $height = false;


  /**
  * If we should display or not in the final video output
  *
  * @var        boolean
  */
  protected $controls = true;


  /**
  * Autoplay disable by default
  *
  * @var        boolean
  */
  protected $autoplay = false;


  /**
  * Autoplay disable by default
  *
  * @var        boolean
  */
  protected $muted = false;


  /**
  * Can be direct_link, youtube or vimeo  
  *
  * @var        string
  */
  protected $video_type = '';


  /**
   * Constructor
   *
   * @param      string  $url    The url
   * @param      array   $args   The arguments
   */
  public function __construct( $url, $args = array() ) {

    $this->url = $url;

    if( array_key_exists('width', $args) ) {
      $this->width =  (int) $args['width'];
    }

    if( array_key_exists('height', $args) ) {
      $this->height =  (int) $args['height'];
    }

    if( array_key_exists('controls', $args) && is_bool($args['controls']) ) {
      $this->controls =  (boolean) $args['controls'];
    }

    if( array_key_exists('autoplay', $args) && is_bool($args['autoplay']) ) {
      $this->autoplay =  (boolean) $args['autoplay'];
    }

    if( array_key_exists('muted', $args) && is_bool($args['muted']) ) {
      $this->muted =  (boolean) $args['muted'];
    }
  }


  /**
  * Gets the final HTML output.
  *
  * @return     string  The HTML output.
  */
  public function get_output() {

    $this->set_output();
    
    if( empty($this->output) ) return '';

    $this->apply_params(); 

    return $this->output;
  } 
}





