<?php

namespace Tangible\WordpressVideo;

class Link extends Base {

    /**
    * Can be direct_link, youtube or vimeo  
    *
    * @var        string
    */
    protected $video_type = 'direct_link';


    /**
    * Get the video HTML output using a wordpress function
    */
    public function set_output() {

      if( $this->url === '' ) return '';

      $args = array( 'src' => $this->url );

      if( $this->width )  $args['width'] = (int) $this->width;
      if( $this->height ) $args['height'] = (int) $this->height;
    
      // wp_video_shortcode() handle more params than wp_oembed_get()
      if( $this->autoplay === true ) $args['autoplay'] = 'on';

      $this->output = wp_video_shortcode( $args );
    }


    /**
    * Set the video according to the options
    * 
    * @param bool    $enable
    * @param string  $content HTML
    *
    * @return string HTML 
    */
    public function apply_params() {
      $this->set_param( 'controls', $this->controls );
      $this->set_param( 'autoplay', $this->autoplay );
      $this->set_param( 'muted', $this->muted );
    }


    /**
    * Sets the local video controls
    */
    private function set_param( $param, $value ) {

      $doc = new \DOMDocument();
      // use of the @ -> video tag is not recognized by the loadHTML method (but works tho)
      @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $this->output );

      // Disable controls
      $controls = $doc->getElementsByTagName('video');
      foreach ($controls as $key => $control) {
        $value === true ?
          $control->setAttribute( $param, '' ) :
          $control->removeAttribute( $param );
      }

      // Remove default size
      $divs = $doc->getElementsByTagName('div');
      foreach ($divs as $key => $div) {
        $div->removeAttribute('style');
      }

      // remove html & body tags
      $html = $doc->getElementsByTagName('html')[0];
      $body = $html->getElementsByTagName('body')[0];
      foreach($body->childNodes as $child) {
        $doc->appendChild($child);
      }
      $doc->removeChild($html);

      $this->output = $doc->saveHTML();
    }

}





