<?php

namespace Tangible\WordpressVideo;

class Vimeo extends Base {

  /**
  * Can be direct_link, youtube or vimeo  
  *
  * @var        string
  */
  protected $video_type = 'vimeo';


  /**
  * Get the video HTML output using a wordpress function
  */
  public function set_output() {

    if( $this->url === '' ) return '';

    $args = array();

    if( $this->width )  $args['width'] = (int)$this->width;
    if( $this->height ) $args['height'] = (int) $this->height;

    $this->output = wp_oembed_get( $this->url, $args );
  }


  /**
  * Set the video according to the options
  * 
  * @param bool    $enable
  * @param string  $content HTML
  *
  * @return string HTML 
  */
  public function apply_params() {

    $this->set_param( 'controls', $this->controls );
    $this->set_param( 'autoplay', $this->autoplay );
    $this->set_param( 'muted', $this->muted );
    
    // For some reasons, height seems to not be set so we will set it manually
    $this->set_height();
  }


  /**
  * Sets the vimeo controls.
  * Not that this option will works only for Plus, PRO, or Business account
  *
  * @param string     $param
  * @param boolean    $value
   * 
  * @see https://github.com/vimeo/player.js#embed-options
  */
  private function set_param( $param, $value ) {

    $value = $value === true ? '1' : '0';

    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $this->output );
    $iframes = $doc->getElementsByTagName('iframe');

    foreach ($iframes as $key => $iframe) {
      $url = $iframe->getAttribute('src');
      $url = add_query_arg( $param, $value, $url );
      $iframes[$key]->setAttribute('src', $url);
    }

    // remove html & body tags
    $html = $doc->getElementsByTagName('html')[0];
    $body = $html->getElementsByTagName('body')[0];
    foreach($body->childNodes as $child) {
      $doc->appendChild($child);
    }
    $doc->removeChild($html);

    $this->output = $doc->saveHTML();
  }


  /**
  * Set the height on the iframe
  * 
  * @see https://www.php.net/manual/en/class.domdocument.php
  * @see https://www.php.net/manual/en/class.domnodelist.php
  */
  function set_height() {

    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $this->output );
    $iframes = $doc->getElementsByTagName( 'iframe' );

    foreach( $iframes as $key => $iframe ) {
      $iframe->setAttribute( 'height', $this->height );
    }

    // remove html & body tags
    $html = $doc->getElementsByTagName('html')[0];
    $body = $html->getElementsByTagName('body')[0];
    foreach($body->childNodes as $child) {
      $doc->appendChild($child);
    }
    $doc->removeChild($html);

    $this->output = $doc->saveHTML();
  }

}





