<?php

namespace Tangible\WordpressVideo;

/**
 * Fallback class
 *
 * If the url match none of the others class, we will use this one and try
 * to use the WPembed function (will only work with the height and width option)
 */
class WPEmbed extends Base {


  /**
  * Can be direct_link, youtube or vimeo  
  *
  * @var        string
  */
  protected $video_type = 'embed';


  /**
  * Get the video HTML output using a wordpress function
  */
  public function set_output() {

    if( $this->url === '' ) return '';

    $args = [];

    if( $this->width )  $args['width']  = (int)$this->width;
    if( $this->height ) $args['height'] = (int) $this->height;

    $this->output = wp_oembed_get( $this->url, $args );
  }


  /**
  * Set the video according to the options
  * 
  * @param bool    $enable
  * @param string  $content HTML
  *
  * @return string HTML 
  */
  public function apply_params() {  
    $this->set_height();
  }
  

  /**
  * Set the height on the iframe
  * 
  * @see https://www.php.net/manual/en/class.domdocument.php
  * @see https://www.php.net/manual/en/class.domnodelist.php
  */
  function set_height() {

    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $this->output );
    $iframes = $doc->getElementsByTagName( 'iframe' );

    foreach( $iframes as $key => $iframe ) {
      $iframe->setAttribute( 'height', $this->height );
    }

    // remove html & body tags
    $html = $doc->getElementsByTagName('html')[0];
    $body = $html->getElementsByTagName('body')[0];
    foreach($body->childNodes as $child) {
      $doc->appendChild($child);
    }
    $doc->removeChild($html);

    $this->output = $doc->saveHTML();
  }

}





