<?php

namespace Tangible\WordpressVideo;

class Youtube extends Base {

  /**
  * Can be direct_link, youtube or vimeo  
  *
  * @var        string
  */
  protected $video_type = 'youtube';


  /**
  * Get the video HTML output using a wordpress function
  */
  public function set_output() {
      
    if( $this->url === '' ) return '';

    $args = array();

    if( $this->width )  $args['width'] = (string) $this->width;
    if( $this->height ) $args['height'] = (string) $this->height;

    $this->output = wp_oembed_get( $this->url, $args );
  }


  /**
  * Set the video according to the options
  * 
  * @param bool    $enable
  * @param string  $content HTML
  *
  * @return string HTML 
  */
  public function apply_params() {

    $this->set_param( 'controls', $this->controls );
    $this->set_param( 'autoplay', $this->autoplay );
    $this->set_param( 'mute', $this->muted );

    // For some reasons, height seems to not be set so we will set it manually
    $this->set_height();
  }


  /**
  * Apply a param in the Youtube iframe.
  *
  * @param  string $param_name Name of the param we want to set
  * @param  bool   $enable     Enable or disable the param
  * 
  * @see https://www.php.net/manual/en/class.domdocument.php
  * @see https://www.php.net/manual/en/class.domnodelist.php
  */
  function set_param( $param_name, $enable ) {

    // Set the value to the params
    if( $enable === true) {
      $value = '&' . $param_name . '=1';
    }
    else if( $param_name === 'controls' ) {
      $value = '&controls=0&disablekb=0&rel=0&modestbranding=1';
    }
    else {
      $value = '&' . $param_name . '=0';
    }

    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $this->output );
    $iframes = $doc->getElementsByTagName( 'iframe' );

    foreach( $iframes as $key => $iframe ) {

      // The url of the embed 
      $url = $iframe->getAttribute( 'src' );

      // Override existing settings (if already set)
      if( $enable === true && strpos($url, $param_name . '=0') !== false ) {
        $url = str_replace($param_name . '=0', $param_name . '=1', $url);
      } 
      else if( $enable !== true && strpos($url, $param_name . '=1') !== false ) {
        $url = str_replace($param_name . '=1', $param_name . '=0', $url);
      }

      $iframes[$key]->setAttribute( 'src', $url . $value );
    }

    // remove html & body tags
    $html = $doc->getElementsByTagName('html')[0];
    $body = $html->getElementsByTagName('body')[0];
    foreach($body->childNodes as $child) {
      $doc->appendChild($child);
    }
    $doc->removeChild($html);

    $this->output = $doc->saveHTML();
  }


  /**
  * Set the height on the youtube iframe
  * 
  * @see https://www.php.net/manual/en/class.domdocument.php
  * @see https://www.php.net/manual/en/class.domnodelist.php
  */
  function set_height() {

    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $this->output );
    $iframes = $doc->getElementsByTagName( 'iframe' );

    foreach( $iframes as $key => $iframe ) {
      $iframe->setAttribute( 'height', $this->height );
    }

    // remove html & body tags
    $html = $doc->getElementsByTagName('html')[0];
    $body = $html->getElementsByTagName('body')[0];
    foreach($body->childNodes as $child) {
      $doc->appendChild($child);
    }
    $doc->removeChild($html);

    $this->output = $doc->saveHTML();
  }
    
}





