<?php 

defined( 'ABSPATH' ) or die( 'Nothing to see here' ); 

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

use Elementor\Utils as elementor;

// Instance if \LLMS_Product
if( empty($plan) ) return '';

if( !isset($settings['structure_items']) ) return ''; 

// Special class for featured plans
$featured_class = $settings['is_featured_hightlight'] === 'yes' && $plan->is_featured()  
  ? 'le-slide-featured'
  : '';

/**
 * ElementorPro 2.7 and below (js library is not the same before that)
 * 
 * @see https://developers.elementor.com/elementor-2-7-moving-sliders-from-slick-to-swiper/
 */
if( version_compare( ELEMENTOR_PRO_VERSION, '2.7.0' ) >= 0 ): ?>
  <div class="swiper-slide">
    <div class="swiper-slide-bg"></div>
    <div class="swiper-slide-inner">
      <div class="swiper-slide-contents elementor-slide-content le-slide-content <?= $featured_class ?>">
<?php else: ?>
  <div class="slick-slide">
    <div class="slick-slide-bg"></div>
      <div class="slick-slide-inner">
        <div class="elementor-slide-content le-slide-content <?= $featured_class ?>">
<?php endif; ?>

        <?php foreach( $settings['structure_items'] as $key => $item ): ?>
            
          <?php switch( $item['item_type'] ) {
            
            case 'title': ?>
              <?php if( $item['is_title_link'] === 'yes' ): ?>
                <a class='le-slide-link le-slide-heading-link' href="<?= $plan->get_checkout_url() ?>">
              <?php endif; ?>
              
              <div class="elementor-slide-heading le-slide-heading">
                <<?= elementor::validate_html_tag($item['item_title_tag']) ?> class='le-slide-heading-text'>
                  <?= $plan->get( 'title' ) ?>
                </<?= elementor::validate_html_tag($item['item_title_tag']) ?>>
              </div>
              
              <?php if( $item['is_title_link'] === 'yes' ): ?>
                </a>
              <?php endif; ?>
            <?php break;

            case 'price': ?>
              <div class="elementor-slide-description le-slide-price le-slide-text">
                
                <!-- We display the informations without label inline -->
                <span class="le-slide-price-inline">

                  <!-- Price -->
                  <?php if( $item['is_regular_price'] === 'yes' && empty($item['item_price_label']) ): ?>
                    <?php if( $item['is_sale_price'] === 'yes' && lift\is_plan_on_sale( $plan->get( 'id' ) ) ): ?>
                      <span class="le-slide-price-value le-slide-old-price-value">
                        <?= lift\plan_price( $plan->get( 'id' ) ) ?>    
                      </span>
                      <span class="le-slide-sale-price-value">
                        <?= lift\plan_price_sale( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php else: ?>
                      <span class="le-slide-price-value">
                        <?= lift\plan_price( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php endif; ?>
                  <?php endif; ?>

                  <!-- Frequency -->
                  <?php if( $item['is_frequency_price'] === 'yes' && !empty(lift\plan_frequency($plan->get( 'id' ))) ): ?>
                    <?php if( empty($item['item_frequency_label']) ): ?>
                      <span class="le-slide-price-value">
                        <?= lift\plan_frequency( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php endif; ?>
                  <?php endif; ?>

                  <!-- Access expiration-->
                  <?php if( $item['is_access_expiration_price'] === 'yes' && lift\is_plan_access_expiration($plan->get( 'id' )) ): ?>
                    <?php if( empty($item['item_access_expiration_label']) ): ?>
                      <span class="le-slide-price-value">
                        <?= lift\plan_access_expiration( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php endif; ?>
                  <?php endif; ?>

                  <!-- Membership -->
                  <?php if( $item['is_membership'] === 'yes' && lift\plan_has_membership( $plan->get( 'id' ) ) ): ?>
                    <?php if( empty($item['item_membership_label']) ): ?>
                        <span class="le-slide-price-value"><?= lift\plan_membership_name( $plan->get( 'id' ) ) ?></span>
                    <?php endif; ?>
                  <?php endif; ?>
                </span>

                <!-- Infos with label -->

                <!-- Price-->
                <?php if( $item['is_regular_price'] === 'yes' && !empty($item['item_price_label']) ): ?>
                  <div class="le-slide-price-regular">
                    <span class="le-slide-price-label"><?= esc_html($item['item_price_label']) ?></span>
                    <?php if( $item['is_sale_price'] === 'yes' && lift\is_plan_on_sale( $plan->get( 'id' ) ) ): ?>
                      <span class="le-slide-price-value le-slide-old-price-value">
                        <?= lift\plan_price( $plan->get( 'id' ) ) ?>
                      </span>
                      <span class="le-slide-sale-price-value">
                        <?= lift\plan_price_sale( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php else: ?>
                      <span class="le-slide-price-value">
                        <?= lift\plan_price( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php endif; ?>
                  </div>
                <?php endif; ?>

                <!-- Frequency -->
                <?php if( $item['is_frequency_price'] === 'yes' && !empty(lift\plan_frequency($plan->get( 'id' ))) ): ?>
                  <?php if( !empty($item['item_frequency_label']) ): ?>
                    <div class="le-slide-price-frequency">
                      <span class="le-slide-price-label"><?= esc_html($item['item_frequency_label']) ?>:</span>
                      <span class="le-slide-price-value"><?= lift\plan_frequency( $plan->get( 'id' ) ) ?></span>
                    </div>
                  <?php endif; ?>
                <?php endif; ?>

                <!-- Access Expiration -->
                <?php if( $item['is_access_expiration_price'] === 'yes' && lift\is_plan_access_expiration($plan->get( 'id' )) ): ?>
                  <?php if( !empty($item['item_access_expiration_label']) ): ?>
                    <div class="le-slide-price-access">
                      <span class="le-slide-price-label"><?= esc_html($item['item_access_expiration_label']) ?>:</span>
                      <span class="le-slide-price-value"><?= lift\plan_access_expiration( $plan->get( 'id' ) ) ?></span>
                    </div>
                  <?php endif; ?>
                <?php endif; ?>

                <!-- Memebership -->
                <?php if( $item['is_membership'] === 'yes' && lift\plan_has_membership( $plan->get( 'id' ) ) ): ?>
                  <?php if( !empty($item['item_membership_label']) ): ?>
                    <div class="le-slide-price-membership">
                      <span class="le-slide-price-label"><?= esc_html($item['item_membership_label']) ?>:</span>
                      <span class="le-slide-price-value"><?= lift\plan_membership_name( $plan->get( 'id' ) ) ?></span>
                    </div>
                  <?php endif; ?>
                <?php endif; ?>
              </div>
            <?php break;
            
            case 'description': ?>
              <div class="elementor-slide-description le-slide-description le-slide-text">
                <?= $plan->get( 'content' ) ?>
              </div>
            <?php break;

            case 'enroll': ?>
              <?= utils\elementor_button([
                'url'         => $plan->get_checkout_url(),  
                'type'        => $item['item_type_button'],  
                'size'        => $item['item_size_button'],  
                'class'       => 'le-slide-enroll',  
                'text'        => $plan->get_enroll_text(),  
                'align'       => $item['item_align_button'],
                'icon'        => $item['item_icon_button'],
                'icon_align'  => $item['item_icon_align_button'],
              ]) ?>
            <?php break;

            case 'custom': ?>
              <div class="elementor-slide-description le-slide-custom le-slide-text">
                <?= esc_html($item['item_custom_text']) ?>
              </div>
            <?php break;

          } ?>

        <?php endforeach; ?> 
      </div>


<?php if( version_compare( ELEMENTOR_PRO_VERSION, '2.7.0' ) >= 0): ?>
    </div>
    <?php if( $settings['is_featured_hightlight'] === 'yes' && $plan->is_featured() ): ?>
      <div class="elementor-price-table__ribbon">
        <div class="elementor-price-table__ribbon-inner"><?= __( 'Featured', 'lifter-elements' ) ?></div>
      </div>
    <?php endif; ?>
  </div>
<?php else: ?>
    <?php if( $settings['is_featured_hightlight'] === 'yes' && $plan->is_featured() ): ?>
      <div class="elementor-price-table__ribbon">
        <div class="elementor-price-table__ribbon-inner"><?= __( 'Featured', 'lifter-elements' ) ?></div>
      </div>
    <?php endif; ?>
        </div>
    </div>
  </div>
<?php endif; ?>

