<?php 

defined( 'ABSPATH' ) or die( 'Nothing to see here' ); 

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

use Elementor\Utils as elementor;

// Instance if \LLMS_Product
if( empty($plan) ) return '';

if( !isset($settings['structure_items']) ) return ''; 

// Special class for featured plans
$featured_class = $settings['is_featured_hightlight'] === 'yes' && $plan->is_featured() ?
  'le-price-table-content-featured' : '';

// If height fit the content 
$height_class = $settings['is_equal_height'] === 'yes' ? 'le-table-price-content-height' : '';

?>
<div class="le-table-price-content-wrapper <?= $height_class ?>">
  <div class="le-table-price-content <?= $featured_class ?>">
    <div class="elementor-price-table">

      <?php foreach( $settings['structure_items'] as $key => $item ): ?>
            
        <?php switch( $item['item_type'] ) { 

          case 'title': ?>
            <?php if( $item['is_title_link'] === 'yes' ): ?>
              <a class='le-table-link le-table-heading-link' href="<?= $plan->get_checkout_url() ?>">
            <?php endif; ?>
              <div class="elementor-price-table__header le-table-heading">
                <<?= elementor::validate_html_tag($item['item_title_tag']) ?> class="elementor-price-table__heading le-table-title">
                  <?= $plan->get( 'title' ) ?>    
                </<?= elementor::validate_html_tag($item['item_title_tag']) ?>>
              </div>
             <?php if( $item['is_title_link'] === 'yes' ): ?>
              </a>
            <?php endif; ?>
          <?php break; 

          case 'description': ?>
            <div class="le-table-description">
              <span>
                <?= $plan->get( 'content' ) ?>    
              </span>
            </div>
          <?php break;

          case 'price': ?>

            <!-- If no label, price info in the header -->
            <div class="elementor-price-table__price le-table-price">  
              
              <!-- Price -->
              <?php if( $item['is_regular_price'] === 'yes' && empty($item['item_price_label']) ): ?>
                <?php if( $item['is_sale_price'] === 'yes' && lift\is_plan_on_sale( $plan->get( 'id' ) ) ): ?>
                  <div class="le-table-old-price-header le-table-old-price">
                    <span class="le-table-old-price-value">
                      <?= lift\plan_price( $plan->get( 'id' ) ) ?>    
                    </span>
                  </div>
                  <span class="elementor-price-table__currency elementor-currency--before le-table-price-currency">
                    <?= lift\currency_symbol() ?>
                  </span>
                  <span class="elementor-price-table__integer-part le-table-price-number">
                    <?= lift\plan_price_number_int($plan->get( 'id' )) ?>    
                  </span>
                <?php else: ?>
                  <span class="elementor-price-table__currency elementor-currency--before le-table-price-currency">
                    <?= lift\currency_symbol() ?>
                  </span>
                  <span class="elementor-price-table__integer-part le-table-price-number">
                    <?= lift\plan_price_number_int($plan->get( 'id' )) ?>    
                  </span>
                <?php endif; ?>
                <?php if( !empty(lift\plan_price_number_decimal($plan->get( 'id' ))) ): ?>
                  <div class="elementor-price-table__after-price">
                    <span class="elementor-price-table__fractional-part le-table-price-decimal">
                      <?= lift\plan_price_number_decimal($plan->get( 'id' )) ?>    
                    </span>
                  </div>
                <?php endif; ?>
              <?php endif; ?>

              <!-- Frequency -->
              <?php if( $item['is_frequency_price'] === 'yes' && !empty(lift\plan_frequency($plan->get( 'id' ))) ): ?>
                <?php if( empty($item['item_frequency_label']) ): ?>
                  <span class="elementor-price-table__period elementor-typo-excluded le-table-price-frequency">
                    <?= ucfirst(lift\plan_frequency($plan->get( 'id' ))) ?>
                  </span>              
                <?php endif; ?>
              <?php endif; ?>

              <!-- Access expiration -->
              <?php if( $item['is_access_expiration_price'] === 'yes' && lift\is_plan_access_expiration($plan->get( 'id' )) ): ?>
                <?php if( empty($item['item_access_expiration_label']) ): ?>
                  <span class="elementor-price-table__period elementor-typo-excluded le-table-price-access-expiration">
                    <?= lift\plan_access_expiration( $plan->get( 'id' ) ) ?>
                  </span>              
                <?php endif; ?>
              <?php endif; ?>

              <!-- Membership -->
              <?php if( $item['is_membership'] === 'yes' && lift\plan_has_membership( $plan->get( 'id' )) ): ?>
                <?php if( empty($item['item_membership_label']) ): ?>
                  <span class="elementor-price-table__period elementor-typo-excluded le-table-price-membership">
                    <?= lift\plan_membership_name( $plan->get( 'id' ) ) ?>
                  </span>              
                <?php endif; ?>
              <?php endif; ?>
            </div>

            <!-- If label, infos in the feature list -->
            <ul class="le-table-price-features-list">
              
              <!-- Price -->
              <?php if( $item['is_regular_price'] === 'yes' && !empty($item['item_price_label']) ): ?>
                <li>
                  <div class="elementor-price-table__feature-inner le-table-features-row">
                    <span class="le-table-price-label le-table-label">
                      <?= $item['item_price_label'] ?>    
                    </span>
                    <?php if( $item['is_sale_price'] === 'yes' && lift\is_plan_on_sale( $plan->get( 'id' ) ) ): ?>
                      <span class="le-table-value">
                        <span class="le-table-price-value le-table-old-price-value">
                          <?= lift\plan_price( $plan->get( 'id' ) ) ?>    
                        </span>
                        <span class="le-table-sale-price-value">
                          <?= lift\plan_price_sale( $plan->get( 'id' ) ) ?>    
                        </span>
                      </span>
                    <?php else: ?>
                      <span class="le-table-value le-table-price-value">
                        <?= lift\plan_price( $plan->get( 'id' ) ) ?>    
                      </span>
                    <?php endif; ?>
                  </div>
                </li>
              <?php endif; ?>

              <!-- Frequency -->
              <?php if( $item['is_frequency_price'] === 'yes' && !empty(lift\plan_frequency($plan->get( 'id' ))) ): ?>
                <?php if( !empty($item['item_frequency_label']) ): ?>
                  <li>
                    <div class="elementor-price-table__feature-inner le-table-features-row">
                      <span class="le-table-frequency-label le-table-label">
                        <?= esc_html($item['item_frequency_label']) ?>    
                      </span>
                      <span class="le-table-value le-table-frequency-value">
                        <?= ucfirst(lift\plan_frequency($plan->get( 'id' ))) ?>    
                      </span>
                    </div>
                  </li>
                <?php endif; ?>
              <?php endif; ?>

              <!-- Access expiration -->
              <?php if( $item['is_access_expiration_price'] === 'yes' && lift\is_plan_access_expiration($plan->get( 'id' )) ): ?>
                <?php if( !empty($item['item_access_expiration_label']) ): ?>
                  <li>
                    <div class="elementor-price-table__feature-inner le-table-features-row">
                      <span class="le-table-access-expiration-label le-table-label">
                        <?= $item['item_access_expiration_label'] ?>    
                      </span>
                      <span class="le-table-value le-table-access-expiration-value">
                        <?= lift\plan_access_expiration( $plan->get( 'id' ) ) ?>    
                      </span>
                    </div>
                  </li>
                <?php endif; ?>
              <?php endif; ?>

              <!-- Membership -->
              <?php if( $item['is_membership'] === 'yes' && lift\plan_has_membership( $plan->get( 'id' ) ) ): ?>
                <?php if( !empty($item['item_membership_label']) ): ?>
                  <li>
                    <div class="elementor-price-table__feature-inner le-table-features-row">
                      <span class="le-table-membership-label le-table-label">
                        <?= esc_html($item['item_membership_label']) ?>    
                      </span>
                      <span class="le-table-value le-table-membership-value">
                        <?= lift\plan_membership_name( $plan->get( 'id' ) ) ?>    
                      </span>
                    </div>
                  </li>
                <?php endif; ?>
              <?php endif; ?>

            </ul>
          <?php break;

          case 'enroll': ?>
            <div>
              <?= utils\elementor_button([
                'url'         => $plan->get_checkout_url(),  
                'type'        => $item['item_type_button'],  
                'size'        => $item['item_size_button'],  
                'class'       => 'le-table-enroll',  
                'text'        => $plan->get_enroll_text(),  
                'align'       => $item['item_align_button'],
                'icon'        => $item['item_icon_button'],
                'icon_align'  => $item['item_icon_align_button'],
              ]) ?>
            </div>
          <?php break;

          case 'custom': ?>
            <div class="le-table-custom le-table-text">
              <?= esc_html($item['item_custom_text']) ?>
            </div>
          <?php break;

        } ?>
      <?php endforeach; ?>
    </div>

    <?php if( $settings['is_featured_hightlight'] === 'yes' && $plan->is_featured() ): ?>
      <div class="elementor-price-table__ribbon">
        <div class="elementor-price-table__ribbon-inner"><?= __( 'Featured', 'lifter-elements' ) ?></div>
      </div>
    <?php endif; ?>
  </div>
</div>

<?php
      
