<?php

namespace Tangible\LifterElements\Utils;

defined( 'ABSPATH' ) or die();

/**
 * Return true if we are in builder mode
 */
function is_builder() {
	return \Elementor\Plugin::$instance->editor->is_edit_mode();
}

/**
 * Return true if we are in a preview mode
 */
function is_preview() {
	return \Elementor\Plugin::$instance->preview->is_preview_mode();
}

/**
 * This warning will render when we render a module in the builder
 * which may not be displayed for the current user in the final page 
 *
 * @param      string  $type   The type: permission, course_content
 *
 * @return     string  
 */
function preview_message( $type = 'permission', $post = 'post' ) {
	
	if( !is_builder() ) return '';

	switch( $type ) {

		case 'permission':
			$message = __( 'You can see this widget because you\'re in the editor. According to your current courses and progression, you may not be able to see it on real course page.', 'lifter-elements' );
			break;
		
		case 'content':
			$message = __( 'The current ' . $post . ' doesn\'t contain the content this widget is meant to display. We\'ve added in some dummy content so you can see and style this widget.', 'lifter-elements' );
      break;

		case 'need_unlogged_user':
			$message = __( 'This widget will not display for logged in users.', 'lifter-elements' );
			break;

		case 'continue_button':
			$message = __( 'The continue button may not display for all the user, according to their course progresion.', 'lifter-elements' );
			break;

		case 'progress':
			$message = __( 'The value used may not reflect your real progress.', 'lifter-elements' );
			break;
	}

	return '
		<div class="le-builder-notice">
			<i>' . $message . '</i>
		</div>
	';
}

/**
 * Get the id of the post currently edited 
 */
function get_edited_post_id() {
	
	$settings = \Elementor\Core\Settings\Manager::get_settings_managers_config();

	return isset($settings['page']['controls']['preview_id'])
    ? (int) $settings['page']['controls']['preview_id']['default']
    : 0;
}
