<?php

namespace Tangible\LifterElements\Utils;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;

/**
 * Hide an existing setting in a module
 * 
 * @param object $element 	Widget instance
 * @param string $name 		Control name
 * @param string $value 	(optional) Modify a default value
 */
function hide_control( $element, $name, $value = false ) {
	
	$control = $element->get_controls( $name );
	$control['type'] 	= 'hidden';

	if( $value !== false ) $control['default'] = $value;

	$element->update_control( $name, $control );
}


/**
 * Update a property of an existing setting
 * 
 * @param object  $element   Widget instance
 * @param string  $name      Control name
 * @param string  $key       Key
 * @param mixed   $value     Value
 */
function update_control( $element, $name, $key, $value ) {
  
  $control = $element->get_controls( $name );
  $control[$key] = $value;

  $element->update_control( $name, $control );
}


/**
 * Hide an existing setting in a module
 * 
 * @param object $element 	Widget instance
 * @param string $string 	Control name
 */
function remove_separator( $element, $name ) {
	
	$control = $element->get_controls( $name );
	$control['separator'] = '';

	$element->update_control( $name, $control );
}


/**
 * Removes a group control.
 * 
 * @see https://github.com/elementor/elementor/issues/5891
 */
function remove_group_control( $field_name, $element, $type ) {

  $controls_manager = \Elementor\Plugin::$instance->controls_manager;
  $typography_group = $controls_manager->get_control_groups( $type );

  foreach( $typography_group->get_fields() as $field_key => $field ) {
    $control_id = "{$field_name}_{$field_key}";
    $element->remove_control( $control_id );
  }

  // We can't remove the toggle field, it throw a JS error and block the editor 
  utils\hide_control( $element, $field_name . '_' . $type );
}
