<?php

namespace Tangible\LifterElements\Utils;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils; 

/**
 * Check if a the class of a given object match a namespace
 * 
 * @param object $object 	
 * 
 * @return boolean
 */
function no_render( $widget ) {

	add_action('elementor/widget/render_content', function( $content, $element ) use ( $widget ) {
		
  	if( $element->get_id() === $widget->get_id() ) return '';
  		
  	return $content;
	}, 10, 2 );
}


/**
 * Add the preview warning to an existing render
 * 
 * @param object $object 	
 */
function preview_render( $widget ) {
	
	add_action('wp', function( $content, $element ) use ( $widget ) {
		
  	if( get_class($element) === get_class($widget) ) return preview_message() . $content;
  		
  	return $content;
	}, 9, 2 );	
}


/**
 * Remplace a link in <a> element 
 *
 * @param      string  $html   The html
 * @param      string  $url    The URL
 *
 * @return     string  HTML
 */
function modify_link( $html, $url ) {
  
  $doc = new \DOMDocument();
  @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $html );

  $links = $doc->getElementsByTagName('a');
  
  if( !empty($links) ) {
    foreach( $links as $key => $link ) {
      $link->setAttribute( 'href', esc_url($url) );
    }
  } 

  return utils\get_html( $doc );
}


/**
 * Gets the html from a DOMDocument object (removed body and header)
 *
 * @param      object  $doc    DOMDocument instance
 *
 * @return     string  The html.
 */
function get_html( $doc ) {

  if( !is_object($doc) ) return;
  
  // remove html & body tags
  $html = $doc->getElementsByTagName('html')[0];
  if( empty($html) ) return '';
  
  $body = $html->getElementsByTagName('body')[0];
  foreach($body->childNodes as $child) $doc->appendChild($child);
  $doc->removeChild($html);

  // prevent encoding issue with special car
  return $doc->saveHTML($doc->documentElement);
}


/**
 * Gets the DOMNode from html
 *
 * @param      string  $html   
 *
 * @return     object  DOMDNode instance
 */
function get_node( $html ) {

  if( empty($html) ) return;

  $doc = new \DOMDocument();
  @$doc->loadHTML( $html );
  
  // remove html & body tags
  $html = $doc->getElementsByTagName('html')[0];
  $body = $html->getElementsByTagName('body')[0];

  // prevent encoding issue with special car
  return $body->childNodes[0];
}


/**
 * Change the text of an Elementor button
 *
 * @param      string  $html   The html
 * @param      string  $text   The text
 */
function change_button_text( $html, $text ) {

  $doc = new \DOMDocument();
  @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $html );

  $spans = $doc->getElementsByTagName('span');

  if( !empty($spans) ) {
    foreach( $spans as $span ) {
      $classes = $span->getAttribute( 'class' );
      if( $classes === 'elementor-button-text' || $classes === 'elementor-button-text elementor-inline-editing' ) {
        $span->nodeValue = esc_html($text);
      }
    }
  } 

  return utils\get_html( $doc );
}


/**
 * Render Elementor style button
 *
 * @param      array  $args   The arguments
 *
 * @return     string  HTML
 */
function elementor_button( $args ) {

  $default_args = array(
    'url'       => '',  
    'type'      => 'default',  
    'size'      => 'sm',  
    'class'     => '',  
    'text'      => 'Click here',  
    'align'     => 'center',
    'icon'      => '',
    'icon_align'=> 'left',
  );

  $args = array_merge( $default_args, $args );
  
  ob_start(); ?>
  <div class="elementor-element elementor-button-<?= esc_attr($args['type']) ?> <?= esc_attr($args['class']) ?> elementor-align-<?= esc_attr($args['align']) ?>">
    
    <div class="elementor-button-wrapper">
      <a class="elementor-button elementor-size-<?= esc_attr($args['size']) ?> elementor-animation-" href="<?= esc_url($args['url']) ?>" role="button">
        <span class="elementor-button-content-wrapper">

          <span class="elementor-button-icon elementor-align-icon-<?= esc_attr($args['icon_align']) ?>">
              <?php \Elementor\Icons_Manager::render_icon( $args['icon'], [ 'aria-hidden' => 'true' ] ); ?>
          </span>
          
          <span class="elementor-button-text">
            <?= esc_html($args['text']) ?>    
          </span>
        </span>
      </a>
    </div>

  </div>
  <?php $button = ob_get_clean(); 

  return $button;
}
