<?php

namespace Tangible\LifterElements\Utils;

defined( 'ABSPATH' ) or die();

/**
 * This function allow us to render a dynamic tag manually. It is usefull for rendering
 * dynamic tag in a loop, otherwise the dynamic tag will render only for the parent ID
 * 
 * @see parse_tags() in elementor/includes/controls/base-data.php
 * @see parse_dynamic_settings() in elementor/includes/base/controls-stack.php
 * 
 * @param      array  $data   The data
 */
function render_tag( array $data, string $key ) {

  // If there is no dynamic tag we return regular text
  if( !isset($data['__dynamic__'][$key]) ) return $data[$key];
    
  $content = \Elementor\Plugin::$instance->dynamic_tags->parse_tags_text( 
    $data['__dynamic__'][$key], 
    [], 
    [ 
      \Elementor\Plugin::$instance->dynamic_tags, 
      'get_tag_data_content' 
    ]
  );

  return do_shortcode( $content );
}


/**
 * /!\ THIS IS NOT WORKING /!\
 * Create a class for a dynamic tag
 *
 * @see https://developers.elementor.com/dynamic-tags/
 *
 * @param      string $slug  		Slug of the dynamic tag
 * @param      array  $params  		The parameters
 * @param      array  $category  	Category informations
 */
function register_class_tag( $slug, $params, $category ) {

	$dynamicTagClass = new class extends \Elementor\Core\DynamicTags\Tag {

		static $init_args = [];

		public $slug;
		public $params;
		public $category;

		public function __construct() {

			// Ignore initial instantiation
			if (empty(self::$init_args)) return;

			// Called by Elementor
			foreach (self::$init_args as $key => $value) {
				$this->{$key} = $value;
			}

			parent::__construct();
		}


		/**
		 * Sluf of the dynamic tag
		 *
		 * @return string
		 */
		public function get_name() {
			return $this->category['group'] . '-' . $this->slug;
		}


		/**
		 * Title of the dynamic tag
		 *
		 * @return string
		 */
		public function get_title() {
			return __( $this->category['name'], 'lifter-elements' );
		}


		/**
		 * @see https://developers.elementor.com/dynamic-tags/
		 *
		 * @return string
		 */
		public function get_group() {
			return $this->category['group'];
		}


		/**
		 * @see https://developers.elementor.com/dynamic-tags/
		 *
		 * @return array
		 */
		public function get_categories() {
			return get_type_class( $this->category['types'] );
		}


		/**
		 * @see https://developers.elementor.com/elementor-controls/
		 */
		protected function register_controls() {

		}


		/**
		 * Output result
		 *
		 * @return string
		 */
		public function render() {

			echo $this->params['output']();
		}
	};

	$dynamicTagClassName = get_class( $dynamicTagClass ); //get the name PHP assigns the anonymous class
	class_alias( $dynamicTagClassName, $params['namespace'] ); //alias the anonymous class with your class name

	// Pass arguments to class
	$dynamicTagClassName::$init_args = [
		'slug' => $slug,
		'params' => $params,
		'category' => $category
	];
}


/**
 * Gets the type class.
 *
 * @param      array  $types   Array of categories that the tag support
 *
 * @return     array
 */
function get_type_class( $types ) {

	$tags_classes = array(
		'text' 				=> \Elementor\Modules\DynamicTags\Module::TEXT_CATEGORY,
		'url' 				=> \Elementor\Modules\DynamicTags\Module::URL_CATEGORY,
		'image' 			=> \Elementor\Modules\DynamicTags\Module::IMAGE_CATEGORY,
		'media' 			=> \Elementor\Modules\DynamicTags\Module::MEDIA_CATEGORY,
		'postmeta' 			=> \Elementor\Modules\DynamicTags\Module::POST_META_CATEGORY,
		'gallery' 			=> \Elementor\Modules\DynamicTags\Module::GALLERY_CATEGORY,
	);

	$response = array();

	foreach( $types as $type ) {
		$response[] = $tags_classes[$type];
	}

	return $response;
}
