<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 * @see elementor-pro/modules/slides/widgets/slides.php
 */
class AccessPlans extends \ElementorPro\Modules\Slides\Widgets\Slides {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-access-plans';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Access Plans', 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * Add before the existing controls 
	 *
	 * @see https://code.elementor.com/php-hooks/#elementorelementbefore_section_start
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 * 
	 * @param      object  $element     The element
	 * @param      string  $section_id  The section identifier
	 * @param      array   $args        The arguments
	 */
	public static function add_controls( $element, $section_id, $args ) {
		
		if( !in_array($section_id, ['section_slides', 'section_style_slides', 'section_style_button']) ) return;

    /**
     * Settings tab
     * Controls which handle the global render settings
     */
		if( $section_id === 'section_slides'  ) {

			$element->start_controls_section(
				'section_display_settings',
				[
					'label' => __( 'Settings', 'lifter-elements' ),
					'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
				]
			);

			$element->add_control(
				'render_type', [
					'label' 	=> __( 'Display', 'lifter-elements' ),
					'type' 		=> \Elementor\Controls_Manager::SELECT,
         'default' => 'slider',
					'options'	=> [
						'slider'	=> __( 'Slider', 'lifter-elements' ),
						'table'		=> __( 'Table', 'lifter-elements' ),
					],
         'render_type' => 'template',
				]
			);

      $element->add_control(
        'plans_from', [
          'label'   => __( 'Access Plans', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
          'default' => 'course',
          'options' => [
            'both'        => __( 'From Course and Memberships', 'lifter-elements' ),
            'course'      => __( 'From Course only', 'lifter-elements' ),
            'membership'  => __( 'From Memberships only', 'lifter-elements' ),
          ],
          'render_type' => 'template',
        ]
      );

			$element->add_control(
				'hr', [
					'type' 			=> \Elementor\Controls_Manager::DIVIDER
				]
			);

			$element->add_control(
				'is_equal_height',
				[
					'label' 		   => __( 'Equal Heights?', 'lifter-elements' ),
					'type' 		     => \Elementor\Controls_Manager::SWITCHER,
					'label_on'     => __( 'On', 'lifter-elements' ),
					'label_off'    => __( 'Off', 'lifter-elements' ),
					'return_value' => 'yes',
          'render_type'  => 'template',
          'conditions'   => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => [ 'table' ],
              ],
            ]
          ]
				]
			);

			$element->add_responsive_control(
				'min_height',
				[
					'label' => __( 'Min Height', 'lifter-elements' ),
					'type' 	=> \Elementor\Controls_Manager::SLIDER,
					'range' => [
						'px' => [
							'max' 	=> 2000,
							'step' 	=> 1,
						],
						'vh' => [
							'max' 	=> 100,
							'step' 	=> 1,
						],
						'em' => [
							'max' 	=> 100,
							'step' 	=> 1,
						],
					],
					'default' => [
						'unit' => 'px',
						'size' => 200,
					],
          'selectors' => [
            '{{WRAPPER}} .le-table-price-content-height' => 'min-height: {{SIZE}}{{UNIT}};',
          ],
					'size_units'  => [ 'px', 'vh', 'em' ],
          'render_type' => 'template',
					'conditions'  => [
						'terms' => [
							[
								'name' 		 => 'is_equal_height',
								'operator' => '!in',
								'value'    => [ 'yes' ],
							],
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => [ 'table' ],
              ],
						]
					]
				]
			);

      $element->add_responsive_control(
        'slides_height_accessplan', [
          'label'       => __( 'Height', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::SLIDER,
          'render_type' => 'template',
          'size_units'  => [ 'px', 'vh', 'em' ],
          'range'       => [
            'px' => [
              'min' => 100,
              'max' => 1000,
            ],
            'vh' => [
              'min' => 10,
              'max' => 100,
            ],
          ],
          'default' => [
            'size' => 400,
          ],
          'selectors'   => [
            '{{WRAPPER}} .elementor-slides .slick-slide' => 'height: {{SIZE}}{{UNIT}};',
            '{{WRAPPER}} .elementor-slides .swiper-slide' => 'height: {{SIZE}}{{UNIT}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => [ 'slider' ],
              ],
            ]
          ]
        ]
      );

      $element->add_control(
        'table_per_row', [
          'label'   => __( 'Columns', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
         'default'  => '3',
          'options' => [
            '1'  => '1',
            '2'  => '2',
            '3'  => '3',
            '4'  => '4',
            '5'  => '5',
            '6'  => '6'
          ],
          'responsive'    => true,
          'render_type'   => 'template',
          'prefix_class'  => "le-table-columns-", // @see access-plan-table.scss
          'conditions'    => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => [ 'table' ],
              ],
            ]
          ]
        ]
      );

      $element->add_responsive_control(
        'table_spacing',
        [
          'label' => __( 'Spacing', 'lifter-elements' ),
          'type'  => \Elementor\Controls_Manager::SLIDER,
          'responsive' => true,
          'range' => [
            'px' => [
              'max'   => 2000,
              'step'  => 1,
            ],
            'vh' => [
              'max'   => 100,
              'step'  => 1,
            ],
            'em' => [
              'max'   => 100,
              'step'  => 1,
            ],
          ],
          'default' => [
            'unit' => 'px',
            'size' => 20,
          ],
          'size_units'  => [ 'px', 'vh', 'em' ],
          'render_type' => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => [ 'table' ],
              ],
            ]
          ]
        ]
      );

			$element->add_control(
				'hr_2', [
					'type' 	=> \Elementor\Controls_Manager::DIVIDER,
          'conditions'   => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => [ 'table' ],
              ],
            ]
          ]
				]
			);

			$element->add_control(
				'is_featured_hightlight',
				[
					'label' 		   => __( 'Hightlight Featured Plan?', 'lifter-elements' ),
					'type' 			   => \Elementor\Controls_Manager::SWITCHER,
					'label_on' 	   => __( 'On', 'lifter-elements' ),
					'label_off'    => __( 'Off', 'lifter-elements' ),
					'return_value' => 'yes',
          'render_type'  => 'template',
				]
			);

      $element->add_control(
        'featured_text_color',
        [
          'label'     => __( 'Banner Text Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'default'   => 'white',
          'selectors' => [
            '{{WRAPPER}} .slick-slide-inner .elementor-price-table__ribbon-inner'      => 'color: {{VALUE}};',
            '{{WRAPPER}} .swiper-slide .elementor-price-table__ribbon-inner'           => 'color: {{VALUE}};',
            '{{WRAPPER}} .le-table-price-content .elementor-price-table__ribbon-inner' => 'color: {{VALUE}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'is_featured_hightlight',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ],
        ]
      );

      $element->add_control(
        'featured_bg_color',
        [
          'label'     => __( 'Banner Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'default'   => '',
          'selectors' => [
            '{{WRAPPER}} .slick-slide-inner .elementor-price-table__ribbon-inner'      => 'background-color: {{VALUE}};',
            '{{WRAPPER}} .swiper-slide .elementor-price-table__ribbon-inner'           => 'background-color: {{VALUE}};',
            '{{WRAPPER}} .le-table-price-content .elementor-price-table__ribbon-inner' => 'background-color: {{VALUE}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'is_featured_hightlight',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ],
        ]
      );

			$element->end_controls_section();
		}
	   
    /**
     * Structure tab
     * Controls which handle the template parts
     */
    if( $section_id === 'section_style_slides'  ) {

      // Structure
			$element->start_controls_section(
				'section_structure_slider',
				[
					'label'       => __( 'Structure', 'lifter-elements' ),
					'tab' 	      => \Elementor\Controls_Manager::TAB_CONTENT,
				]
			);

			// Start Repeater
			$repeater = new \Elementor\Repeater();

			$repeater->add_control(
				'item_type', [
					'label' 	=> __( 'Part Type', 'lifter-elements' ),
					'type' 		=> \Elementor\Controls_Manager::SELECT,
					'default'	=> 'title',
					'options'	=> [
						'title'			  => __( 'Title', 'lifter-elements' ),
						'price'			  => __( 'Price', 'lifter-elements' ),
						'enroll'		  => __( 'Enroll Button', 'lifter-elements' ),
						'description'	=> __( 'Description', 'lifter-elements' ),
						'custom'		  => __( 'Custom label', 'lifter-elements' ),
					],
          'render_type' => 'template',		
				]
			);

      $repeater->add_control(
        'hr_3', [
          'type'  => \Elementor\Controls_Manager::DIVIDER,
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => '!in',
                'value'    => ['description'],
              ],
            ],
          ]
        ]
      );

      // Title
      $repeater->add_control(
        'item_title_tag', [
          'label'   => __( 'Heading tag', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
          'default' => 'title',
          'default' => 'span',
          'options' => [
            'h1'   => __( 'H1', 'lifter-elements' ),
            'h2'   => __( 'H2', 'lifter-elements' ),
            'h3'   => __( 'H3', 'lifter-elements' ),
            'h4'   => __( 'H4', 'lifter-elements' ),
            'h5'   => __( 'H5', 'lifter-elements' ),
            'h6'   => __( 'H6', 'lifter-elements' ),
            'div'  => __( 'div', 'lifter-elements' ),
            'span' => __( 'span', 'lifter-elements' ),
            'p'    => __( 'p', 'lifter-elements' ),
          ],
          'render_type' => 'template',    
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['title'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'is_title_link',
        [
          'label'        => __( 'Purchase Link', 'lifter-elements' ),
          'type'         => \Elementor\Controls_Manager::SWITCHER,
          'label_on'     => __( 'On', 'lifter-elements' ),
          'label_off'    => __( 'Off', 'lifter-elements' ),
          'return_value' => 'yes',
          'render_type'  => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['title'],
              ],
            ],
          ]
        ]
      );
      // End Title
      
      // Price
      $repeater->add_control(
        'is_regular_price',
        [
          'label'        => __( 'Show Regular Price?', 'lifter-elements' ),
          'type'         => \Elementor\Controls_Manager::SWITCHER,
          'label_on'     => __( 'On', 'lifter-elements' ),
          'label_off'    => __( 'Off', 'lifter-elements' ),
          'return_value' => 'yes',
          'default'      => 'yes',
          'render_type'  => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'is_sale_price',
        [
          'label'        => __( 'Show Sale Price?', 'lifter-elements' ),
          'type'         => \Elementor\Controls_Manager::SWITCHER,
          'label_on'     => __( 'On', 'lifter-elements' ),
          'label_off'    => __( 'Off', 'lifter-elements' ),
          'return_value' => 'yes',
          'render_type'  => 'template',
          'default'      => 'yes',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
              [
                'name'     => 'is_regular_price',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_price_label',
        [
          'label'       => __( 'Price Label', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::TEXT,
          'render_type' => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
              [
                'name'     => 'is_regular_price',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'is_frequency_price',
        [
          'label'        => __( 'Show Frequency?', 'lifter-elements' ),
          'type'         => \Elementor\Controls_Manager::SWITCHER,
          'label_on'     => __( 'On', 'lifter-elements' ),
          'label_off'    => __( 'Off', 'lifter-elements' ),
          'return_value' => 'yes',
          'render_type'  => 'template',
          'default'      => 'yes',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_frequency_label',
        [
          'label'       => __( 'Frequency Label', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::TEXT,
          'render_type' => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
              [
                'name'     => 'is_frequency_price',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'is_access_expiration_price',
        [
          'label'        => __( 'Show Access Expiration?', 'lifter-elements' ),
          'type'         => \Elementor\Controls_Manager::SWITCHER,
          'label_on'     => __( 'On', 'lifter-elements' ),
          'label_off'    => __( 'Off', 'lifter-elements' ),
          'return_value' => 'yes',
          'default'      => 'yes',
          'render_type'  => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_access_expiration_label',
        [
          'label'       => __( 'Access Expiration Label', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::TEXT,
          'render_type' => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
              [
                'name'     => 'is_access_expiration_price',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'is_membership',
        [
          'label'        => __( 'Show Membership?', 'lifter-elements' ),
          'type'         => \Elementor\Controls_Manager::SWITCHER,
          'label_on'     => __( 'On', 'lifter-elements' ),
          'label_off'    => __( 'Off', 'lifter-elements' ),
          'return_value' => 'yes',
          'default'      => 'yes',
          'render_type'  => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_membership_label',
        [
          'label'       => __( 'Membership Label', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::TEXT,
          'render_type' => 'template',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['price'],
              ],
              [
                'name'     => 'is_membership',
                'operator' => 'in',
                'value'    => ['yes'],
              ],
            ],
          ]
        ]
      );
      // End Price

      // Button
      $repeater->add_control(
        'item_type_button', [
          'label'   => __( 'Type', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
          'default' => 'default',
          'options' => [
            'default' => __( 'Default', 'lifter-elements' ),
            'info'    => __( 'Info', 'lifter-elements' ),
            'success' => __( 'Success', 'lifter-elements' ),
            'warning' => __( 'Warning', 'lifter-elements' ),
            'danger'  => __( 'Danger', 'lifter-elements' ),
          ],
          'render_type' => 'template',    
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['enroll'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_responsive_control(
        'item_align_button',
        [
          'label' => __( 'Alignment', 'lifter-elements' ),
          'type'  => \Elementor\Controls_Manager::CHOOSE,
          'options' => [
            'left'  => [
              'title' => __( 'Left', 'lifter-elements' ),
              'icon'  => 'eicon-text-align-left',
            ],
            'center' => [
              'title' => __( 'Center', 'lifter-elements' ),
              'icon'  => 'eicon-text-align-center',
            ],
            'right' => [
              'title' => __( 'Right', 'lifter-elements' ),
              'icon'  => 'eicon-text-align-right',
            ],
            'justify' => [
              'title' => __( 'Justified', 'lifter-elements' ),
              'icon'  => 'eicon-text-align-justify',
            ],
          ],
          'default'     => '',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['enroll'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_size_button', [
          'label'   => __( 'Size', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
          'default' => 'sm',
          'options' => [
            'xs'  => __( 'Extra Small', 'lifter-elements' ),
            'sm'  => __( 'Small', 'lifter-elements' ),
            'md'  => __( 'Medium', 'lifter-elements' ),
            'lg'  => __( 'Large', 'lifter-elements' ),
            'xl'  => __( 'Extra Large', 'lifter-elements' ),
          ],
          'render_type' => 'template',    
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['enroll'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_icon_button',
        [
          'label'       => __( 'Icon', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::ICONS,
          'label_block' => true,
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['enroll'],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_icon_align_button',
        [
          'label'   => __( 'Icon Position', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
          'default' => 'left',
          'options' => [
            'left'  => __( 'Before', 'lifter-elements' ),
            'right' => __( 'After', 'lifter-elements' ),
          ],
          'conditions'  => [
            'terms'   => [
              [
                'name'     => 'item_icon_button[value]',
                'operator' => '!in',
                'value'    => [''],
              ],
            ],
          ]
        ]
      );

      $repeater->add_control(
        'item_icon_indent_button',
        [
          'label' => __( 'Icon Spacing', 'lifter-elements' ),
          'type'  => \Elementor\Controls_Manager::SLIDER,
          'range' => [
            'px'  => [
            'max' => 50,
            ],
          ],
          'selectors' => [
            '{{WRAPPER}} .le-slide-enroll .elementor-button .elementor-align-icon-right' => 'margin-left: {{SIZE}}{{UNIT}};',
            '{{WRAPPER}} .le-table-enroll .elementor-button .elementor-align-icon-right' => 'margin-left: {{SIZE}}{{UNIT}};',
            '{{WRAPPER}} .le-slide-enroll .elementor-button .elementor-align-icon-left' => 'margin-right: {{SIZE}}{{UNIT}};',
            '{{WRAPPER}} .le-table-enroll .elementor-button .elementor-align-icon-left' => 'margin-right: {{SIZE}}{{UNIT}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['enroll'],
              ],
            ],
          ]
        ]
      );
      // End Button

			// Custom
      $repeater->add_control(
        'item_custom_admin_label',
        [
          'label'       => __( 'Label (admin)', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::TEXT,
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],   
          'render_type' => 'template',
        ]
      );

			$repeater->add_control(
				'item_custom_text',
				[
					'label' 	    => __( 'Custom Content', 'lifter-elements' ),
					'type' 		    => \Elementor\Controls_Manager::TEXT,
          'dynamic'     => [ 'active' => true ],
					'conditions'  => [
						'terms' => [
							[
								'name' 		 => 'item_type',
								'operator' => 'in',
								'value' 	 => ['custom'],
							],
						],
					],
          'render_type' => 'template',
				]
			);

      $repeater->add_group_control(
        \Elementor\Group_Control_Typography::get_type(),
        [
          'name'      => 'item_custom_typography',
          'scheme'    => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_4,
          'selector'  => '{{WRAPPER}} .le-slide-custom, {{WRAPPER}} .le-table-custom',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],
        ]
      );

      $repeater->add_control(
      'item_custom_text_color',
        [
          'label'     => __( 'Text Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'default'   => '',
          'selectors' => [
            '{{WRAPPER}} .le-slide-custom' => 'fill: {{VALUE}}; color: {{VALUE}};',
            '{{WRAPPER}} .le-table-custom' => 'fill: {{VALUE}}; color: {{VALUE}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],
        ]
      );

      $repeater->add_control(
        'item_custom_text_background_color',
        [
          'label'     => __( 'Background Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'selectors' => [
            '{{WRAPPER}} .le-slide-custom' => 'background-color: {{VALUE}};',
            '{{WRAPPER}} .le-table-custom' => 'background-color: {{VALUE}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],
        ]
      );

      $repeater->add_group_control(
        \Elementor\Group_Control_Border::get_type(),
        [
          'name'      => 'item_custom_border',
          'selector'  => '{{WRAPPER}} .le-slide-custom, {{WRAPPER}} .le-table-custom',
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],
        ]
      );

      $repeater->add_control(
        'item_custom_border_radius',
        [
          'label'      => __( 'Border Radius', 'lifter-elements' ),
          'type'       => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%' ],
          'selectors'  => [
            '{{WRAPPER}} .le-slide-custom' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            '{{WRAPPER}} .le-table-custom' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],
        ]
      );

      $repeater->add_responsive_control(
        'item_custom_text_padding',
        [
          'label'       => __( 'Padding', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units'  => [ 'px', 'em', '%' ],
          'selectors'   => [
            '{{WRAPPER}} .le-slide-custom' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            '{{WRAPPER}} .le-table-custom' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'item_type',
                'operator' => 'in',
                'value'    => ['custom'],
              ],
            ],
          ],
        ]
      );
			// End Custom

			// End Repeater
			
			$element->add_control(
				'structure_items',
				[
					'label'    => __( 'Parts', 'plugin-domain' ),
					'type'     => \Elementor\Controls_Manager::REPEATER,
					'fields'   => $repeater->get_controls(),
					'default'  => [
						['item_type' => 'title'],
						['item_type' => 'price'],
						['item_type' => 'enroll'],
						['item_type' => 'description'],
						['item_type' => 'custom'],
					],
					'title_field' => '{{{ item_type }}}',
          'render_type' => 'template',
				]
			);

      $element->end_controls_section();
		}      

    
    /**
     * Table style tab and Typography tab 
     */
    if( $section_id === 'section_style_button'  ) {
      
      /**
       * Table style tab
       */
      $element->start_controls_section(
        'section_style_price_table',
        [
          'label' => __( 'Table', 'lifter-elements' ),
          'tab'   => \Elementor\Controls_Manager::TAB_STYLE,
        ]
      );

      $element->add_control(
        "price_table_text_color",
        [
          'label'     => __( "Background Color", 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'selectors' => ["{{WRAPPER}} .le-table-price-content" => 'background-color: {{VALUE}};'],
          'conditions' => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['table'],
              ],
            ],
          ],
        ]
      );

      $element->add_group_control(
        \Elementor\Group_Control_Border::get_type(),
        [
          'name'       => "price_table_border",
          'selector'   => '{{WRAPPER}} .le-table-price-content',
          'conditions' => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['table'],
              ],
            ],
          ],
        ]
      );

      $element->add_control(
        "border_radius_price_table",
        [
          'label'      => __( 'Border Radius', 'lifter-elements' ),
          'type'       => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%' ],
          'selectors'  => [
            '{{WRAPPER}} .le-table-price-content' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
          'conditions' => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['table'],
              ],
            ],
          ],
        ]
      );

      $element->add_group_control(
        \Elementor\Group_Control_Box_Shadow::get_type(),
        [
          'name'        => 'box_shadow_price_table',
          "label"       => __( "Box Shadow", 'lifter-elements' ),
          'selector'    => '{{WRAPPER}} .le-table-price-content',
          'conditions' => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['table'],
              ],
            ],
          ],
        ]
      );

      $element->add_responsive_control(
        "padding_price_table",
        [
          'label'       => __( 'Padding', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units'  => [ 'px', 'em', '%' ],
          'selectors'   => [
            '{{WRAPPER}} .le-table-price-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
          'conditions' => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['table'],
              ],
            ],
          ],
        ]
      );

			$element->end_controls_section();

      /**
       * Typography tab
       * Controls which handle the typography style (heading, desc, price)
       */
      $element->start_controls_section(
        'section_style_typo',
        [
          'label' => __( 'Typography', 'lifter-elements' ),
          'tab'   => \Elementor\Controls_Manager::TAB_STYLE,
        ]
      );

      // Same controls for heading, description and price
      foreach( ['heading', 'description', 'price'] as $section ) {

        $label = ucfirst($section);
        
        $element->add_control(
          "{$section}_text_color",
          [
            'label'     => __( "{$label} Color", 'lifter-elements' ),
            'type'      => \Elementor\Controls_Manager::COLOR,
            'default'   => '',
            'separator' => 'before',
            'selectors' => [
              "{{WRAPPER}} .le-slide-{$section}" => 'color: {{VALUE}};',
              "{{WRAPPER}} .le-table-{$section}" => 'color: {{VALUE}};',
              "{{WRAPPER}} .le-table-{$section} .le-table-title" => 'color: {{VALUE}};',
            ],
          ]
        );

        if($section === 'heading') {
          $selector = "{{WRAPPER}} .le-slide-{$section}-text, {{WRAPPER}} .le-table-{$section}";
        }
        else{
          $selector = "{{WRAPPER}} .le-slide-{$section}, {{WRAPPER}} .le-table-{$section}";
        }

        $element->add_group_control(
          \Elementor\Group_Control_Typography::get_type(),
          [
            'name'        => "{$section}_text_typography",
            "label"       => __( "{$label} Typography", 'lifter-elements' ),
            'selector'    => $selector,
          ]
        );

        $element->add_control(
          "{$section}_text_align_slider",
          [
            'label'   => __( 'Text Alignment', 'lifter-elements' ),
            'type'    => \Elementor\Controls_Manager::CHOOSE,
            'options'   => [
              'left'    => [
                'title'   => __( 'Left', 'lifter-elements' ),
                'icon'    => 'fa fa-align-left',
              ],
              'center'  => [
                'title'   => __( 'Center', 'lifter-elements' ),
                'icon'    => 'fa fa-align-center',
              ],
              'right'   => [
                'title'   => __( 'Right', 'lifter-elements' ),
                'icon'    => 'fa fa-align-right',
              ],
            ],
            'default'       => '',
            'prefix_class'  => "le-slider-{$section}-align-",
            'toggle'        => true,
            'render_type'   => 'template',
            'conditions'    => [
              'terms' => [
                [
                  'name'     => 'render_type',
                  'operator' => 'in',
                  'value'    => ['slider'],
                ],
              ],
            ]
          ]
        );
        
        $element->add_control(
          "{$section}_text_align_table",
          [
            'label'   => __( 'Text Alignment', 'lifter-elements' ),
            'type'    => \Elementor\Controls_Manager::CHOOSE,
            'options'   => [
              'left'    => [
                'title'   => __( 'Left', 'lifter-elements' ),
                'icon'    => 'fa fa-align-left',
              ],
              'center'  => [
                'title'   => __( 'Center', 'lifter-elements' ),
                'icon'    => 'fa fa-align-center',
              ],
              'right'   => [
                'title'   => __( 'Right', 'lifter-elements' ),
                'icon'    => 'fa fa-align-right',
              ],
            ],
            'default'       => '',
            'prefix_class'  => "le-table-{$section}-align-",
            'toggle'        => true,
            'render_type'   => 'template',
            'conditions'    => [
              'terms' => [
                [
                  'name'     => 'render_type',
                  'operator' => 'in',
                  'value'    => ['table'],
                ],
              ],
            ]
          ]
        );

        // Controls specific for the heading in the table render
        if( $section === 'heading' ) {

          $conditions = [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['table'],
              ],
            ],
          ];

          $element->add_control(
            "{$section}_background_color", 
            [
              'label'     => __( 'Background Color', 'lifter-elements' ),
              'type'      => \Elementor\Controls_Manager::COLOR,
              'selectors' => [
                '{{WRAPPER}} .le-table-heading' => 'background-color: {{VALUE}};',
              ],
              'conditions' => $conditions
            ]
          );

          $element->add_group_control(
            \Elementor\Group_Control_Border::get_type(),
            [
              'name'       => "{$section}__border",
              'selector'   => '{{WRAPPER}} .le-table-heading',
              'conditions' => $conditions
            ]
          );

          $element->add_control(
            "{$section}_border_radius",
            [
              'label'      => __( 'Border Radius', 'lifter-elements' ),
              'type'       => \Elementor\Controls_Manager::DIMENSIONS,
              'size_units' => [ 'px', '%' ],
              'selectors'  => [
                '{{WRAPPER}} .le-table-heading' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
              ],
              'conditions' => $conditions
            ]
          );

          $element->add_responsive_control(
            "{$section}_padding",
            [
              'label'       => __( 'Padding', 'lifter-elements' ),
              'type'        => \Elementor\Controls_Manager::DIMENSIONS,
              'size_units'  => [ 'px', 'em', '%' ],
              'selectors'   => [
                '{{WRAPPER}} .le-table-heading' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
              ],
              'conditions' => $conditions
            ]
          );

        }

      } 
      // End foreach similar sections

      $element->end_controls_section();
    }

	}


	/**
	 * Modify control defined in the parent class
	 * 
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 * 
	 * @param      obj     $element     The element
	 * @param      string  $section_id  The section identifier
	 * @param      array   $args        The arguments
	 */
	public static function modify_controls( $element, $section_id, $args ) {

    if( !in_array($section_id, [
      'section_slides', 
      'section_style_title', 
      'section_style_description',
      'section_style_slides',
      'section_style_button',
      'section_slider_options',
      'section_style_navigation'
    ])) return;
 
    /**
     * Hide slider repeater (content)
     * Hide description/title tab, we will use one global tab called typography (style)
     */
		if( in_array($section_id, ['section_slides', 'section_style_title', 'section_style_description']) ) {

			$control = $element->get_controls( $section_id );
			$control['conditions'] = ['terms' => [
				[
					'name' 		 => 'render_type',
					'operator' => 'in',
					'value' 	 => ['never'],
				],
			]];
			$element->update_control( $section_id, $control );
		}

    /**
     * Slider tabs
     * Hide if table render 
     */
    if( in_array($section_id, ['section_style_slides', 'section_slider_options', 'section_style_navigation']) ) {

      $condition = ['terms' => [
        [
          'name'      => 'render_type',
          'operator'  => 'in',
          'value'     => ['slider'],
        ],
      ]];
      utils\update_control( $element, $section_id, 'conditions', $condition );
    }

    /**
     * Add global background color to the existing slides style tab
     */
    if( $section_id === 'section_style_slides' ) {

      // Change the label
      utils\update_control( $element, 'content_max_width', 'label', __( 'Content Max Width', 'lifter-elements') );

      $element->add_control(
        'slider_background_color',
        [
          'label'     => __( 'Background Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'default'   => '#FF9A00',
          'selectors' => [
            '{{WRAPPER}} .slick-slide-inner'=> 'background-color: {{VALUE}};',
            '{{WRAPPER}} .swiper-slide-bg'  => 'background-color: {{VALUE}};',
          ],
          'conditions'  => [
            'terms' => [
              [
                'name'     => 'render_type',
                'operator' => 'in',
                'value'    => ['slider'],
              ],
            ],
          ],
        ]
      );

    }


    /**
     * Modify the existing button tab 
     */
    if( $section_id === 'section_style_button' ) {

      $element->remove_control( 'button_size' );
      $element->remove_control( 'button_color' );
      $element->remove_control( 'button_border_radius' );
      $element->remove_control( 'button_border_width' );
  
      utils\remove_group_control( 'button_typography', $element, 'typography' );

      // Change the selector
      $selector = "{{WRAPPER}} .le-slide-enroll .elementor-button, {{WRAPPER}} .le-table-enroll .elementor-button";
      
      utils\update_control( 
        $element, 
        'button_text_color', 
        'selectors', 
        ["{$selector}" => 'color: {{VALUE}};'] 
      );

      utils\update_control( 
        $element, 
        'button_background_color', 
        'selectors', 
        ["{$selector}" => 'background-color: {{VALUE}};'] 
      );

      utils\update_control( 
        $element, 
        'button_border_color', 
        'selectors', 
        ["{$selector}" => 'border-color: {{VALUE}};'] 
      );
      
      $selector = "{{WRAPPER}} .le-slide-enroll .elementor-button:hover, {{WRAPPER}} .le-table-enroll .elementor-button:hover";

      utils\update_control( 
        $element, 
        'button_hover_text_color', 
        'selectors', 
        ["{$selector}" => 'color: {{VALUE}};']
      );

      utils\update_control( 
        $element, 
        'button_hover_background_color', 
        'selectors', 
        ["{$selector}" => 'background-color: {{VALUE}};'] 
      );

      utils\update_control( 
        $element, 
        'button_hover_border_color', 
        'selectors', 
        ["{$selector}" => 'border-color: {{VALUE}};'] 
      );


      $element->add_control(
        'hr_4', ['type' => \Elementor\Controls_Manager::DIVIDER]
      );

      $element->add_group_control(
        \Elementor\Group_Control_Typography::get_type(),
        [
          'name'        => "enroll_text_typography",
          "label"       => __( "Typography", 'lifter-elements' ),
          'selector'    => "{{WRAPPER}} .le-slide-enroll .elementor-button-text, {{WRAPPER}} .le-table-enroll .elementor-button-text",
          'separator'   => "before",
        ]
      );

      $element->add_group_control(
        \Elementor\Group_Control_Text_Shadow::get_type(),
        [
          'name'        => 'enroll_text_shadow',
          "label"       => __( "Text Shadow", 'lifter-elements' ),
          'selector'    => '{{WRAPPER}} .le-slide-enroll, {{WRAPPER}} .le-table-enroll',
        ]
      );

      $element->add_control(
        'hr_5', [
          'type'  => \Elementor\Controls_Manager::DIVIDER
        ]
      );

      $element->add_group_control(
        \Elementor\Group_Control_Border::get_type(),
        [
          'name'        => 'enroll_group_border',
          'label'       => __( 'Border Style', 'lifter-elements' ),
          'selector'    => '{{WRAPPER}} .le-slide-enroll .elementor-button, {{WRAPPER}} .le-table-enroll .elementor-button',
          'exclude'     => [ 'color' ],
        ]
      );

     $element->add_control(
        'enroll_border_radius',
        [
          'label'      => __( 'Border Radius', 'lifter-elements' ),
          'type'       => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units' => [ 'px', '%' ],
          'selectors'  => [
            '{{WRAPPER}} .le-slide-enroll .elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            '{{WRAPPER}} .le-table-enroll .elementor-button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
        ]
      );

      $element->add_group_control(
        \Elementor\Group_Control_Box_Shadow::get_type(),
        [
          'name'        => 'enroll_box_shadow',
          "label"       => __( "Box Shadow", 'lifter-elements' ),
          'selector'    => '{{WRAPPER}} .le-slide-enroll .elementor-button, {{WRAPPER}} .le-table-enroll .elementor-button',
        ]
      );

      $element->add_control(
        'hr_6', [
          'type'  => \Elementor\Controls_Manager::DIVIDER
        ]
      );

      $element->add_responsive_control(
        'enroll_padding',
        [
          'label'       => __( 'Padding', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units'  => [ 'px', 'em', '%' ],
          'selectors'   => [
            '{{WRAPPER}} .le-slide-enroll .elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
            '{{WRAPPER}} .le-table-enroll .elementor-button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
          ],
        ]
      );
    }
    // End button tab

	}


	/**
	 * HTML output
	 */
	protected function render() {

		$settings = $this->get_settings_for_display();

    $plans = [];

    // Get access plans related to the current course
    if( in_array( $settings['plans_from'], ['both', 'course'] ) ) {
      $plans = lift\course_plans();
    }
    
    // Get access plans from membership related to the current course
    if( in_array( $settings['plans_from'], ['both', 'membership'] ) ) {
      $plans = array_merge( $plans, lift\course_membership_plans() );
    }

    // Attempt to get from current membership (as a fallback in case we are not on a course page)
    if( empty($plans) && in_array( $settings['plans_from'], ['both', 'membership'] ) ) {
      $plans = lift\membership_plans();
    }

    // Choose the render method
    if( $settings['render_type'] === 'slider' ) {
      $this->render_slider( $settings, $plans );
    }
		else if( $settings['render_type'] === 'table' ) {
			$this->render_table( $settings, $plans );
    }
	}


  /**
   * HTML output for the tables render 
   *
   * @param      array    $settings  The settings
   * @param      array    $plans     Array of LLMS_Access_Plan instance 
   */
  protected function render_table( $settings, $plans ) {
    
    $tables = [];

    // Generate the HTLML of the tables
    if( !empty($plans) ) {
      foreach( $plans as $plan ) {
        ob_start();
        include $template = LifterElements_DIR . 'includes/templates/widgets/access-plan-table.php';
        $tables[] = ob_get_clean();        
      }
    }

    if( empty($tables) ) { echo ' '; return; }

    echo $this->get_table_style( $settings );
    echo '<div class="le-price-table-wrapper">';
    foreach( $tables as $table ) {
      echo $table;
    }   
    echo '</div>';
  }


  /**
   * Gets the table style.
   *
   * @param      array  $settings  The settings
   * 
   * @return    string  CSS
   */
  protected function get_table_style( $settings ) {
      
    $wrapper  = ".elementor-element-" . $this->get_id();
    $table_per_row = (int) $settings['table_per_row'];

    ob_start(); ?>
    <!-- Generated CSS -->
    <style>

      <?= $wrapper ?>.le-table-columns-<?= $table_per_row ?> .le-table-price-content-wrapper {
        width: calc(<?= 100 / $table_per_row ?>% - <?= $settings['table_spacing']['size']/2 ?><?= $settings['table_spacing']['unit'] ?>);
      }
    
    </style>
    <?php $css = ob_get_clean();

    return $css;
  }


  /**
   * HTML output for the slider render 
   *
   * @param      array    $settings  The settings
   * @param      array    $plans     Array of LLMS_Access_Plan instance
   * 
   * @see render() in elementor-pro/modules/slides/widgets/slides.php 
   */
  protected function render_slider( $settings, $plans ) {

    $slides_html = [];
    
    if( utils\is_builder() ) $this->builder_slider();
    
    // Generate the HTLML of the slides

    if( ! empty($plans) ) {
      foreach( $plans as $plan ) {
        ob_start();
        include $template = LifterElements_DIR . 'includes/templates/widgets/access-plan-slide.php';
        $slides_html[] = ob_get_clean();        
      }
    }

    if( empty($slides_html) ) { echo ' '; return; }

    // Get the slider from the parent class render method
    ob_start();
    parent::render();
    $parent_render = ob_get_clean();

    // Remplace the slides from the parents by our slides
    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $parent_render );
    
    $divs = $doc->getElementsByTagName('div');

    foreach( $divs as $div ) {

      $classes = $div->getAttribute( 'class' );

      if( strpos($classes, 'elementor-slides ') === false 
       && strpos($classes, 'swiper-wrapper elementor-slides') === false 
      ) continue;
      
      // Remove the initial slides

      while( $div->hasChildNodes() ) $div->removeChild( $div->childNodes->item(0) );

      if( empty($slides_html) ) continue;

      // Remplace by our slides

      foreach( $slides_html as $slide ) {
        $div->appendChild( 
          $doc->importNode( utils\get_node($slide), true ) 
        );
      }
    }

    echo utils\get_html( $doc );
  }


  /**
   * Actions specific to the builder mode
   */
  protected function builder_slider() {

    // We don't need the fix for the legacy mode
    if( version_compare( ELEMENTOR_PRO_VERSION, '2.7.0' ) === -1 ) return;

    /**
     * The configuration of the slider is saved in an html attribute (data-settings)
     * The data-settings are create by Elementor. It gets all the controls with the
     * 'frontend_available' parameter set and adds them all into the attibute using
     * the JSON format
     * 
     * For some reasons that I don't understand, this attribute is not rendered into 
     * the editor, so we will manually add it.
     * 
     * @see elementor-pro/modules/slides/widgets/slides.php
     * @see elementor/includes/base/element-base.php
     */
    $slider_config = $this->get_frontend_settings();
    
    ?><script>
      jQuery(function() {
        const widget = document.getElementsByClassName("elementor-element-<?= $this->get_id() ?>")[0] 
        widget.setAttribute('data-settings', '<?= wp_json_encode($slider_config) ?>')
      })
    </script><?php
  }


  /**
   * Seems that style dependencies are not printed correctly if we extends existing widget?
   */
	public function get_widget_css_config( $widget_name ) {
    return parent::get_widget_css_config(
      parent::get_name()
    ); 
	}

  
  protected function content_template() {}

}


