<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Audio extends \Elementor\Widget_Base {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-audio';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Audio', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-headphones";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {
		
		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Dimensions', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'important_note',
			[
				'type'	=> \Elementor\Controls_Manager::RAW_HTML,
				'raw' 	=> '<i>' . __( 'This widget is currently only fully compatible with self hosted link.', 'lifter-elements' ) . '</i>',
			]
		);

		$this->add_control(
			'is_auto_size',
			[
				'label' 		=> __( 'Dimension auto', 'lifter-elements' ),
				'type' 			=> \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->add_control(
			'width', [
				'label' 		=> __( 'Width', 'lifter-elements' ),
				'type' 			=> \Elementor\Controls_Manager::NUMBER,
				'selectors' => [
					'{{WRAPPER}} audio.le-course-audio' => 'width: {{VALUE}}px;',
				],
				'conditions' 	=> [
					'terms' 	=> [
						[
							'name' 		=> 'is_auto_size',
							'operator' 	=> '!in',
							'value' 	=> [
								'yes'
							],
						],
					]
				],
				'default' => 640,
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'content_behavior',
			[
				'label' => __( 'Behavior', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'is_muted',
			[
				'label' 		=> __( 'Mute audio', 'lifter-elements' ),
				'type' 			=> \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->add_control(
			'is_autoplay',
			[
				'label' 		=> __( 'Enable autoplay', 'lifter-elements' ),
				'type' 			=> \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->end_controls_section();
	}


	/**
	 * Front-end render
	 *
	 * @see  lifterlms/templates/course/syllabus.php
	 * 
	 * @return string
	 */	
	protected function render() {
		
		$audio_url = lift\course_audio_url();

		$this->render_audio( $audio_url );
	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}

	
	/**
	 * Render the audio (separate method for giving access to class which extends this one)
	 *
	 * @param      url  $audio_url  The audio url
	 */
	protected function render_audio( $audio_url ) {

		$settings = $this->get_settings_for_display();

		$is_auto_size	= $settings['is_auto_size'] === 'yes';
		$is_controls	= true;
		$is_muted		  = $settings['is_muted'] === 'yes';
		$is_autoplay	= $settings['is_autoplay'] === 'yes';

		$width = $is_auto_size ? '100%' : intval($settings['width']) . 'px'; 	
		
		$audio = wp_oembed_get( $audio_url, ['width' => $settings['width']] );

		// Embed video
		if( !empty($audio) ) {
      echo $audio;
      return;
    }
			
		// Local video
		if( !empty($audio_url) ): ?>

			<audio <?= $is_controls ? 'controls' : ''; ?> <?= $is_autoplay ? 'autoplay' : ''; ?> <?= $is_muted ? 'muted' : ''; ?> class="<?= $this->get_name() ?>">
			  	<source src="<?= $audio_url ?>">
			  	<p>Your browser doesn't support HTML5 audio. Here is a <a href="<?= $audio_url ?>">link to the audio</a> instead.</p>
			</audio> 
      
		<?php elseif( empty($audio_url) && utils\is_builder() ): ?>

			<?= utils\preview_message( 'content', 'course' ); ?>
			<audio <?= $is_controls ? 'controls' : ''; ?> <?= $is_autoplay ? 'autoplay' : ''; ?> <?= $is_muted ? 'muted' : ''; ?> class="<?= $this->get_name() ?>">
			  	<source src="<?= LifterElements_URL . 'assets/media/sample/sample.mp3' ?>">
			  	<p>Your browser doesn't support HTML5 audio. Here is a <a href="<?= $audio_url ?>">link to the audio</a> instead.</p>
			</audio>

		<?php endif;
	}


}


