<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

use Elementor\Utils as elementor;

/**
 * We will just extends the default Elementor Pro Author_Box widgets
 * and remplace the source fields value by the current Author(s) 
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 * @see /elementor-pro/modules/theme-elements/widgets/author-box.php
 */
class Author extends \ElementorPro\Modules\ThemeElements\Widgets\Author_Box {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-author';
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Author', 'lifter-elements' );
	}


	/**
	 * If we have a parent class defined in the widgets.json, the plugin
	 * will run this function if it exist
	 * 
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 */
	public static function modify_controls( $element, $section_id, $args ) {
		
		if( $section_id !== 'section_author_info' ) return;
		
		// Hide field for custom avatar and source
		utils\hide_control( $element, 'source' );
		utils\hide_control( $element, 'author_avatar' );
		utils\hide_control( $element, 'author_name' );

		// Remove separator from the first field
		utils\remove_separator( $element, 'show_avatar' );

    $element->add_control(
      'show_instructor_label',
      [
        'separator'    => 'before',
        'label'        => __( 'Display Label', 'lifter-elements' ),
        'type'         => \Elementor\Controls_Manager::SWITCHER,
        'label_on'     => __( 'Show', 'lifter-elements' ),
        'label_off'    => __( 'Hide', 'lifter-elements' ),
        'return_value' => 'yes',
        'default'      => 'yes',
      ]
    );

  }


  /**
   * Render 
   */
  protected function render() {

    $course_id  = lift\course_id();
    $instuctors = lift\course_instuctors( $course_id );

    if( empty($instuctors) ) return; ?>
    
    <div class="le-instructor-box">
      <?php foreach( $instuctors as $instuctor ): ?>
        <?php $this->render_instuctor( $instuctor ); ?>
      <?php endforeach; ?>
    </div><?php    
  }
	

  /**
	 * Front-end render
	 * Modified version of the render method of the author-box widget
	 * 
	 * @see  lifterlms/includes/llms.template.functions.php
	 * @see  elementor-pro/modules/theme-elements/widgets/author-box.php
	 * 
   * @param array $instuctor
   * 
	 * @return string
	 */
	protected function render_instuctor( $instuctor ) {

    // Reset the value (We will keep the render_attribute of the previous instructor otherwise)
    $this->reset_render_attribute();

		$settings         = $this->get_active_settings();
		$author           = [];
		$link_tag         = 'div';
		$link_url         = '';
		$link_target      = '';
		$author_name_tag  = elementor::validate_html_tag($settings['author_name_tag']);

		$avatar_args['size'] = 300;
		
		$author['avatar'] 		  = utils\get_avatar_url( (int) $instuctor['id'], $avatar_args );
    $author['display_name'] = $instuctor['name'];
		$author['website'] 		  = \get_the_author_meta( 'user_url', $instuctor['id'] );
		$author['bio'] 			    = \get_the_author_meta( 'description', $instuctor['id'] );
		$author['posts_url'] 	  = \get_author_posts_url( $instuctor['id'] );

		$print_avatar = $settings['show_avatar'] === 'yes';
		$print_name 	= $settings['show_name'] === 'yes';
    $print_bio    = $settings['show_biography'] === 'yes';
		$print_label  = $settings['show_instructor_label'] === 'yes';
		$print_link 	= $settings['show_link'] === 'yes' && !empty($settings['link_text']);

		if( !empty($settings['link_to']) ) {

			if( 'website' === $settings['link_to'] && !empty($author['website']) ) {
				$link_tag     = 'a';
				$link_url     = $author['website'];
				$link_target  = '_blank';
			} 
			else if( 'posts_archive' === $settings['link_to'] && !empty($author['posts_url']) ) {
				$link_tag = 'a';
				$link_url = $author['posts_url'];
			}

			if( !empty($link_url) ) {
				$this->add_render_attribute( 'author_link', 'href', $link_url );
				if( !empty($link_target) ) {
					$this->add_render_attribute( 'author_link', 'target', $link_target );
				}
			}
		}

		$this->add_render_attribute(
			'button',
			'class', [
				'elementor-author-box__button',
				'elementor-button',
				'elementor-size-xs',
			]
		);

		if( $print_link ) $this->add_render_attribute( 'button', 'href', $author['posts_url'] );

		if( $print_link && !empty($settings['button_hover_animation']) ) {
			$this->add_render_attribute(
				'button',
				'class',
				'elementor-animation-' . esc_attr($settings['button_hover_animation'])
			);
		}

		if( $print_avatar ) {
			$this->add_render_attribute( 'avatar', 'src', $author['avatar'] );

			if ( !empty($author['display_name']) ) {
				$this->add_render_attribute( 'avatar', 'alt', $author['display_name'] );
			}
		}

		?>
		<div class="elementor-author-box le-instructor-box">
			<?php if( $print_avatar ): ?>
				<<?= $link_tag; ?> <?= $this->get_render_attribute_string( 'author_link' ); ?> class="elementor-author-box__avatar">
					<img <?= $this->get_render_attribute_string( 'avatar' ); ?>>
				</<?= $link_tag; ?>>
			<?php endif; ?>

			<div class="elementor-author-box__text">
				<?php if( $print_name ): ?>
					<<?= $link_tag; ?> <?= $this->get_render_attribute_string( 'author_link' ); ?>>
						<?= '<' . $author_name_tag . ' class="elementor-author-box__name">' . $author['display_name'] . '</' . $author_name_tag . '>'; ?>
					</<?= $link_tag; ?>>
        <?php endif; ?>

        <?php if( $print_label ): ?>
          <span class="le-instructor-label">
            <?= $instuctor['label'] ?> 
          </span>
				<?php endif; ?>
				
        <?php if( $print_bio ): ?>
					<div class="elementor-author-box__bio">
						<?= $author['bio']; ?>
					</div>
				<?php endif; ?>

				<?php if( $print_link ): ?>
					<a <?= $this->get_render_attribute_string( 'button' ); ?>>
						<?= esc_html($settings['link_text']) ?>
					</a>
				<?php endif; ?>
			</div>
		</div>
		<?php
	}

  /**
   * Reset the instructor render attributes, so that we can render multiples
   * instructors 
   */
  private function reset_render_attribute() {
    $this->add_render_attribute( 'avatar', 'src', '', true );
    $this->add_render_attribute( 'author_link', 'href', '', true );
    $this->add_render_attribute( 'author_link', 'target', '', true );
    $this->add_render_attribute( 'button','class', [], true );
    $this->add_render_attribute( 'button', 'href', '', true );
    $this->add_render_attribute( 'button', 'class', '', true );
    $this->add_render_attribute( 'avatar', 'alt', '', true );
  }

}




