<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;


/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class BackToCourse extends \Elementor\Widget_Button {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-course-back-to-course';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Back to Course Button', 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * Modify control defined in the parent class
   * 
   * @see inc/Elementor.php in the update_controls_widgets() method
   * 
   * @param      obj     $element     The element
   * @param      string  $section_id  The section identifier
   * @param      array   $args        The arguments
   */
  public static function modify_controls( $element, $section_id, $args ) {

    // Hide the controls if it's not a custom button 
    if( $section_id !== 'link_type' ) {

      $control = $element->get_controls( $section_id );
      $control['conditions'] = [
        'terms' => [
          [
            'name'    => 'outline_type',
            'operator' => 'in',
            'value'   => [
              'custom_button'
            ],
          ],
        ]
      ];

      $element->update_control( $section_id, $control );
    }

    if( $section_id !== 'section_button' ) return;

    // We hide the link control and set the default value to the next item of the lesson
    $link = lift\back_to_course_url();

    $default = [
      'url'         => $link,
      'is_external' => false,
      'nofollow'    => true,
    ];

    utils\hide_control( $element, 'link', $default );
    
    // If the link is empty, the render should not be output 
    if( empty($link) && !utils\is_builder() ) {
      utils\no_render( $element );  
    }
    
    $element->set_render();
  }


  /**
   * Adds before the existing controls 
   *
   * @see https://code.elementor.com/php-hooks/#elementorelementbefore_section_start
   * @see inc/Elementor.php in the update_controls_widgets() method
   * 
   * @param      obj     $element     The element
   * @param      string  $section_id  The section identifier
   * @param      array   $args        The arguments
   */
  public static function add_controls( $element, $section_id, $args ) {
    
    if( $section_id !== 'section_button' ) return;

    $element->start_controls_section(
      'link_type',
      [
        'label' => __( 'Link type', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $element->add_control(
      'outline_type', [
        'label'   => __( 'Render', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'custom_button',
        'options' => [
          'lifter_element'  => __( 'LifterLMS render', 'lifter-elements' ),
          'custom_button'   => __( 'Custom button', 'lifter-elements' ),
        ],
      ]
    );

    $element->end_controls_section();
  }


  /**
   * Sets the render.
   *
   * @param      object  $widget  The widget
   */
  public function set_render() {
    
    $widget = &$this;
    
    add_action('elementor/widget/render_content', function( $content, $element ) use( $widget ) {
        
      if( $element->get_id() === $widget->get_id() ) {
          
        $settings = $element->get_settings_for_display();
        
        /**
         * If outline_type is set on lifter_element 
         * we bypass the render defiened in the parent class
         */
        if( $settings['outline_type'] === 'lifter_element' ) {
          return lift\back_to_course();
        }
      }

      return $content;
    }, 10, 2 );
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {
    $this->set_render();  
  }
}


