<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

/**
 * 
 * Extends the default Elementor Button widget 
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Content extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-course-content';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Course Content', 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-post-content";
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {

    $this->start_controls_section(
      'section_style',
      [
        'label' => __( 'Text', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_STYLE,
      ]
    );

    $this->add_responsive_control(
      'align',
      [
        'label'   => __( 'Alignment', 'elementor' ),
        'type'    => \Elementor\Controls_Manager::CHOOSE,
        'options'   => [
          'left'    => [
            'title'   => __( 'Left', 'elementor' ),
            'icon'    => 'eicon-text-align-left',
          ],
          'center'  => [
            'title'   => __( 'Center', 'elementor' ),
            'icon'    => 'eicon-text-align-center',
          ],
          'right'   => [
            'title'   => __( 'Right', 'elementor' ),
            'icon'    => 'eicon-text-align-right',
          ],
          'justify'   => [
            'title'   => __( 'Justified', 'elementor' ),
            'icon'    => 'eicon-text-align-justify',
          ],
        ],
        'selectors' => [
          '{{WRAPPER}} .le-course-content' => 'text-align: {{VALUE}};',
        ],
      ]
    );

    $this->add_control(
      'text_color',
      [
        'label'   => __( 'Text Color', 'elementor' ),
        'type'    => \Elementor\Controls_Manager::COLOR,
        'default'   => '',
        'selectors' => [
          '{{WRAPPER}} .le-course-content' => 'color: {{VALUE}};',
        ],
        'scheme' => [
          'type'  => \Elementor\Core\Schemes\Color::get_type(),
          'value' => \Elementor\Core\Schemes\Color::COLOR_3,
        ],
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Typography::get_type(),
      [
        'name'    => 'typography',
        'scheme'  => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_3,
      ]
    );

    $text_columns = range( 1, 10 );
    $text_columns = array_combine( $text_columns, $text_columns );
    $text_columns[''] = __( 'Default', 'elementor' );

    $this->add_responsive_control(
      'text_columns',
      [
        'label'   => __( 'Columns', 'elementor' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'separator' => 'before',
        'options'   => $text_columns,
        'selectors' => [
          '{{WRAPPER}} .le-course-content' => 'columns: {{VALUE}};',
        ],
      ]
    );

    $this->add_responsive_control(
      'column_gap',
      [
        'label'     => __( 'Columns Gap', 'elementor' ),
        'type'      => \Elementor\Controls_Manager::SLIDER,
        'size_units'=> [ 'px', '%', 'em', 'vw' ],
        'range' => [
          'px' => [
            'max' => 100,
          ],
          '%' => [
            'max'   => 10,
            'step'  => 0.1,
          ],
          'vw' => [
            'max'   => 10,
            'step'  => 0.1,
          ],
          'em' => [
            'max'   => 10,
            'step'  => 0.1,
          ],
        ],
        'selectors' => [
          '{{WRAPPER}} .le-course-content' => 'column-gap: {{SIZE}}{{UNIT}};',
        ],
      ]
    );

    $this->end_controls_section();

  }


  /**
   * Gets the post content. (Using a method allow use to use 
   * another content in widgets extendting this one, like 
   * lesson content)
   *
   * @return     string  The post content.
   */
  protected function get_post_content() {
    return lift\course_content();
  }


  /**
   * Front-end render
   *
   * @return string
   */
  public function render() {

    $content = $this->get_post_content();

    if( !ctype_space($content) ) {
      ?>
        <div class="le-course-content">
          <?= $content ?>
        </div>
      <?php
    }
    // Preview in the builder if no content
    else if( utils\is_builder() ) {

      echo utils\preview_message( 'content', 'course' ); 
      
      ?><div class="le-course-content">
        <p>Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vestibulum sagittis maximus turpis, sit amet luctus est efficitur sed. Vestibulum congue eu orci sed mattis. Nullam quam tortor, suscipit non enim nec, cursus mattis nisl. Nulla vehicula vulputate enim, nec venenatis quam auctor sed. Duis dapibus ultricies porttitor.</p>
        <p>Proin accumsan neque eget elit convallis rutrum. Quisque non faucibus orci, ut euismod nibh. Sed quis laoreet leo. Proin dictum tristique congue. Integer ut magna eu eros tempor suscipit. Etiam ligula urna, interdum quis ipsum at, mollis luctus ipsum.</p>
      </div><?php

    }
  }
}


