<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

/**
 * 
 * Extends the default Elementor Button widget 
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 * @see /elementor/includes/widgets/button.php
 */
class ContinueButton extends \Elementor\Widget_Button {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-continue-button';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Continue Button', 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * Remove the url field and get the right url according to the
	 * course/user
	 * 
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 */
	public static function modify_controls($element, $section_id, $args ) {
		
		if( $section_id !== 'section_button' ) return;

		// We hide the link control and set the default value to the next item of the lesson
		$link = lift\continue_url();

		$default = [
			'url'          => $link,
			'is_external'  => false,
			'nofollow'     => true,
		];

		utils\hide_control( $element, 'link', $default );
 
		// If the link is empty, the render shall not be output 
		if( empty($link) && !utils\is_builder() ) {
			utils\no_render( $element );	
		}
		// Builder mode: render but add the warning before
		elseif( empty($link) && utils\is_builder() ){
			utils\preview_render( $element );
		}
	}
}


