<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

/**
 * Can't use List as a class name
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Courses extends \Elementor\Widget_Base {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-list';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course List', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-bullet-list";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


  /**
   * Gets the filter that can be used for filtering terms.
   */
  public function get_taxonomy_filters() {
    return apply_filters( 'le-elementor-courses-widget-filter-taxonomies', [
      'category'  => 'course_cat',
      'tag'       => 'course_tag',
      'track'     => 'course_track',
      'difficulty'=> 'course_difficulty',
    ]);
  }


	/**
	 * Return an array with all the categories for the select control
	 * 
	 * @return array
	 */
	public function get_terms( $slug ) {

		$terms = get_terms([
    	'taxonomy' 		=> $slug,
    	'hide_empty' 	=> false,
		]);

		$response = ['' => __( 'None', 'lifter-elements' )];
		foreach( $terms as $key => $term ) $response[$term->slug] = $term->name;			 

		return $response;
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {
		
		$this->start_controls_section(
			'render_type',
			[
				'label' => __( 'Render', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'template', [
				'label' 	=> __( 'Render', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'custom',
				'options'	=> [
					'lifter'	=> __( 'LifterLMS render', 'lifter-elements' ),
					'custom'	=> __( "LifterElements' render", 'lifter-elements' ),
				],
			]
		);
    
    $this->add_responsive_control(
      'items_per_row', [
        'label'   => __( 'Columns', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => '3',
        'options' => [
          '1'  => '1',
          '2'  => '2',
          '3'  => '3',
          '4'  => '4',
          '5'  => '5',
          '6'  => '6'
        ],
        'separator'     => 'before',
        'responsive'    => true,
        'render_type'   => 'template',
        'prefix_class'  => "le-course-loop-items-%s-", // @see _courses.scss
      ]
    );

    $this->add_control(
      'is_equal_height',
      [
        'label'        => __( 'Equal Heights?', 'lifter-elements' ),
        'type'         => \Elementor\Controls_Manager::SWITCHER,
        'label_on'     => __( 'On', 'lifter-elements' ),
        'label_off'    => __( 'Off', 'lifter-elements' ),
        'return_value' => 'yes',
        'render_type'  => 'template',
        'default'      => 'no',
      ]
    );

    $this->add_responsive_control(
      'min_height',
      [
        'label' => __( 'Min Height', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max'   => 2000,
            'step'  => 1,
          ],
          'vh' => [
            'max'   => 100,
            'step'  => 1,
          ],
          'em' => [
            'max'   => 100,
            'step'  => 1,
          ],
        ],
        'default' => [
          'unit' => 'px',
          'size' => 200,
        ],
        'selectors' => [
          '{{WRAPPER}} .le-course-item-loop, {{WRAPPER}} .llms-loop-item-content' => 'min-height: {{SIZE}}{{UNIT}};',
        ],
        'size_units'  => [ 'px', 'vh', 'em' ],
        'render_type' => 'template',
        'conditions'  => [
          'terms' => [
            [
              'name'     => 'is_equal_height',
              'operator' => '!in',
              'value'    => [ 'yes' ],
            ],
          ]
        ]
      ]
    );
 
    $this->add_responsive_control(
      'height_items', [
        'label'       => __( 'Height', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SLIDER,
        'render_type' => 'template',
        'size_units'  => [ 'px', 'vh', 'em' ],
        'range'       => [
          'px' => [
            'min' => 100,
            'max' => 1000,
          ],
          'vh' => [
            'min' => 10,
            'max' => 100,
          ],
        ],
        'default' => [
          'size' => 400,
        ],
        'selectors'   => [
          '{{WRAPPER}} .le-course-item-loop, {{WRAPPER}} .llms-loop-item-content' => 'height: {{SIZE}}{{UNIT}};',
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'     => 'is_equal_height',
              'operator' => 'in',
              'value'    => [ 'yes' ],
            ],
          ]
        ]
      ]
    );

		$this->end_controls_section();

		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

    foreach( $this->get_taxonomy_filters() as $key => $slug ) {
  	
      $this->add_control(
  			$key, [
  				'label'     => sprintf( __( 'Course %s', 'lifter-elements' ), ucfirst($key) ),
  				'type'      => \Elementor\Controls_Manager::SELECT,
  				'default'   => '',
  				'options'   => $this->get_terms( $slug ),
        ]
  		);
    
    }

    $this ->add_control(
      'hr_1', [
        'type'  => \Elementor\Controls_Manager::DIVIDER
      ]
    );

		$this->add_control(
			'hidden',
			[
				'label' 		   => __( 'Show hidden courses', 'lifter-elements' ),
				'type' 			   => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		 => __( 'Show', 'lifter-elements' ),
				'label_off' 	 => __( 'Hide', 'lifter-elements' ),
				'return_value' => 'yes',
				'default' 		 => 'yes'
			]
		);

		$this->add_control(
			'mine', [
				'label' 	=> __( 'Filters:', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'any',
				'options'	=> [
					'no'		     => __( 'None (show all courses)', 'lifter-elements' ),
					'any'		     => __( 'Show all enrolled courses (including expired & cancelled)', 'lifter-elements' ),
          'enrolled'   => __( 'Show only available enrolled courses', 'lifter-elements' ),
					'unenrolled' => __( 'Show only available unenrolled courses', 'lifter-elements' ),
					'expired'	   => __( 'Show expired enrolled courses', 'lifter-elements' ),
					'cancelled'	 => __( 'Show cancelled enrolled courses', 'lifter-elements' ),
				]		
      ]
		);

		$this->add_control(
			'order', [
				'label' 	=> __( 'Order', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'ASC',
				'options'	=> [
					'ASC'		=> __( 'Ascending order', 'lifter-elements' ),
					'DESC'	=> __( 'Descending order', 'lifter-elements' ),
				]			
			]
		);

		$this->add_control(
			'orderby', [
				'label' 	=> __( 'Order by', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'title',
				'options'	=> [
					'author'           => __( 'Author', 'lifter-elements' ),
					'course_completion'=> __( 'Course completion', 'lifter-elements' ),
					'date'             => __( 'Date', 'lifter-elements' ),
					'ID'               => __( 'ID', 'lifter-elements' ),
					'modified'         => __( 'Last modification', 'lifter-elements' ),
          'menu_order'       => __( 'Menu order', 'lifter-elements' ),
					'name'             => __( 'Name', 'lifter-elements' ),
					'rand'             => __( 'Random', 'lifter-elements' ),
					'title'            => __( 'Title', 'lifter-elements' ),
				]			
			]
		);

		$this->add_control(
			'posts_per_page',
			[
				'label' 		  => __( 'Posts per page', 'lifter-elements' ),
        'separator'   => 'before',
				'type' 			  => \Elementor\Controls_Manager::NUMBER,
				'description' => __( '-1 for all', 'lifter-elements' ),
				'min' 			  => -1,
				'step' 			  => 1,
				'default'     => -1
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'custom_template',
			[
				'label'       => __( 'Custom template', 'lifter-elements' ),
				'tab' 			  => \Elementor\Controls_Manager::TAB_CONTENT,
				'conditions'  => [
					'terms' => [
						[
							'name' 		  => 'template',
							'operator' 	=> 'in',
							'value' 	  => ['custom'],
						],
					]
				]
			]
		);

		// Start Repeater
		$repeater = new \Elementor\Repeater();

		$repeater->add_control(
			'element_type', [
				'label' 	=> __( 'Element type', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'title',
				'options'	=> [
					'title'			     => __( 'Course title', 'lifter-elements' ),
					'excerpt'		     => __( 'Course excerpt', 'lifter-elements' ),
					'link'			     => __( 'Course link', 'lifter-elements' ),
					'price'			     => __( 'Course price', 'lifter-elements' ),
					'progress'	     => __( 'Course progress', 'lifter-elements' ),
          'custom'         => __( 'Custom text', 'lifter-elements' ),
					'image'			     => __( 'Image', 'lifter-elements' ),
          'taxonomy'       => __( 'Taxonomy', 'lifter-elements' ),
					'video'			     => __( 'Video', 'lifter-elements' ),
					'video_or_image' => __( 'Video or featured image', 'lifter-elements' ),
				]
			]
		);

    // Control common to all element type
    $repeater->add_control(
      'element_classes',
      [
        'label'   => __( 'CSS classes', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::TEXT,   
        'default' => '',   
      ]
    );
    // End common controls

    // Start Excerpt
    $repeater->add_control(
     'element_excerpt_characters_number',
     [
       'label'      => __( 'Max. characters number', 'lifter-elements' ),
       'type'       => \Elementor\Controls_Manager::NUMBER,
       'default'    => 100,
       'conditions' => [
         'terms' => [
           [
             'name'     => 'element_type',
             'operator' => 'in',
             'value'    => ['excerpt'],
           ],
         ]
       ]   
     ]
    );

    $repeater->add_control(
     'element_excerpt_more',
     [
       'label'      => __( 'Content when text is trimmed', 'lifter-elements' ),
       'type'       => \Elementor\Controls_Manager::TEXT,
       'default'    => '...',
       'conditions' => [
         'terms' => [
           [
             'name'     => 'element_type',
             'operator' => 'in',
             'value'    => ['excerpt'],
           ],
         ]
       ]   
     ]
    );
    // End Excerpt
    
    // Progress
    $repeater->add_responsive_control(
      'progress_bar_height',
      [
        'label' => __( 'Height', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max'   => 100,
            'step'  => 1,
          ],
          'vh' => [
            'max'   => 20,
            'step'  => 1,
          ],
          'em' => [
            'max'   => 10,
            'step'  => 1,
          ],
        ],
        'default' => [
          'unit' => 'em',
          'size' => 0.4,
        ],
        'selectors' => [
          '{{WRAPPER}} .le-course-progress-loop .llms-progress-bar' => 'height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-loop .llms-progress' => 'height: {{SIZE}}{{UNIT}};',
        ],
        'size_units'  => [ 'px', 'vh', 'em' ],
        'render_type' => 'template',
        'conditions'  => [
          'terms' => [
            [
              'name'     => 'element_type',
              'operator' => 'in',
              'value'    => [ 'progress' ],
            ],
          ]
        ]
      ]
    );

		$repeater->add_control(
			'show_progress_number',
			[
				'label' 	=> __( 'Show progress number?', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default' => 'no',
				'options'	=> [
					'no'		   => __( 'No', 'lifter-elements' ),
					'percent'	 => __( 'Percentage', 'lifter-elements' ),
					'steps'	   => __( 'Steps', 'lifter-elements' ),
				],
				'conditions' => [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator'  => 'in',
							'value' 	  => ['progress'],
						],
					]
				]		
			]
		);
    // End progress

    // Start Image
    $repeater->add_control(
      'image_type',
      [
        'label'   => __( 'Image type', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'featured',
        'options' => [
          'featured'  => __( 'Course featured image', 'lifter-elements' ),
          'custom'    => __( 'Custom image', 'lifter-elements' ),
        ],
        'conditions' => [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['image'],
            ],
          ]
        ]   
      ]
    );

    $repeater->add_control(
      'custom_image',
      [
        'label'     => __( 'Add Image', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::MEDIA,
        'conditions'=> [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['image'],
            ],[
              'name'      => 'image_type',
              'operator'  => 'in',
              'value'     => ['custom'],
            ],
          ]
        ]
      ]
    );

    $repeater->add_control(
      'is_image_link',
      [
        'label'         => __( 'Add link?', 'lifter-elements' ),
        'type'          => \Elementor\Controls_Manager::SWITCHER,
        'label_on'      => __( 'Yes', 'lifter-elements' ),
        'label_off'     => __( 'No ', 'lifter-elements' ),
        'return_value'  => 'yes',
        'default'       => 'no',
        'conditions'    => [
          'terms' => [
            [
              'terms' => [
                [
                  'name'      => 'element_type',
                  'operator'  => 'in',
                  'value'     => ['image', 'video_or_image'],
                ],
              ],
            ]
          ]
        ]   
      ]
    );
    // End Image

		// Price
		$repeater->add_control(
			'element_price',
			[
				'label' 	=> __( 'Plan', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default' => 'first_featured',
				'options'	=> [
					'first_featured' => __( 'First featured plan', 'lifter-elements' ),
					'first_plan'		 => __( 'First plan', 'lifter-elements' ),
				],
				'conditions' => [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['price'],
						],
					]
				]		
			]
		);

		$repeater->add_control(
			'element_price_output',
			[
				'label' 	=> __( 'Output style', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default' => 'frequency_price',
				'options'	=> [
					'frequency_price'	 => __( 'Price and frequency', 'lifter-elements' ),
					'price'				     => __( 'Price', 'lifter-elements' ),
					'frequency'			   => __( 'Frequency', 'lifter-elements' ),
				],
				'conditions' => [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['price'],
						],
					]
				]		
			]
		);

    $repeater->add_control(
      'element_price_old',
      [
        'label'         => __( 'Show old price?', 'lifter-elements' ),
        'type'          => \Elementor\Controls_Manager::SWITCHER,
        'label_on'      => __( 'Show', 'lifter-elements' ),
        'label_off'     => __( 'Hide', 'lifter-elements' ),
        'return_value'  => 'show',
        'default'       => 'hide',
        'description'   => __( 'Will display the original price when the access plan is on sale.', 'lifter-elements' ),
        'conditions'    => [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['price'],
            ],
          ]
        ]
      ]
    );
		// End Price

		// Progress
		$repeater->add_control(
			'progress_show_visitor',
			[
				'label' 		    => __( 'Show progress for non enrolled user', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		  => __( 'Show', 'lifter-elements' ),
				'label_off' 	  => __( 'Hide', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		  => 'yes',
				'conditions'    => [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['progress'],
						],
					]
				]		
			]
		);
		// End Progress

		// Title
		$repeater->add_control(
			'element_title_link',
			[
				'label' 		    => __( 'Title link to the course', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		  => __( 'Enable', 'lifter-elements' ),
				'label_off' 	  => __( 'Disable', 'lifter-elements' ),
				'return_value'  => 'yes',
				'default' 		  => 'yes',
				'conditions'    => [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['title'],
						],
					]
				]		
			]
		);
		
		$repeater->add_control(
			'element_title_tag',
			[
				'label' 	=> __( 'HTML Tag', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default' => 'h4',
				'options'	=> [
					'h1'	=> __( 'H1', 'lifter-elements' ),
					'h2'	=> __( 'H2', 'lifter-elements' ),
					'h3'	=> __( 'H3', 'lifter-elements' ),
					'h4'	=> __( 'H4', 'lifter-elements' ),
					'h5'	=> __( 'H5', 'lifter-elements' ),
					'h6'	=> __( 'H6', 'lifter-elements' ),
					'div'	=> __( 'div', 'lifter-elements' ),
					'span'=> __( 'span', 'lifter-elements' ),
					'p'		=> __( 'p', 'lifter-elements' ),
				],
				'conditions' => [
					'terms' => [
						[
							'name' 		=> 'element_type',
							'operator'=> 'in',
							'value' 	=> ['title'],
						],
					]
				]		
			]
		);
    // End Title

    // Link (some controls are also used in image and video_or_image)
    $repeater->add_control(
      'element_link_destination',
      [
        'label'       => __( 'Target', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SELECT,
        'return_value'=> 'yes',
        'default'     => 'course_page',
        'options'     => [
          'course_page'     => __( 'Course page', 'lifter-elements' ),
          'continue_course' => __( 'Continue course', 'lifter-elements' ),
        ],
        'conditions'    => [
          'relation' => 'or',
          'terms' => [
            [
              'terms' => [
                [
                  'name'      => 'element_type',
                  'operator'  => 'in',
                  'value'     => ['link'],
                ],
              ],
            ],[
              'terms' => [
                [
                  'name'      => 'is_image_link',
                  'operator'  => 'in',
                  'value'     => ['yes'],
                ],[
                  'name'      => 'element_type',
                  'operator'  => 'in',
                  'value'     => ['image', 'video_or_image'],
                ],
              ],
            ]
          ]
        ]   
      ]
    );

		$repeater->add_control(
			'element_link_is_custom',
			[
				'label' 		   => __( 'Use custom text', 'lifter-elements' ),
				'type' 			   => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		 => __( 'Yes', 'lifter-elements' ),
				'label_off' 	 => __( 'No', 'lifter-elements' ),
				'return_value' => 'yes',
				'default' 		 => 'no',
				'conditions'   => [
					'terms' => [
						[
							'name'     => 'element_type',
							'operator' => 'in',
							'value'    => ['link'],
						],
					]
				]		
			]
		);

		$repeater->add_control(
			'element_link_custom_text',
			[
				'label' 	  => __( 'Custom text', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::TEXT,
				'default' 	=> __( 'Link to the course', 'lifter-elements' ),
				'conditions' => [
					'terms' => [
						[
							'name'     => 'element_type',
							'operator' => 'in',
							'value'    => ['link'],
						],
						[
							'name'     => 'element_link_is_custom',
							'operator' => 'in',
							'value'    => ['yes'],
						],
					]
				]		
			]
		);

    $repeater->add_responsive_control(
      'element_link_align',
      [
        'label' => __( 'Alignment', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::CHOOSE,
        'options' => [
          'left'  => [
            'title' => __( 'Left', 'lifter-elements' ),
            'icon'  => 'eicon-text-align-left',
          ],
          'center' => [
            'title' => __( 'Center', 'lifter-elements' ),
            'icon'  => 'eicon-text-align-center',
          ],
          'right' => [
            'title' => __( 'Right', 'lifter-elements' ),
            'icon'  => 'eicon-text-align-right',
          ],
        ],
        'selectors' => [
          '{{WRAPPER}} .le-course-link-loop' => 'text-align: {{VALUE}};',
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'     => 'element_type',
              'operator' => 'in',
              'value'    => ['link'],
            ],
          ],
        ]
      ]
    );

    $repeater->add_control(
      'element_link_style',
      [
        'label'   => __( 'HTML Tag', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'link',
        'options' => [
          'link'    => __( 'Text', 'lifter-elements' ),
          'button'  => __( 'Button', 'lifter-elements' ),
        ],
        'conditions' => [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['link'],
            ],
          ]
        ]   
      ]
    );

    $repeater->add_control(
      'element_link_button_render',
      [
        'label'   => __( 'Button type', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'lifter',
        'options' => [
          'lifter'     => __( 'LifterLMS Button', 'lifter-elements' ),
          'elementor'  => __( 'Elementor Button', 'lifter-elements' ),
        ],
        'conditions' => [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['link'],
            ],
            [
              'name'      => 'element_link_style',
              'operator'  => 'in',
              'value'     => ['button'],
            ],
          ]
        ]   
      ]
    );

    $repeater->add_control(
      'item_type_button', [
        'label'   => __( 'Type', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'default',
        'options' => [
          'default' => __( 'Default', 'lifter-elements' ),
          'info'    => __( 'Info', 'lifter-elements' ),
          'success' => __( 'Success', 'lifter-elements' ),
          'warning' => __( 'Warning', 'lifter-elements' ),
          'danger'  => __( 'Danger', 'lifter-elements' ),
        ],
        'render_type' => 'template',    
        'conditions'  => [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['link'],
            ],
            [
              'name'      => 'element_link_style',
              'operator'  => 'in',
              'value'     => ['button'],
            ],
            [
              'name'      => 'element_link_button_render',
              'operator'  => 'in',
              'value'     => ['elementor'],
            ],
          ]
        ]
      ]
    );

    $repeater->add_control(
      'item_size_button', [
        'label'   => __( 'Size', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'sm',
        'options' => [
          'xs'  => __( 'Extra Small', 'lifter-elements' ),
          'sm'  => __( 'Small', 'lifter-elements' ),
          'md'  => __( 'Medium', 'lifter-elements' ),
          'lg'  => __( 'Large', 'lifter-elements' ),
          'xl'  => __( 'Extra Large', 'lifter-elements' ),
        ],
        'render_type' => 'template',    
        'conditions'  => [
          'terms' => [
            [
              'name'     => 'element_type',
              'operator' => 'in',
              'value'    => ['link'],
            ],
            [
              'name'      => 'element_link_style',
              'operator'  => 'in',
              'value'     => ['button'],
            ],
            [
              'name'      => 'element_link_button_render',
              'operator'  => 'in',
              'value'     => ['elementor'],
            ],
          ],
        ]
      ]
    );

		$repeater->add_control(
			'link_selected_icon',
			[
				'label' 		  => __( 'Icon', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::ICONS,
				'label_block' => true,
				'conditions' 	=> [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['link'],
						],
					]
				]
			]
		);

		$repeater->add_control(
			'link_icon_align',
			[
				'label' 	  => __( 'Icon Position', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::SELECT,
				'default' 	=> 'before',
				'options' 	=> [
					'before' 	=> __( 'Before', 'lifter-elements' ),
					'after' 	=> __( 'After', 'lifter-elements' ),
				],
				'conditions' => [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['link'],
						],
					],
				]
			]
		);

    $repeater->add_control(
      'link_icon_indent',
      [
        'label' => __( 'Icon Spacing', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px'  => [
          'max' => 50,
          ],
        ],
        'selectors' => [
          '{{WRAPPER}} .le-course-link-loop .elementor-button .elementor-align-icon-right' => 'margin-left: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-link-loop .elementor-button .elementor-align-icon-left' => 'margin-right: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-link-loop .le-course-link-loop-text .elementor-align-icon-right' => 'margin-left: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-link-loop .le-course-link-loop-text .elementor-align-icon-left' => 'margin-right: {{SIZE}}{{UNIT}};',
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'     => 'element_type',
              'operator' => 'in',
              'value'    => ['link'],
            ],
          ],
        ]
      ]
    );

		// For creating the different color sections of the icon
		$icon_colors = [
			[ 
        'name' => 'normal', 
        'css' => '{{WRAPPER}} .llms-button-primary i, {{WRAPPER}} .elementor-button i', 
        'default' => "" 
      ],
			[ 
        'name' => 'hover', 
        'css' => '{{WRAPPER}} .llms-button-primary:hover i, {{WRAPPER}} .elementor-button:hover i', 
        'default' => "" 
      ]
		];

		$repeater->start_controls_tabs( 'tabs_icon_style' );

		foreach( $icon_colors as $icon ) {
			
			$repeater->start_controls_tab(
				"tab_link_icon_{$icon['name']}",
				[
					'label' 	   => __( lcfirst($icon['name']), 'lifter-elements' ),
					'conditions' => [
						'terms' => [
							[
								'name'      => 'element_type',
								'operator'  => 'in',
								'value'     => ['link'],
							],
						],
					],
				]
			);

			$repeater->add_control(
				"link_icon_{$icon['name']}_background_color",
				[
					'label' 	  => __( 'Icon Color', 'lifter-elements' ),
					'type' 		  => \Elementor\Controls_Manager::COLOR,
					'default'	  => $icon['default'],
					'selectors' => [
						"{$icon['css']}" => "color: {{VALUE}};",
					],
					'conditions' => [
						'terms' => [
							[
								'name' 		  => 'element_type',
								'operator' 	=> 'in',
								'value' 	  => ['link'],
							],
						],
					],
				]
			);
			
			$repeater->end_controls_tab();
		}

		$repeater->end_controls_tabs();
		// End Link

		// Custom
		$repeater->add_control(
			'element_custom_text',
			[
				'label'     => __( 'Custom text', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::TEXT,
				'dynamic'   => [ 'active' => true ],
				'conditions'=> [
					'terms' => [
						[
							'name' 		  => 'element_type',
							'operator' 	=> 'in',
							'value' 	  => ['custom'],
						],
					]
				]		
			]
		);
		// End Custom
    
    // Taxonomy
    $repeater->add_control(
      'taxonomy_name',
      [
        'label'     => __( 'Taxonomy', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::SELECT,
        'options'   => utils\get_taxonomies_by_post_types( 'course' ),
        'conditions'=> [
          'terms' => [
            [
              'name'      => 'element_type',
              'operator'  => 'in',
              'value'     => ['taxonomy'],
            ],
          ],
        ]
      ]
    );
    // End Taxonomy

    // Start Video/Image
    $repeater->add_control(
      'is_fallback',
      [
        'label'         => __( 'Fallback image if no content?', 'lifter-elements' ),
        'type'          => \Elementor\Controls_Manager::SWITCHER,
        'label_on'      => __( 'Show', 'lifter-elements' ),
        'label_off'     => __( 'Hide', 'lifter-elements' ),
        'return_value'  => 'yes',
        'default'       => 'yes',
        'conditions'    => [
          'relation' => 'or',
          'terms' => [
            [
              'terms' => [
                [
                  'name'      => 'element_type',
                  'operator'  => 'in',
                  'value'     => ['video', 'video_or_image'],
                ],
              ],
            ],[
              'terms' => [
                [
                  'name'      => 'image_type',
                  'operator'  => '!in',
                  'value'     => ['custom'],
                ],[
                  'name'      => 'element_type',
                  'operator'  => 'in',
                  'value'     => ['image'],
                ],
              ],
            ]
          ]
        ]   
      ]
    );
    // End Video/Image

		// End Repeater
		
		$this->add_control(
			'template_items',
			[
				'label'   => __( 'Template items', 'plugin-domain' ),
				'type'    => \Elementor\Controls_Manager::REPEATER,
				'fields'  => $repeater->get_controls(),
				'default' => [
					['element_type' => 'video_or_image'],
					['element_type' => 'title'],
					['element_type' => 'progress'],
					['element_type' => 'price'],
				],
				'title_field' => '{{{ element_type }}}',
			]
		);

    $this->end_controls_section();

    // Item tab
    $this->start_controls_section(
      'section_style_items',
      [
        'label' => __( 'Items', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_STYLE,
      ]
    );

    $this->add_control(
      "items_color",
      [
        'label'       => __( "Background Color", 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::COLOR,
        'selectors'   => [
          "{{WRAPPER}} .le-course-item-loop"    => 'background-color: {{VALUE}};',
          "{{WRAPPER}} .llms-loop-item-content" => 'background-color: {{VALUE}};'
        ],
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Border::get_type(),
      [
        'name'       => "items_border",
        'selector'   => '{{WRAPPER}} .le-course-item-loop, {{WRAPPER}} .llms-loop-item-content',
      ]
    );

    $this->add_control(
      "border_radius_items",
      [
        'label'      => __( 'Border Radius', 'lifter-elements' ),
        'type'       => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors'  => [
          '{{WRAPPER}} .le-course-item-loop' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          '{{WRAPPER}} .llms-loop-item-content' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Box_Shadow::get_type(),
      [
        'name'        => 'box_shadow_items',
        "label"       => __( "Box Shadow", 'lifter-elements' ),
        'selector'    => '{{WRAPPER}} .le-course-item-loop, {{WRAPPER}} .llms-loop-item-content',
      ]
    );

    $this->add_responsive_control(
      "padding_items",
      [
        'label'       => __( 'Padding', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units'  => [ 'px', 'em', '%' ],
        'selectors'   => [
          '{{WRAPPER}} .le-course-item-loop' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          '{{WRAPPER}} .llms-loop-item-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->end_controls_section();
    // End item

    // Typography tabs 
    $this->start_controls_section(
      'style_tab_typography',
      [
        'label'       => __( 'Typography', 'lifter-elements' ),
        'tab'         => \Elementor\Controls_Manager::TAB_STYLE,
        'conditions'  => [
          'terms' => [
            [
              'name'      => 'template',
              'operator'  => 'in',
              'value'     => ['custom'],
            ],
          ]
        ]
      ]
    );

    // Same controls for heading, description and price
    foreach( ['title', 'price', 'link', 'custom'] as $section ) {

      $label = ucfirst($section);
      
      $this->add_control(
        "{$section}_text_color",
        [
          'label'     => __( "{$label} Color", 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'default'   => '',
          'separator' => 'before',
          'selectors' => [
            "{{WRAPPER}} .le-course-{$section}-loop" => 'color: {{VALUE}};',
            "{{WRAPPER}} .le-course-{$section}-loop .le-course-{$section}-loop-text" => 'color: {{VALUE}};',
          ],
        ]
      );

      $this->add_group_control(
        \Elementor\Group_Control_Typography::get_type(),
        [
          'name'        => "{$section}_text_typography",
          "label"       => __( "{$label} Typography", 'lifter-elements' ),
          'selector'    => "{{WRAPPER}} .le-course-{$section}-loop, {{WRAPPER}} .le-course-{$section}-loop .le-course-{$section}-loop-text",
        ]
      );

      $this->add_group_control(
        \Elementor\Group_Control_Text_Shadow::get_type(),
        [
          'name'        => "{$section}_text_shadow",
          "label"       => __( "{$label} Text Shadow", 'lifter-elements' ),
          'selector'    => "{{WRAPPER}} .le-course-{$section}-loop, {{WRAPPER}} .le-course-{$section}-loop .le-course-{$section}-loop-text",
        ]
      );

      $this->add_control(
        "{$section}_text_align",
        [
          'label'   => __( 'Text Alignment', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::CHOOSE,
          'options'   => [
            'left'    => [
              'title'   => __( 'Left', 'lifter-elements' ),
              'icon'    => 'fa fa-align-left',
            ],
            'center'  => [
              'title'   => __( 'Center', 'lifter-elements' ),
              'icon'    => 'fa fa-align-center',
            ],
            'right'   => [
              'title'   => __( 'Right', 'lifter-elements' ),
              'icon'    => 'fa fa-align-right',
            ],
          ],
          'default'       => '',
          'prefix_class'  => "le-course-loop-{$section}-align-",
          'toggle'        => true,
          'render_type'   => 'template',
        ]
      );
      
      $this->add_responsive_control(
        "{$section}_padding",
        [
          'label'       => __( "{$label} Padding", 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::DIMENSIONS,
          'size_units'  => [ 'px', 'em', '%' ],
          'selectors'   => [
            "{{WRAPPER}} .le-course-{$section}-loop-text" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          ],
        ]
      );
    }

    $this->end_controls_section();
    // End typography tab

    // Progress bar style
    $this->start_controls_section(
      'style_tab_progress',
      [
        'label'       => __( 'Progress Bar', 'lifter-elements' ),
        'tab'         => \Elementor\Controls_Manager::TAB_STYLE,
        'conditions'  => [
          'terms' => [
            [
              'name'      => 'template',
              'operator'  => 'in',
              'value'     => ['custom'],
            ],
          ]
        ]
      ]
    );

    $this->add_control(
      "progress_color",
      [
        'label'     => __( "Bar Color", 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'default'   => '',
        'selectors' => [
          "{{WRAPPER}} .llms-loop-item-content .le-course-progress-loop .llms-progress .llms-progress-bar" => 'background-color: {{VALUE}};',
        ],
      ]
    );

    $this->add_control(
      "progress_bg_color",
      [
        'label'     => __( "Bar Background Color", 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'default'   => '',
        'selectors' => [
          "{{WRAPPER}} .le-course-progress-loop .llms-progress .progress-bar-complete" => 'background-color: {{VALUE}};',
        ],
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Border::get_type(),
      [
        'name'      => 'progress_bar_border',
        'selector'  => '{{WRAPPER}} .llms-loop-item-content .le-course-progress-loop .llms-progress .llms-progress-bar', 
        'separator' => 'before',
      ]
    );

    $this->add_responsive_control(
      'border_radius',
      [
        'label'       => __( 'Border Radius', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units'  => [ 'px', '%', 'em' ],
        'default'     => [
          'unit'      => 'px',
          "top"       => 30,
          "right"     => 30,
          "bottom"    => 30,
          "left"      => 30,
          "isLinked"  => true,
        ],
        'selectors'   => [
          '{{WRAPPER}} .llms-loop-item-content .le-course-progress-loop .llms-progress .llms-progress-bar' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-loop .llms-progress .progress-bar-complete' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],  
      ]
    );

    $this->add_responsive_control(
      "bar_padding",
      [
        'label'       => __( "Padding", 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units'  => [ 'px', 'em', '%' ],
        'separator'   => 'before',
        'default'     => [
          'unit'      => 'px',
          "top"       => 20,
          "right"     => 20,
          "bottom"    => 20,
          "left"      => 20,
          "isLinked"  => true,
        ],
        'selectors'   => [
          "{{WRAPPER}} .le-course-progress-loop .llms-progress" => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->end_controls_section();
    // End progress style

    // Button style
    $this->start_controls_section(
      'style_tab_button',
      [
        'label'       => __( 'Button', 'lifter-elements' ),
        'tab'         => \Elementor\Controls_Manager::TAB_STYLE,
        'conditions'  => [
          'terms' => [
            [
              'name'      => 'template',
              'operator'  => 'in',
              'value'     => ['custom'],
            ],
          ]
        ]
      ]
    );

    $this->start_controls_tabs( 'button_tabs' );

    $tabs = [
      [ 'name' => 'normal', 'css' => '{{WRAPPER}} .llms-button-primary, {{WRAPPER}} .elementor-button' ],
      [ 'name' => 'hover', 'css' => '{{WRAPPER}} .llms-button-primary:hover, {{WRAPPER}} .elementor-button:hover' ]
    ];

    foreach( $tabs as $tab ) {
      
      $this->start_controls_tab( $tab['name'], [ 'label' => __( ucfirst($tab['name']), 'lifter-elements' ) ] );
      
      $this->add_control(
        "{$tab['name']}_button_text_color",
        [
          'label'     => __( 'Text Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'selectors' => [
            $tab['css'] => 'color: {{VALUE}};',
          ],
        ]
      );

      $this->add_control(
        "{$tab['name']}_button_background_color",
        [
          'label'     => __( 'Background Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'selectors' => [
            $tab['css'] => 'background-color: {{VALUE}};',
          ],
        ]
      );

      $this->add_control(
        "{$tab['name']}_button_border_color",
        [
          'label'     => __( 'Border Color', 'elementor-pro' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'selectors' => [
            $tab['css'] => 'border-color: {{VALUE}};',
          ],
        ]
      );

      $this->end_controls_tab();
    }

    $this->end_controls_tabs();

    $this->add_control(
      'hr_9', [
        'type' => \Elementor\Controls_Manager::DIVIDER
      ]
    );

    $button_selector = "{{WRAPPER}} .llms-button-primary, {{WRAPPER}} .elementor-button";

    $this->add_group_control(
      \Elementor\Group_Control_Border::get_type(),
      [
        'name'        => 'enroll_group_border',
        'label'       => __( 'Border Style', 'lifter-elements' ),
        'selector'    => $button_selector,
        'exclude'     => [ 'color' ],
      ]
    );

   $this->add_control(
      'button_border_radius',
      [
        'label'      => __( 'Border Radius', 'lifter-elements' ),
        'type'       => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units' => [ 'px', '%' ],
        'selectors'  => [
          $button_selector => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
        ],
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Box_Shadow::get_type(),
      [
        'name'      => 'button_box_shadow',
        "label"     => __( "Box Shadow", 'lifter-elements' ),
        'selector'  => $button_selector,
      ]
    );

    $this ->add_control(
      'hr_11', [
        'type'  => \Elementor\Controls_Manager::DIVIDER
      ]
    );

    $this->add_responsive_control(
      'enroll_padding',
      [
        'label'       => __( 'Padding', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units'  => [ 'px', 'em', '%' ],
        'selectors'   => [
          $button_selector => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
        ],
      ]
    );

		$this->end_controls_section();
    // End button style
	
  }


	/**
	 * Front-end render
	 * 
	 * @return string
	 */
	protected function render() {

		$settings = $this->get_settings_for_display();
		
    $this->generate_styles();

    global $wp_query;
    $this->prev_query = $wp_query;

    foreach( $this->get_taxonomy_filters() as $key => $slug ) {
      $this->{$key . '_slug'} = $settings[$key] === '' ? null : $settings[$key];
    }

    $accepted_mine = ['no', 'any', 'enrolled', 'unenrolled', 'expired', 'cancelled'];	 
		
    $hidden 		    = $settings['hidden'] === 'yes' ? 'yes' : 'no'; 
		$mine			      = in_array($settings['mine'], $accepted_mine) ? $settings['mine'] : 'no';
		$order			    = esc_attr($settings['order']);
		$orderby		    = esc_attr($settings['orderby']);
		$posts_per_page	= (int) $settings['posts_per_page'];

		// If the template set for the loop is custom, we change the default template by our 
		if( $settings['template'] === 'custom' ) {
      add_filter( 'llms_get_template_part', [$this, 'set_template'], 10, 3 );
    	add_filter( 'le-course-loop-template', [$this, 'get_settings_from_template'], 10, 1 );
		}

    // The filter unenrolled dosen't exist into LifterLMS, we need to modify $wp_query (post__in to post__not_in)
    if( $mine === 'unenrolled' ) {
      add_action( 'lifterlms_before_loop', [$this ,'reverse_query_filter'] );
      $mine = 'enrolled';
    }

    // We add custom value for orderby (ASC and DESC course completion)
    if( $orderby === 'course_completion' ) {
      add_action( 'lifterlms_before_loop', [$this ,'change_query_order'], 99 );
    }

    // Lifter shortcode only support category filter, we need to handle the others
    add_action( 'lifterlms_before_loop', [$this, 'filter_taxonomies'], 99 );

    // Remove hook needed to the render of this instance of the widget (avoid conflict if courses widget)
    $this->remove_hooks();
    
		echo do_shortcode('[lifterlms_courses 
      hidden="' . $hidden . '" 
      category="' . $this->category_slug . '"  
      mine="' . $mine . '" 
      order="' . $order . '" 
      orderby="' . $orderby . '" 
      posts_per_page="' . $posts_per_page . '"
    ]');
  }


	/**
	 * Replace the LifterELement template for the loop by our template
	 */
	public function set_template( $template, $slug, $name ) {
		
    if( $slug === 'loop/content' && $name === 'course' ) {
      $template = LifterElements_DIR . 'includes/templates/widgets/course-loop.php';
    }

		return $template;
	}


  /**
   * Remove actions/filters needed by the render of the current instance of the widget
   * 
   * It avoids issues if we have multiple course list widget on the page 
   */
  public function remove_hooks() {

    add_filter( 'do_shortcode_tag', function( $output, $tag ) {
    
      if( $tag !== 'lifterlms_courses' ) return $output;
        
      remove_action('lifterlms_before_loop', [$this, 'reverse_query_filter']); 
      remove_action('lifterlms_before_loop', [$this, 'change_query_order'], 99 ); 
      remove_action('lifterlms_before_loop', [$this, 'filter_taxonomies'], 99 ); 
      remove_filter('llms_get_template_part', [$this, 'set_template']); 
      remove_filter('le-course-loop-template', [$this, 'get_settings_from_template']); 

      wp_reset_postdata();

      return $output;
    }, 1, 2 );

  }


  /**
   * Pass settings to the custom template (get settings from outside the class thanks to a filter)
   *
   * @param      array  $args   The arguments
   */
  public function get_settings_from_template( $args ) {
    return $this->get_settings_for_display();
  }


  /**
   * Change the initial order of $wp_query
   * 
   * @see lifterlms/includes/functions/llms.functions.templates.loop.php
   */
  public function change_query_order() {

    global $wp_query;
    usort($wp_query->posts, function( $a, $b ) use( $wp_query ) {
      return $wp_query->query['order'] === 'ASC' ?
        lift\course_progress_percent_int($a->ID) <=> lift\course_progress_percent_int($b->ID) : 
        lift\course_progress_percent_int($b->ID) <=> lift\course_progress_percent_int($a->ID); 
    });
  }

  public function filter_taxonomies() {

    $is_modifications = false;

    foreach( $this->get_taxonomy_filters() as $key => $slug ) {

      // Exception for category, handled by shortcode
      if( $key === 'category' ) continue;
      if( empty($this->{$key . '_slug'}) ) continue;

      $is_modifications = true;
      $this->filter_by_taxonomy( $this->{$key . '_slug'}, $slug );
    }

  }


  /**
   * Add filter by taxonomy to current query
   * 
   * @see lifterlms/includes/functions/llms.functions.templates.loop.php
   */
  public function filter_by_taxonomy( $value, $slug ) {

    if( empty($value) ) return;

    $args = [
      'taxonomy' => $slug,
      'field'    => 'slug',
      'terms'    => $value,
    ];

    global $wp_query;
    if( isset($wp_query->query['tax_query']) && is_array($wp_query->query['tax_query']) ) {
      $wp_query->query['tax_query'][] = $args;  
    }
    else{
      $wp_query->query['tax_query'] = [
        ['relation' => 'AND' ],
        $args
      ];  
    }

    $wp_query = new \WP_Query( $wp_query->query );
  }


  /**
   * Change a post__in filter to a post__not_in on $wp_query
   * 
   * @see lifterlms/includes/functions/llms.functions.templates.loop.php
   */
  public function reverse_query_filter() {
    
    global $wp_query;

    $wp_query->query['post__not_in'] = $wp_query->query['post__in'];
    unset($wp_query->query['post__in']);

    $wp_query = new \WP_Query( $wp_query->query );
  }


	/*
	 * Enqueue scripts for the current widget
	 */
	public function get_script_depends() {
		return ['wp-mediaelement'];
	}


	/**
	 * Enqueue styles for the current widget
	 */
	public function get_style_depends() {
		return ['wp-mediaelement'];
	}


  /**
   * General CSS for responsive element
   * 
   * @see elementor/includes/settings/settings.php
   */
  public function generate_styles() {

    // Get the breakpont set in the Elementor settings
    $breakpoints = \Elementor\Core\Breakpoints\Manager::get_default_config();
    
    $mobile = $breakpoints['mobile']['default_value'];
    $tablet = $breakpoints['tablet']['default_value'];

    ?><style>
      
      @media screen and (min-width: 0px) and (max-width: <?= $mobile ?>px) {
        <?php foreach( [1, 2, 3, 4, 5, 6] as $columns ): ?>
          .elementor-widget-le-course-list.le-course-loop-items--mobile-<?= $columns ?> .llms-loop-list .llms-loop-item {
            <?php if( $columns === 1 ): ?>
              width: 100%;
            <?php else: ?>
              width: calc(100% / <?= $columns ?>);
            <?php endif; ?>
          }
        <?php endforeach; ?>
      }

      @media screen and (min-width: <?= $mobile ?>px) and (max-width: <?= $tablet ?>px) {
        <?php foreach( [1, 2, 3, 4, 5, 6] as $columns ): ?>
          .elementor-widget-le-course-list.le-course-loop-items--tablet-<?= $columns ?> .llms-loop-list .llms-loop-item {
            <?php if( $columns === 1 ): ?>
              width: 100%;
            <?php else: ?>
              width: calc(100% / <?= $columns ?>);
            <?php endif; ?>
          }
        <?php endforeach; ?>
      }
    
    </style><?php
  }
	
}
