<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Elementor\Utils as elementor;

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Meta extends \Elementor\Widget_Base {

 
	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-meta';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Meta', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-post-info";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {
		
		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'container_tag', [
				'label' 	=> __( 'Container HTML tag', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'div',
				'options'	=> [
					'div'		  => 'div',
					'span'		=> 'span',
					'section'	=> 'section',
				]			
			]
		);

		$this->add_control(
			'link_content',
			[
				'label' 	  => __( 'Links', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'is_links',
			[
				'label' 		  => __( 'Enable all links', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Enable', 'lifter-elements' ),
				'label_off' 	=> __( 'Disable', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes'
			]
		);

		$this->add_control(
			'is_tracks_links',
			[
				'label' 		  => __( 'Enable tracks links', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Enable', 'lifter-elements' ),
				'label_off' 	=> __( 'Disable', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
				'conditions' 	=> [
					'terms' 	=> [
						[
							'name' 		=> 'is_links',
							'operator'=> '!in',
							'value' 	=> [
								'yes'
							],
						],
						[
							'name' 		=> 'is_course_tracks',
							'operator'=> 'in',
							'value' 	=> [
								'yes'
							],
						],
					]
				]
			]
		);

		$this->add_control(
			'is_categories_links',
			[
				'label' 		  => __( 'Enable categories links', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Enable', 'lifter-elements' ),
				'label_off' 	=> __( 'Disable', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
				'conditions' 	=> [
					'terms' 	=> [
						[
							'name' 		=> 'is_links',
							'operator'=> '!in',
							'value' 	=> [
								'yes'
							],
						],
						[
							'name' 		=> 'is_course_categories',
							'operator'=> 'in',
							'value' 	=> [
								'yes'
							],
						],
					]
				]
			]
		);

		$this->add_control(
			'is_tags_links',
			[
				'label' 		  => __( 'Enable tags links', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Enable', 'lifter-elements' ),
				'label_off' 	=> __( 'Disable', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
				'conditions' 	=> [
					'terms' 	=> [
						[
							'name' 		=> 'is_links',
							'operator'=> '!in',
							'value' 	=> [
								'yes'
							],
						],
						[
							'name' 		=> 'is_course_tags',
							'operator'=> 'in',
							'value' 	=> [
								'yes'
							],
						],
					]
				]
			]
		);

		$this->add_control(
			'sections_visibility:',
			[
				'label' 	  => __( 'Sections visibility', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'is_length',
			[
				'label' 		  => __( 'Show Course Estimated Completion Time', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Show', 'lifter-elements' ),
				'label_off' 	=> __( 'Hide', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes'
			]
		);

		$this->add_control(
			'is_difficulty',
			[
				'label' 		  => __( 'Show Course Difficulty', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Show', 'lifter-elements' ),
				'label_off' 	=> __( 'Hide', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes'
			]
		);

		$this->add_control(
			'is_course_tracks',
			[
				'label' 		  => __( 'Show Course Tracks', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Show', 'lifter-elements' ),
				'label_off' 	=> __( 'Hide', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes'
			]
		);

		$this->add_control(
			'is_course_categories',
			[
				'label' 		    => __( 'Show Course Categories', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		  => __( 'Show', 'lifter-elements' ),
				'label_off' 	  => __( 'Hide', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		  => 'yes'
			]
		);

		$this->add_control(
			'is_course_tags',
			[
				'label' 		    => __( 'Show Course Tags', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		  => __( 'Show', 'lifter-elements' ),
				'label_off' 	  => __( 'Hide', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		  => 'yes'
			]
		);

		$this->add_control(
			'custom_meta:',
			[
				'label' 	  => __( 'Custom', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$repeater = new \Elementor\Repeater();

		$repeater->add_control(
			'custom_label', [
				'label' 		  => __( 'Label', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::TEXT,
				'default' 		=> __( 'New meta' , 'lifter-elements' ),
				'label_block' => true,
			]
		);

		$repeater->add_control(
			'custom_content', [
				'label' 		 => __( 'Content', 'lifter-elements' ),
				'type' 			 => \Elementor\Controls_Manager::TEXT,
				'default' 	 => __( 'Meta Content' , 'lifter-elements' ),
				'label_block'=> true,
				'dynamic'		 => ['active' => true],
			]
		);

		$repeater->add_control(
			'is_custom_link',
			[
				'label' 		    => __( 'Link', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		  => __( 'Enable', 'lifter-elements' ),
				'label_off' 	  => __( 'Disable', 'lifter-elements' ),
				'return_value' 	=> 'yes',
				'default' 		  => 'yes'
			]
		);

		$repeater->add_control(
			'custom_link',
			[
				'label' 		    => __( 'Add a link', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::URL,
				'placeholder' 	=> __( 'https://your-link.com', 'lifter-elements' ),
				'show_external' => true,
				'dynamic' 		  => ['active' => true],
				'conditions' 	  => [
					'terms' 	=> [
						[
							'name' 		=> 'is_custom_link',
							'operator'=> 'in',
							'value' 	=> [
								'yes'
							],
						],
					]
				],
			]
		);

		$this->add_control(
			'custom_fields',
			[
				'label' 		    => __( 'Metas', 'lifter-elements' ),
				'type' 			    => \Elementor\Controls_Manager::REPEATER,
				'fields' 		    => $repeater->get_controls(),
				'title_field' 	=> '{{{ custom_label }}}',
				'prevent_empty' => false
			]
		);

		$this->end_controls_section();
		
		$this->start_controls_section(
			'section_style',
			[
				'label' => __( 'Text Editor', 'elementor' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'align',
			[
				'label' 	=> __( 'Alignment', 'elementor' ),
				'type' 		=> \Elementor\Controls_Manager::CHOOSE,
				'options' 	=> [
					'left' 		=> [
						'title' 	=> __( 'Left', 'elementor' ),
						'icon' 		=> 'fa fa-align-left',
					],
					'center' 	=> [
						'title' 	=> __( 'Center', 'elementor' ),
						'icon' 		=> 'fa fa-align-center',
					],
					'right' 	=> [
						'title' 	=> __( 'Right', 'elementor' ),
						'icon' 		=> 'fa fa-align-right',
					],
					'justify' 	=> [
						'title' 	=> __( 'Justified', 'elementor' ),
						'icon' 		=> 'fa fa-align-justify',
					],
				],
				'selectors' => [
					'{{WRAPPER}} .le-llms-meta-info' => 'text-align: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'Links:',
			[
				'label' 	  => __( 'Text', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_group_control(
			\Elementor\Group_Control_Typography::get_type(),
			[
				'name' 		=> 'typography',
				'scheme' 	=> \Elementor\Core\Schemes\Typography::TYPOGRAPHY_3,
			]
		);

		$this->start_controls_tabs( 'tabs_text_style' );
		
		$this->start_controls_tab(
			'tab_text_normal',
			[
				'label' => __( 'Normal', 'elementor' ),
			]
		);

		$this->add_control(
			'text_color',
			[
				'label' 	  => __( 'Color', 'elementor' ),
				'type' 		  => \Elementor\Controls_Manager::COLOR,
				'default' 	=> '',
				'selectors' => [
					'{{WRAPPER}}' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' 	=> \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
			]
		);

		$this->end_controls_tab();
		
		$this->start_controls_tab(
			'tab_text_hover',
			[
				'label' => __( 'Hover', 'elementor' ),
			]
		);

		$this->add_control(
			'text_color_hover',
			[
				'label'		  => __( 'Color', 'elementor' ),
				'type' 		  => \Elementor\Controls_Manager::COLOR,
				'default' 	=> '',
				'selectors' => [
					'{{WRAPPER}}:hover' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' 	=> \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->add_control(
			'link',
			[
				'label' 	  => __( 'Links', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->start_controls_tabs( 'tabs_link_style' );
		
		$this->start_controls_tab(
			'tab_link_normal',
			[
				'label' => __( 'Normal', 'elementor' ),
			]
		);

		$this->add_control(
			'link_color',
			[
				'label' 	  => __( 'Color', 'elementor' ),
				'type' 		  => \Elementor\Controls_Manager::COLOR,
				'default' 	=> '',
				'selectors' => [
					'{{WRAPPER}} a' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' 	=> \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
			]
		);

		$this->end_controls_tab();
		
		$this->start_controls_tab(
			'tab_link_hover',
			[
				'label' => __( 'Hover', 'elementor' ),
			]
		);

		$this->add_control(
			'link_color_hover',
			[
				'label' 	  => __( 'Color', 'elementor' ),
				'type' 		  => \Elementor\Controls_Manager::COLOR,
				'default' 	=> '',
				'selectors' => [
					'{{WRAPPER}}:hover a' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' 	=> \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'tab_link_focus',
			[
				'label' => __( 'Focus', 'elementor' ),
			]
		);

		$this->add_control(
			'link_color_focus',
			[
				'label' 	  => __( 'Color', 'elementor' ),
				'type' 		  => \Elementor\Controls_Manager::COLOR,
				'default' 	=> '',
				'selectors' => [
					'{{WRAPPER}} a:focus' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' 	=> \Elementor\Core\Schemes\Color::get_type(),
					'value' => \Elementor\Core\Schemes\Color::COLOR_3,
				],
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();
	}

	/**
	 * Remove link from the output of the 'lifterlms_template_*' function's output
	 *
	 * @param string $function_name
	 * 
	 * @return string
	 */
	protected function remove_link( $function_name ) {
		
		ob_start();
		call_user_func( $function_name );
		$output = ob_get_clean();

		$content = preg_replace( "/<a(.*?)>/","", $output );
		$content = str_replace( '</a>', '', $content );

		return $content;
	}
	

	/**
	 * Front-end render
	 *
	 * @see  lifterlms/includes/shortcodes/class.llms.shortcode.course.meta.info.php
	 * @see  lifterlms/includes/llms.template.functions.php
	 * 
	 * @return string
	 */
	protected function render() {

		$settings = $this->get_settings_for_display();

		$container_tag = elementor::validate_html_tag($settings['container_tag']);

		$is_links				      = $settings['is_links'] === 'yes';
		$is_tags_links 			  = $settings['is_tags_links'] === 'yes';
		$is_categories_links 	= $settings['is_categories_links'] === 'yes';
		$is_tracks_links 		  = $settings['is_tracks_links'] === 'yes';

		$is_length 				    = $settings['is_length'] === 'yes';
		$is_difficulty 			  = $settings['is_difficulty'] === 'yes';
		$is_course_tracks 		= $settings['is_course_tracks'] === 'yes';
		$is_course_categories = $settings['is_course_categories'] === 'yes';
		$is_course_tags 		  = $settings['is_course_tags'] === 'yes';

		echo '<' . $container_tag . ' class="le-llms-meta-info">';

		/**
		 * Quick way to adapt the the widget for course, we temporary modify
		 * the post global with the course (and reset it to normal just after)
		 * 
		 * This is temporary, the right method will be to create functions in BetterLifter 
		 * for having adaptative functions (an potentially use them in dynamic tags)
		 */
		global $post;
		$course = lift\course();

		if( !is_object($course) ) return '';
		
    $prev_post = $post;
		$post = $course;

		if( $is_length) \lifterlms_template_single_length();
		if( $is_difficulty) \lifterlms_template_single_difficulty();
		
		if( $is_course_tracks && ( $is_links === true || $is_tracks_links === true ) ) { 
			\lifterlms_template_single_course_tracks();
    } 
		else if( $is_course_tracks ) {
			echo $this->remove_link( '\lifterlms_template_single_course_tracks' );
    }
		
		if( $is_course_categories && ( $is_links === true || $is_categories_links === true ) ) {
			\lifterlms_template_single_course_categories();
    } 
		else if( $is_course_categories ) {
			echo $this->remove_link( '\lifterlms_template_single_course_categories' );
    }

		if( $is_course_tags && ( $is_links === true || $is_tags_links === true ) ) {
			\lifterlms_template_single_course_tags();
    } 
		else if( $is_course_tags ) {
			echo $this->remove_link( '\lifterlms_template_single_course_tags' );
    }

		// Custom fields
		if( !empty($settings['custom_fields']) ) {
			foreach( $settings['custom_fields'] as $meta ): ?>

				<div class="llms-meta">
					<p>
						<?= esc_html($meta['custom_label']) ?>:

						<?php if( $meta['is_custom_link'] === 'yes' ): 
							$target   = $meta['custom_link']['is_external'] ? ' target="_blank"' : '';
							$nofollow = $meta['custom_link']['nofollow'] ? ' rel="nofollow"' : ''; ?>	
							<a href="<?= esc_url($meta['custom_link']['url']) ?>"  <?= $target ?> <?= $nofollow ?>> 
						<?php endif; ?>	

						<?= esc_html($meta['custom_content']) ?>
						<?= $meta['is_custom_link'] === 'yes' ? '</a>' : '' ?>
					</p>
				</div>	
			
			<?php endforeach;
		}

		echo '</' . $container_tag . '>';

		// Reset the global post to the initial state
		$post = $prev_post;
	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}

}


