<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils; 

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Outline extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {

    /**
     * If for a reason or another we change the name, we need to change the css rules 
     * linked to the .elementor-widget-le-course-outline class as well
     * 
     * @see assets/src/widgets/outline/ 
     */
    return 'le-course-outline';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Course outline', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-editor-list-ol";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {

    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'collapse',
      [
        'label'       => __( 'Make the outline sections collapsible', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'On', 'lifter-elements' ),
        'label_off'   => __( 'Off', 'lifter-elements' ),
        'return_value'=> 'true',
        'default'     => 'false',
      ]
    );

    $this->add_control(
      'toggles',
      [
        'label'       => __( 'Collapse/Expand buttons', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'Collapse All', 'lifter-elements' ),
        'label_off'   => __( 'Expand All', 'lifter-elements' ),
        'return_value'=> 'true',
        'default'     => 'false',
        'conditions'  => [
          'terms'   => [
            [
              'name'      => 'collapse',
              'operator'  => 'in',
              'value'     => [
                'true'
              ],
            ],
          ]
        ]
      ]
    );

    $this->add_control(
      'outline_type', [
        'label'   => __( 'Outline type', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'full',
        'options' => [
          'full'            => __( 'Full', 'lifter-elements' ),
          'current_section' => __( 'Current section', 'lifter-elements' ),
        ],
      ]
    );

    $this->add_control(
      'item_spacing',
      [
        'label'     => __( 'Outline Gap', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::NUMBER,
        'min'       => 0,
        'step'      => 1,
        'default'   => 2,
        'selectors' => [
          "{{WRAPPER}} .llms-widget-syllabus ul li ul li" => "margin: 0 0 {{VALUE}}px 0;",
        ],
      ]
    );

    $this->add_control(
      'hr',
      [
        'type' => \Elementor\Controls_Manager::DIVIDER,
      ]
    );

    $this->start_controls_tabs( 'tabs_icons' );

    // The completed type will not take any css selector for being compatible wuth older versions
    foreach( ['completed', 'uncompleted'] as $icon_type ) {
    
      $suffix   = $icon_type === 'completed' ? '' : '_uncompleted'; 
      $selector = $icon_type === 'completed' ? '.llms-lesson-complete.done' : '.llms-lesson-complete';

      $this->start_controls_tab(
        "tab_icon_{$icon_type}",
        [
          'label'     => __( lcfirst($icon_type), 'lifter-elements' ),
        ]
      );


      if( $icon_type !== 'completed' ) {
        $this->add_control(
          "selected_icon_note{$suffix}",
          [
            'label'     => '',
            'type'      => \Elementor\Controls_Manager::RAW_HTML,
            'raw'       => '<i>If no icon selected, the completed icon will be used</i>',
            'condition' => [
              "selected_icon{$suffix}[value]" => '',
            ],
          ]
        );
      }

      $this->add_control(
        "selected_icon{$suffix}",
        [
          'label'       => __( 'Icon', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::ICONS,
          'label_block' => true,
          'default'     => $icon_type === 'completed' ? [
            'value'       => 'fa fa-check-circle',
            'library'     => 'solid',
          ] : [],
        ]
      );

      $this->add_control(
        "selected_icon{$suffix}_size",
        [
          'label'       => __( 'Icon size', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::SLIDER,
          'size_units'  => [ 'px' ],
          'range'       => [
            'px' => [
              'min'   => 0,
              'max'   => 100,
              'step'  => 5,
            ],
          ],
          'default' => [
            'unit'    => 'px',
            'size'    => 18,
          ],
          'selectors' => [
            "{{WRAPPER}} {$selector} i:before" => 'font-size: {{SIZE}}{{UNIT}};',
            "{{WRAPPER}} {$selector} svg" => 'width: {{SIZE}}{{UNIT}};',
          ],
        ]
      );

      $this->add_control(
        "icon_align{$suffix}",
        [
          'label'   => __( 'Icon Position', 'lifter-elements' ),
          'type'    => \Elementor\Controls_Manager::SELECT,
          'default' => 'before',
          'options' => [
            'before'  => __( 'Before', 'lifter-elements' ),
            'after'   => __( 'After', 'lifter-elements' ),
          ],
          'condition' => [
            "selected_icon{$suffix}[value]!" => '',
          ],
        ]
      );
      $this->end_controls_tab();
    } // End icons tabs

    $this->end_controls_tabs();

    // For creating the different color sections of the icon
    $icon_colors = [
      [ 'name' => 'normal',   'css' => 'llms-lesson-complete',        'default' => "#ccc" ],
      [ 'name' => 'hover',    'css' => 'llms-lesson-complete:hover',  'default' => "#646464" ],
      [ 'name' => 'complete', 'css' => 'llms-lesson-complete.done',   'default' => "#e5554e" ],
    ];

    $this->start_controls_tabs( 'tabs_icon_style' );

    foreach( $icon_colors as $icon ) {
      
      $this->start_controls_tab(
        "tab_icon_{$icon['name']}",
        [
          'label'     => __( lcfirst($icon['name']), 'lifter-elements' ),
          'condition' => [
            'selected_icon[value]!' => '',
          ],
        ]
      );

      $this->add_control(
        "icon_{$icon['name']}_background_color",
        [
          'label'     => __( 'Icon Color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'default'   => $icon['default'],
          'selectors' => [
            "{{WRAPPER}} .llms-widget-syllabus span.{$icon['css']}" => "color: {{VALUE}};",
          ],
          'condition' => [
            'selected_icon[value]!' => '',
          ],
        ]
      );
      
      $this->end_controls_tab();
    } // End icons colors tabs

    $this->end_controls_tabs();

    $this->add_control(
      'hr2',
      [
        'type' => \Elementor\Controls_Manager::DIVIDER,
      ]
    );

    // For creating the different typography sections of the icon
    $typos = [
      [ 'name' => 'normal',   'css' => 'lesson-title',    'default' => "#ccc" ],
      [ 'name' => 'complete', 'css' => 'lesson-title.done',   'default' => "#e5554e" ],
    ];

    $this->start_controls_tabs( 'tabs_typo_style' );

    foreach( $typos as $typo ) {
      
      $this->start_controls_tab(
        "tab_typo_{$typo['name']}",
        [
          'label' => __( 'Item ' . lcfirst($typo['name']), 'lifter-elements' ),
        ]
      );

      $this->add_group_control(
        \Elementor\Group_Control_Typography::get_type(),
        [
          'name'    => "{$typo['name']}_typography",
          'label'   => __( 'List item typography', 'lifter-elements' ),
          'scheme'  => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_1,
          'selector'=> "{{WRAPPER}} .llms-widget-syllabus span.{$typo['css']}, {{WRAPPER}} .llms-widget-syllabus span.{$typo['css']} a",
          'fields_options' => [
            'text_decoration' => [ 'default' => $typo['name'] === 'complete' ? 'line-through' : 'none' ],
          ],
        ]
      );  
      
      $this->add_control(
        "item_{$typo['name']}_color",
        [
          'label'     => __( 'List item text color', 'lifter-elements' ),
          'type'      => \Elementor\Controls_Manager::COLOR,
          'selectors' => [
            "{{WRAPPER}} .llms-widget-syllabus span.{$typo['css']}"   => "color: {{VALUE}};",
            "{{WRAPPER}} .llms-widget-syllabus span.{$typo['css']} a" => "color: {{VALUE}};",
          ],
        ]
      );

      $this->end_controls_tab();
    }

    $this->end_controls_tabs();

    $this->add_control(
      'hr3',
      [
        'type' => \Elementor\Controls_Manager::DIVIDER,
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Typography::get_type(),
      [
        'name'    => "title_typography",
        'label'   => __( 'Section Title Typography', 'lifter-elements' ),
        'scheme'  => \Elementor\Core\Schemes\Typography::TYPOGRAPHY_1,
        'selector'=> "{{WRAPPER}} .llms-widget-syllabus span.section-title",
      ]
    ); 

    $this->add_control(
      "section_text_color",
      [
        'label'     => __( 'Section Title Color', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'selectors' => [
          "{{WRAPPER}} .llms-widget-syllabus span.section-title" => "color: {{VALUE}};",
        ],
      ]
    ); 

    $this->end_controls_section();
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {

    $settings = $this->get_settings_for_display();
    
    $outline_type = in_array($settings['outline_type'],  ['full', 'current_section']) ? $settings['outline_type'] : 'full'; 
    $collapse     = $settings['collapse'];
    $toggles      = $settings['toggles'];

    $content = '';

    // We will not display if no sections or to an unlogged user if set to current_section
    $should_display = $settings['outline_type'] === 'current_section' ?
      !empty(lift\section_id()) || !is_user_logged_in() : 
      !empty(lift\course_sections_number());

    if( !$should_display ) return '';

    ob_start();
    echo do_shortcode('[lifterlms_course_outline outline_type="' . $outline_type . '" collapse="' . $collapse . '" toggles="' . $toggles . '"]');
    $content = ob_get_clean();

    // Temporary method (?), we will need to stop using the shortcode and recreate it at some point
    $content = !empty($content) ? $this->set_icon( $content, $settings ) : '';

    echo $content;
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {}


  /**
   * Sets the icon (probably temporary)
   *
   * @param      string  $content  The content (HTML)
   * 
   * @return    string $content 
   */
  protected function set_icon( $content, $settings ) {

    if( empty($content) ) return '';

    $doc = new \DOMDocument();
    @$doc->loadHTML( '<?xml encoding="utf-8" ?>' . $content );

    $spans = $doc->getElementsByTagName('span');

    if( empty($spans) ) return '';

    foreach( $spans as $span ) {
      $classes = $span->getAttribute('class');

      if( strpos($classes, 'llms-lesson-complete') !== false ) {
        // Remove the initial icon 
        $span->nodeValue = "";

        // Remplace with the new icon (check if icon complete or uncomplete)
        $icon_type  = $this->get_icon_type( $settings, $classes ); 
        $new_icon   = utils\get_node( $this->get_icon_html( $settings, $icon_type ) );

        if( !empty($new_icon) ) {
          $span->appendChild( $doc->importNode( $new_icon, true ) );
          
          // If icon is after the text we move the span as the next child
          $icon_align = $icon_type === 'completed' ? $settings['icon_align'] : $settings['icon_align_uncompleted'];
          if( $icon_align === 'after') $span->parentNode->appendChild($span);
        }
      } 
    }

    return utils\get_html( $doc );
  }


  /**
   * Gets the icon html.
   *
   * @param     array   $settings   The settings
   * @param     string  $icon_type  The icon type (completed/uncompleted)
   * 
   * @return    string $icon
   */
  protected function get_icon_html( $settings, $icon_type = 'completed' ) {
    
    if( !in_array( $icon_type, ['completed', 'uncompleted'] ) ) return '';

    $icon_settings = $icon_type === 'completed' ? $settings['selected_icon'] : $settings['selected_icon_uncompleted'];

    ob_start();
    \Elementor\Icons_Manager::render_icon( $icon_settings );
    return ob_get_clean();
  }

  /**
   * Gets the icon type. If no icon set for uncompleted, we will use
   * the complete icon instead
   *
   * @param       array    $settings  The settings
   * @param       string   $classes   The classes
   * 
   * @return      string
   */
  protected function get_icon_type( $settings, $classes ) {

    $icon_type = strpos($classes, 'done') !== false ? 'completed' : 'uncompleted';
    
    return isset($settings['selected_icon_uncompleted']) && $icon_type === 'uncompleted' && !empty($settings['selected_icon_uncompleted']['value']) 
      ? 'uncompleted'  
      : 'completed';  
  }

}

