<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Prerequisites extends \Elementor\Widget_Base {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-prerequisites';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Prerequisites', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-checkbox";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {
		
		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'output', [
				'label' 	=> __( 'Output type:', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'full',
				'options'	=> [
					'full' => __( 'Full template', 'lifter-elements' ),
					'text' => __( 'Text only', 'lifter-elements' ),
				],		
			]
		);

		$this->add_control(
			'is_custom_text',
			[
				'label' 		  => __( 'Use a custom text', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'no',
				'conditions' 	=> [
					'terms' => [
						[
							'name' 		=> 'output',
							'operator'=> 'in',
							'value' 	=> ['text'],
						],
					]
				]
			]
		);

		$this->add_control(
			'custom_text_course', [
				'label' 		  => __( 'Custom text for courses', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::TEXTAREA,
				'label_block'	=> true,
				'default'		  =>  __( 'Before starting this course you must complete the required prerequisite course:', 'lifterlms' ),
				'conditions'  => [
					'terms' => [
						[
							'name' 		=> 'is_custom_text',
							'operator'=> 'in',
							'value' 	=> ['yes'],
						],
						[
							'name' 		=> 'output',
							'operator'=> 'in',
							'value' 	=> ['text'],
						],
					]
				]		
			]
		);

		$this->add_control(
			'custom_text_track', [
				'label' 		  => __( 'Custom text for tracks', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::TEXTAREA,
				'label_block'	=> true,
				'default'		  =>  __( 'Before starting this course you must complete the required prerequisite track:', 'lifterlms' ),
				'conditions' 	=> [
					'terms' => [
						[
							'name' 		=> 'is_custom_text',
							'operator'=> 'in',
							'value' 	=> ['yes'],
						],[
							'name' 		=> 'output',
							'operator'=> 'in',
							'value' 	=> ['text'],
						],
					]
				]		
			]
		);

		$this->end_controls_section();
	}


	/**
	 * Front-end render
	 *
	 * @see lifterlms/templates/course/prerequistes.php
	 * 
	 * @return string
	 */
	protected function render() {

		$settings = $this->get_settings_for_display();

		$output 			      = $settings['output'];
		$is_custom_text 	  = $settings['is_custom_text'] === 'yes';
		$custom_text_track 	= !empty($settings['custom_text_track']) ? $settings['custom_text_track'] : false;
		$custom_text_course = !empty($settings['custom_text_course']) ? $settings['custom_text_course'] : false;

		if( $output === 'full' ) {
			echo do_shortcode("[lifterlms_course_prerequisites]");
		}
		else if( $output === 'text' ) {

			global $post;

			if( $post->post_type !== 'course' ) return '';

			$course = new \LLMS_Course( $post );

			if( $course->has_prerequisite( 'course' ) && ! $course->is_prerequisite_complete( 'course' ) ) {
				
				$prereq_id = $course->get_prerequisite_id( 'course' );

        $text = $is_custom_text && $custom_text_course 
          ? '<p>' . esc_textarea($custom_text_course) . ' %s <p>'
          : __( 'Before starting this course you must complete the required prerequisite course: %s', 'lifterlms' );

				echo sprintf( $text, '<a href="' . \get_permalink($prereq_id) . '">' . \get_the_title( $prereq_id ) . '</a>' );
			}

			if( $course->has_prerequisite( 'course_track' ) && ! $course->is_prerequisite_complete( 'course_track' ) ) {

				$track = new \LLMS_Track( $course->get_prerequisite_id( 'course_track' ) );
				$text = 

        $text = $is_custom_text && $custom_text_track 
          ? '<p>' . esc_textarea($custom_text_track) . ' %s <p>'
          : __( 'Before starting this course you must complete the required prerequisite track: %s', 'lifterlms' );

				echo sprintf( $text, '<a href="' . $track->get_permalink() . '">' . $track->term->name . '</a>' );
			}
		}

	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}

}


