<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

/**
 * TODO: Find a way to get the course activity
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Progress extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-course-progress';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Course Progress', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-skill-bar";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {
    
    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'render_type',
      [
        'label'   => __( 'Output', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'bar',
        'options' => [
          'lifter'  => __( 'LifterLMS render', 'lifter-elements' ),
          'bar'     => __( 'Bar', 'lifter-elements' ),
          'circular'=> __( 'Circle', 'lifter-elements' ),
        ],    
      ]
    );

    $this->add_control(
      'circle_style',
      [
        'label'   => __( 'Circle style', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'lifter',
        'options' => [
          'pie'       => __( 'Pie', 'lifter-elements' ),
          'circular'  => __( 'Circle', 'lifter-elements' )
        ],
        'condition' => ['render_type' => [ 'circular' ]], 
      ]
    );

    $this->add_control(
      'hr', [
        'type' => \Elementor\Controls_Manager::DIVIDER,
      ]
    );

    $this->add_control(
      'show_progress_number',
      [
        'label'   => __( 'Show progress number?', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'no',
        'options' => [
          'no'       => __( 'No', 'lifter-elements' ),
          'percent'  => __( 'Percentage', 'lifter-elements' ),
          'steps'    => __( 'Steps', 'lifter-elements' ),
        ],    
      ]
    );

    $this->add_control(
      'is_button',
      [
        'label'       => __( 'Display continue button', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'On', 'lifter-elements' ),
        'label_off'   => __( 'Off', 'lifter-elements' ),
        'return_value'=> 'yes',
        'default'     => 'no',
        'condition'   => [
          'render_type' => ['lifter'],
        ],  
      ]
    );

    $this->add_control(
      'show_for_visitor',
      [
        'label'       => __( 'Show for non enrolled user?', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'Yes', 'lifter-elements' ),
        'label_off'   => __( 'No', 'lifter-elements' ),
        'return_value'=> 'yes',
        'default'     => 'yes'
      ]
    );

    $this->end_controls_section();
    
    $this->start_controls_section(
      'section_progress_style',
      [
        'label' => __( 'Progress Bar', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_STYLE,
      ]
    );

    $this->add_control(
      'is_full_width',
      [
        'label'   => __( 'Width', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'yes',
        'options' => [
          'yes' => __( 'Full', 'lifter-elements' ),
          'no'  => __( 'Custom', 'lifter-elements' ),
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
          ],
        ]   
      ]
    );

    $this->add_responsive_control(
      'progress_bar_lifter_width',
      [
        'label' => __( 'Max Width', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max'   => 2000,
            'step'  => 1,
          ],
          '%' => [
            'max'   => 100,
            'step'  => 1,
          ],
        ],
        'default' => [
          'unit' => 'px',
          'size' => 480,
        ],
        'size_units'  => [ 'px', '%', 'vw' ],
        'selectors'   => [
          '{{WRAPPER}} .llms-progress .llms-progress-bar' => 'width: {{SIZE}}{{UNIT}}; max-width: {{SIZE}}{{UNIT}}',
          '{{WRAPPER}} .le-course-progress-wrapper.course .llms-course-progress' => 'max-width: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'render_type' => [ 'lifter' ],
        ],
      ]
    );

    $this->add_responsive_control(
      'progress_bar_width',
      [
        'label' => __( 'Max Width', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max'   => 2000,
            'step'  => 1,
          ],
          '%' => [
            'max'   => 100,
            'step'  => 1,
          ],
        ],
        'default' => [
          'unit' => 'px',
          'size' => 480,
        ],
        'size_units'  => [ 'px', '%', 'vw' ],
        'selectors'   => [
          '{{WRAPPER}} .le-course-progress-custom .le-progress' => 'max-width: {{SIZE}}{{UNIT}};',
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
            [
              'name'    => 'is_full_width',
              'operator'=> '!in',
              'value'   => [ 'yes' ],
            ],
          ],
        ]
      ]
    );

    $this->add_responsive_control(
      'progress_bar_width_circular',
      [
        'label' => __( 'Max Width', 'elementor' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px' => [
            'max'   => 2000,
            'step'  => 1,
          ],
        ],
        'render_type' => 'template',
        'default' => [
          'unit' => 'px',
          'size' => 150,
        ],
        'size_units'  => [ 'px', '%', 'vw' ],
        'selectors'   => [
          '{{WRAPPER}} .le-circle-progress-circle, {{WRAPPER}} .le-circle-progress-inner' => 'width: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-circle-progress-inner' => 'height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} #pie' => 'width: auto;',
        ],
        'render_type' => 'template',
        'condition'   => ['render_type' => [ 'circular' ]],  
      ]
    );

    $this->add_control(
      'text_align_circle',
      [
        'label'       => __( 'Alignment', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::CHOOSE,
        'default'     => 'center',
        'prefix_class'=> 'le-course-circle-progress-align-',
        'toggle'      => true,
        'options'     => [
          'left'  => [
            'title'   => __( 'Left', 'lifter-elements' ),
            'icon'    => 'fa fa-align-left',
          ],
          'center'  => [
            'title'   => __( 'Center', 'lifter-elements' ),
            'icon'    => 'fa fa-align-center',
          ],
          'right'   => [
            'title'   => __( 'Right', 'lifter-elements' ),
            'icon'    => 'fa fa-align-right',
          ],
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'circular' ],
            ],
          ],
        ]
      ]
    );

    $this->add_control(
      'text_align',
      [
        'label'       => __( 'Alignment', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::CHOOSE,
        'default'     => 'center',
        'prefix_class'=> 'le-course-progress-align-',
        'toggle'      => true,
        'options'     => [
          'left'    => [
            'title'   => __( 'Left', 'lifter-elements' ),
            'icon'    => 'fa fa-align-left',
          ],
          'center'  => [
            'title'   => __( 'Center', 'lifter-elements' ),
            'icon'    => 'fa fa-align-center',
          ],
          'right'   => [
            'title'   => __( 'Right', 'lifter-elements' ),
            'icon'    => 'fa fa-align-right',
          ],
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
            [
              'name'    => 'is_full_width',
              'operator'=> '!in',
              'value'   => [ 'yes' ],
            ],
          ],
        ]
      ]
    );

    $this->add_responsive_control(
      'progress_lifter_bar_height',
      [
        'label'       => __( 'Height', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SLIDER,
        'render_type' => 'template',
        'size_units'  => [ 'px', 'vh', 'em' ],
        'range' => [
          'px' => [
            'max'   => 100,
            'step'  => 1,
          ],
          '%' => [
            'max'   => 100,
            'step'  => 1,
          ],
        ],
        'default' => [
          'unit' => 'em',
          'size' => '.4',
        ],
        'selectors'   => [
          '{{WRAPPER}} .llms-progress .llms-progress-bar' => 'height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-wrapper.course .llms-course-continue-button' => 'top: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'render_type' => [ 'lifter' ],
        ],  
      ]
    );

    $this->add_responsive_control(
      'progress_bar_height',
      [
        'label'       => __( 'Height', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SLIDER,
        'render_type' => 'template',
        'size_units'  => [ 'px', 'vh', 'em' ],
        'range'       => [
          'px' => [
            'max'   => 100,
            'step'  => 1,
          ],
          '%' => [
            'max'   => 100,
            'step'  => 1,
          ],
        ],
        'default' => [
          'unit' => 'px',
          'size' => '20',
        ],
        'selectors'   => [
          '{{WRAPPER}} .le-progress-inner' => 'height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-overlay .le-progress-number' => 'line-height: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-progress .le-progress-number' => 'line-height: {{SIZE}}{{UNIT}};',
        ],
        'condition' => [
          'render_type' => [ 'bar' ],
        ],  
      ]
    );

    $this->add_control(
      'hr5', [
        'type'      => \Elementor\Controls_Manager::DIVIDER,
        'condition' => [ 'render_type' => [ 'bar' ] ],
      ]
    );

    $this->add_control(
      'progress_bg_bar_color',
      [
        'label'     => __( 'Background Color', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'default'   => "#f1f2f1",
        'selectors' => [
          '{{WRAPPER}} .llms-progress .llms-progress-bar' => 'background-color: {{VALUE}}',
          '{{WRAPPER}} .le-circle-progress-circle .le-circle-progress-inner' => 'background-color: {{VALUE}}',
          '{{WRAPPER}} .le-progress-inner' => 'background-color: {{VALUE}}',
        ],
      ]
    );

    $this->add_control(
      'cicular_inner_bg_color',
      [
        'label'     => __( 'Background Color (Inner)', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'default'   => "white",
        'selectors' => [
          '{{WRAPPER}} .le-circle-progress .le-circle-progress-number' => 'background-color: {{VALUE}}',
        ],
        'condition' => [ 'render_type' => [ 'circular' ] ], 
      ]
    );


    $this->add_control(
      'circle_color_type',
      [
        'label'   => __( 'Color style', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'gradient',
        'options' => [
          'solid'   => __( 'Solid', 'lifter-elements' ),
          'gradient'=> __( 'Gradient', 'lifter-elements' ),
        ],
        'condition' => [ 'render_type' => [ 'circular' ] ], 
      ]
    );

    $this->add_control(
      'progress_bar_color',
      [
        'label'     => __( 'Progress Color', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'default'   => "#2295ff",
        'selectors' => [
          '{{WRAPPER}} .llms-progress .progress-bar-complete' => 'background-color: {{VALUE}}',
          '{{WRAPPER}} .le-progress-bar .le-progress-value' => 'background-color: {{VALUE}}',
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'lifter', 'bar' ],
            ],
          ],
        ]
      ]
    );

    $this->add_control(
      'progress_circle_color',
      [
        'label'       => __( 'Progress Color', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::COLOR,
        'default'     => "#ff3c00",
        'render_type' => 'template',
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'circular' ],
            ],
          ],
        ]
      ]
    );

    $this->add_control(
      'progress_circle_color_2',
      [
        'label'       => __( 'Progress Color 2', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::COLOR,
        'render_type' => 'template',
        'default'     => "#58395b",
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'circular' ],
            ],
            [
              'name'    => 'circle_color_type',
              'operator'=> '!in',
              'value'   => [ 'solid' ],
            ],
          ],
        ]
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Border::get_type(),
      [
        'name'    => 'progress_bar_border',
        'selector'=> '{{WRAPPER}} .llms-progress .progress-bar, {{WRAPPER}} .le-progress-bar .le-progress-inner, {{WRAPPER}} .le-circle-progress-circle .le-circle-progress-inner', 
      ]
    );

    $this->add_responsive_control(
      'border_radius',
      [
        'label'     => __( 'Border Radius', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::DIMENSIONS,
        'size_units'=> [ 'px', '%', 'em' ],
        'default'   => [
          'unit'    => 'px',
          "top"     => 30,
          "right"   => 30,
          "bottom"  => 30,
          "left"    => 30,
          "isLinked"=> true,
        ],
        'selectors'   => [
          '{{WRAPPER}} .llms-progress .llms-progress-bar' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          '{{WRAPPER}} .llms-progress .llms-progress-bar .progress-bar-complete' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          '{{WRAPPER}} .le-progress-bar .le-progress-inner' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
          '{{WRAPPER}} .le-progress-value' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}}',
        ],
        'condition' => [
          'render_type' => [ 'lifter', 'bar' ],
        ],  
      ]
    );

    $this->add_control(
      'hr7', [
        'type'      => \Elementor\Controls_Manager::DIVIDER,
        'condition' => [ 'render_type' => [ 'bar' ] ],
      ]
    );

    $this->end_controls_section();

    $this->start_controls_section(
      'section_number_styles',
      [
        'label'     => __( 'Number', 'lifter-elements' ),
        'tab'       => \Elementor\Controls_Manager::TAB_STYLE,
        'condition' => [ 'render_type' => [ 'bar', 'circular' ] ],
      ]
    );

    $this->add_control(
      'number_vertical',
      [
        'label'       => __( 'Position', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SELECT,
        'default'     => 'inline',
        'render_type' => 'template',
        'options' => [
          'inline'  => __( 'Inline', 'lifter-elements' ),
          'overlay' => __( 'Overlay', 'lifter-elements' ),
          'follow'  => __( 'Follow Progress', 'lifter-elements' ),
          'above'   => __( 'Above', 'lifter-elements' ),
          'below'   => __( 'Below', 'lifter-elements' ),
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
          ],
        ],
      ]
    );

    $this->add_control(
      'number_horizontal',
      [
        'label'   => __( 'Alignment', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::CHOOSE,
        'options' => [
          'left'    => [
            'title'   => __( 'Left', 'lifter-elements' ),
            'icon'    => 'fa fa-align-left',
          ],
          'center'  => [
            'title'     => __( 'Center', 'lifter-elements' ),
            'icon'      => 'fa fa-align-center',
            'condition' => [ 'number_vertical' => [ 'above' ] ],
          ],
          'right'   => [
            'title'   => __( 'Right', 'lifter-elements' ),
            'icon'    => 'fa fa-align-right',
          ],
        ],
        'default'     => 'center',
        'render_type' => 'template',
        'prefix_class'=> 'le-course-progress-number-',
        'toggle'      => true,
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
            [
              'name'    => 'number_vertical',
              'operator'=> '!in',
              'value'   => [ 'overlay', 'inline', 'follow' ],
            ],
          ],
        ]
      ]
    );

    $this->add_control(
      'number_horizontal_center',
      [
        'label'   => __( 'Alignment', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::CHOOSE,
        'options' => [
          'left'    => [
            'title'   => __( 'Left', 'lifter-elements' ),
            'icon'    => 'fa fa-align-left',
          ],
          'right'   => [
            'title'   => __( 'Right', 'lifter-elements' ),
            'icon'    => 'fa fa-align-right',
          ],
        ],
        'default'     => 'left',
        'render_type' => 'template',
        'prefix_class'=> 'le-course-progress-number-',
        'toggle'      => true,
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
            [
              'name'    => 'number_vertical',
              'operator'=> 'in',
              'value'   => [ 'inline' ],
            ],
          ],
        ]
      ]
    );

    $this->add_responsive_control(
      'number_bar_distance',
      [
        'label' => __( 'Spacing', 'lifter-elements' ),
        'type'  => \Elementor\Controls_Manager::SLIDER,
        'range' => [
          'px'  => [
            'max'   => 100,
            'step'  => 1,
          ],
          '%'   => [
            'max'   => 100,
            'step'  => 1,
          ],
          'em'  => [
            'max'   => 3,
            'step'  => 0,
          ],
        ],
        'render_type' => 'template',
        'default'     => [
          'unit' => 'em',
          'size' => '.6',
        ],
        'size_units'  => [ 'px', 'vh', 'em' ],
        'selectors'   => [
          '{{WRAPPER}} .le-course-progress-number-overlay .le-progress-number' => 'padding-left: {{SIZE}}{{UNIT}}; padding-right: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-progress .le-progress-number' => 'padding-left: {{SIZE}}{{UNIT}}; padding-right: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-above .le-progress-number' => 'margin-bottom: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-below .le-progress-number' => 'margin-top: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-inline .le-progress-number-left' => 'margin-right: {{SIZE}}{{UNIT}};',
          '{{WRAPPER}} .le-course-progress-number-inline .le-progress-number-right' => 'margin-left: {{SIZE}}{{UNIT}};',
        ],
        'conditions'  => [
          'terms' => [
            [
              'name'    => 'render_type',
              'operator'=> 'in',
              'value'   => [ 'bar' ],
            ],
            [
              'name'    => 'number_vertical',
              'operator'=> '!in',
              'value'   => [ 'overlay', 'follow' ],
            ],
          ],
        ]
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Typography::get_type(),
      [
        'name'      => 'progress_text_typography',
        'label'     => __( 'Typography', 'plugin-domain' ),
        'selector'  => '{{WRAPPER}} .le-circle-progress-number, {{WRAPPER}} .le-progress-bar .le-progress-value, {{WRAPPER}} .le-progress-number',
        'exclude'   => [ 'line-height' ],
        'condition' => [ 'render_type' => [ 'circular', 'bar' ] ],
      ]
    );

    $this->add_group_control(
      \Elementor\Group_Control_Text_Shadow::get_type(),
      [
        'name'      => 'progress_text_shadow',
        'label'     => __( 'Text Shadow', 'plugin-domain' ),
        'selector'  => '{{WRAPPER}} .le-circle-progress-number, {{WRAPPER}} .le-progress-bar .le-progress-value, {{WRAPPER}} .le-progress-number',
        'condition' => [ 'render_type' => [ 'circular', 'bar' ] ],
      ]
    );

    $this->add_control(
      'progress_text_color',
      [
        'label'     => __( 'Color', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::COLOR,
        'selectors' => [
          '{{WRAPPER}} .le-circle-progress-number, {{WRAPPER}} .le-progress-bar .le-progress-value, {{WRAPPER}} .le-progress-number' => 'color: {{VALUE}}',
        ],
        'condition' => [ 'render_type' => [ 'circular', 'bar' ] ],
      ]
    );

    $this->end_controls_section();
  }


  /**
   * Gets the last activity according to the settings
   * 
   * @param      integer  $number  The number of the meta (1 or 2)
   * 
   * @return     string  The last activity.
   */
  protected function get_last_activity( $number = 1 ) {

    if( !in_array( $number, [1, 2]) ) return;

    $settings = $this->get_settings_for_display();

    // Last activity of the user in the course 
    if( lift\user_course_last_activity() !== 'Never') {
      
      $last_activity  = \DateTime::createFromFormat( 'Y-m-d H:i:s', lift\user_course_last_activity() ); 
      
      return $settings["date_format_" . $number] !== 'custom'
        ? $last_activity->format( $settings["date_format_" . $number] )
        : $date_activity = $last_activity->format( $settings["date_custom_format_" . $number] );    
    }
    
    return 'Never';
  }


  /**
   * Gets the meta text.
   *
   * @param      integer  $number  The number of the meta (1 or 2)
   * 
   * @return    string
   * @return    string
   */
  protected function get_meta_field( $number = 1 ) {
     
    if( !in_array( $number, [1, 2]) ) return;

    $settings = $this->get_settings_for_display();

    return $settings["meta_type_" . $number] === "activity"
      ? $this->get_last_activity( $number )
      : $settings["progress_meta_value_" . $number];
  }


  /**
   * Gets the progress text.
   *
   * @return     string  The progress text.
   */
  protected function get_progress_text() {

    $settings = $this->get_settings_for_display();

    // If no progress in the builder
    if( utils\is_builder() && lift\course_progress_percent() === "0%" ) {
      if( $settings['show_progress_number'] === 'percent' ) return "50%";
      if( $settings['show_progress_number'] === 'steps' ) {
        return round(lift\course_lessons_number()/2, 1) . '/' . lift\course_lessons_number();
      }      
    }
    
    // Text in the center of the circle
    if( $settings['show_progress_number'] === 'percent' ) {
      return lift\course_progress_percent();
    }
    else if( $settings['show_progress_number'] === 'steps' ) {
      return lift\course_lessons_completed_number() . '/' . lift\course_lessons_number();
    }

    return '';
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {
    
    $settings = $this->get_settings_for_display();

    if( $settings['render_type'] === 'lifter' ) {
      $this->render_lifter();
    }
    else if( $settings['render_type'] === 'circular' ) {
      $this->render_circular();
    }
    else if( $settings['render_type'] === 'bar' ) {
      $this->render_bar();
    }
  }


  /**
   * Render a circular bar
   * 
   * @see sources/tangible/progress.scss
   */
  protected function render_bar() {
    
    $settings = $this->get_settings_for_display();

    if( empty(lift\course_progress()) && $settings['show_for_visitor'] !== 'yes' && !utils\is_builder() ) {
      return '';
    }

    // If the current user progression is 0, we display a fake a progression so he can modify easily 
    if( utils\is_builder() && lift\course_progress_percent() === "0%" ) {
      $progress_percent     = '50%';
      $progress_percent_int = 50; 
    }
    else {
      $progress_percent     = lift\course_progress_percent();
      $progress_percent_int = lift\course_progress_percent_int();   
    }

    // If inline, we use the number_horizontal_center instead
    $settings['number_horizontal'] = $settings['number_vertical'] === 'inline' 
      ? $settings['number_horizontal_center'] 
      : $settings['number_horizontal'];
    
    // Number position 
    if( $settings['number_vertical'] === 'follow' ){
      $number_vertical = 'le-course-progress-number-progress';
    }
    else if( $settings['number_vertical'] === 'overlay' ) {

      if( $settings['number_horizontal_center'] === 'left-inline') {
        $number_horizontal  = 'left';
        $number_vertical    = 'le-course-progress-number-inline';
      }
      else if( $settings['number_horizontal_center'] === 'right-inline') {
        $number_horizontal  = 'right';
        $number_vertical    = 'le-course-progress-number-inline';
      }
      else{
        $number_horizontal  = $settings['number_horizontal_center'];
        $number_vertical    = 'le-course-progress-number-overlay';
      }
    }
    else{
      $number_horizontal  = $settings['number_horizontal'];
      $number_vertical    = 'le-course-progress-number-' . $settings['number_vertical'];
    }

    // Width
    $bar_width = $settings['is_full_width'] === 'yes' ? 'le-course-progress-full' : 'le-course-progress-custom';
    
    ?>

    <?php if( utils\is_builder() && lift\course_progress_percent() === "0%" ): ?>
      <?= utils\preview_message( 'progress' ); ?>
    <?php endif; ?>

    <style><?= $this->get_bar_style(); ?></style>

    <div class="le-course-progress-widget">
      <div class="le-course-progress-wrapper <?= $bar_width ?> <?= esc_attr($number_vertical) ?>"> 
        <div class="le-progress le-progress-bar">
          <div class="le-progress-inner">
            <div class="le-progress-value" style="width: <?= $progress_percent ?>">
              <?php if( $settings['number_vertical'] === 'follow' ): ?>
                <div class="le-progress-number">
                  <span><?= $this->get_progress_text() ?></span>
                </div>
              <?php endif; ?>
            </div>
          </div>
          <?php if( $settings['number_vertical'] !== 'follow' ): ?>
            <div class="le-progress-number le-progress-number-<?= esc_attr($number_horizontal) ?>">
              <span><?= $this->get_progress_text() ?></span>
            </div>
          <?php endif; ?>
        </div>
      </div>
    </div>

    <?php
  }


  protected function get_bar_style() {

    $settings = $this->get_settings_for_display();
    $wrapper  = ".elementor-element-" . $this->get_id();

    ob_start(); ?>
    <style>
      <?= $wrapper ?> .le-progress-number {
        text-align: <?= esc_html($settings['text_align']) ?>;
      }
    </style> <?php 

    return ob_get_clean();
  }


  /**
   * Render a circular bar
   * 
   * @see sources/tangible/_progress.scss
   */
  protected function render_circular() {

    $settings = $this->get_settings_for_display();

    if( empty(lift\course_progress()) && $settings['show_for_visitor'] !== 'yes' && !utils\is_builder() ) return '';
    
    $is_pie = $settings['circle_style'] === 'pie';

    // If the current user progression is 0, we display a fake a progression so he can modify easily 
    $progress_percent = utils\is_builder() && lift\course_progress_percent() === '0%' 
      ? '50%' : lift\course_progress_percent();

    $circle_size = $settings['progress_bar_width_circular']['size'];
    $circle_unit = $settings['progress_bar_width_circular']['unit'];

    $progress_number_style = 'top: ' . $circle_size * 0.15 . $circle_unit . ';';      
    $progress_number_style .= 'left: ' . $circle_size * 0.15 . $circle_unit . ';';      
    $progress_number_style .= 'width: ' . $circle_size * 0.7 . $circle_unit . ';';
    $progress_number_style .= 'height: ' . $circle_size * 0.7 . $circle_unit . ';';
    $progress_number_style .= 'line-height: ' . $circle_size * 0.7 . $circle_unit . ';';

    $color_1 = $settings['progress_circle_color'];
    $color_2 = $settings['circle_color_type'] === 'solid' ? $color_1 : $settings['progress_circle_color_2'];

    ?>

    <?php if( utils\is_builder() && lift\course_progress_percent() === "0%" ): ?>
      <?= utils\preview_message( 'progress' ); ?>
    <?php else: ?>
      <!-- Not working in inline style -> overide by js -->
      <style>
        .elementor-element-<?= $this->get_id() ?> .le-progress-inner {
          background-image: conic-gradient(<?= esc_html($color_1) ?> 0, <?= esc_html($color_2) ?> <?= lift\course_progress_percent(); ?>, rgba(0, 0, 0, 0) 0);
        }
      </style>
    <?php endif; ?>
    <div class="le-course-progress-wrapper" data-percent="<?= $progress_percent ?>" data-color-one="<?= esc_attr($color_1) ?>" data-color-two="<?= esc_attr($color_2) ?>" data-size="<?= esc_attr($settings['progress_bar_width_circular']['size']) ?>">
      <div class="le-circle-progress le-circle-progress-circle" <?= $is_pie ? 'id="pie"' : '' ?>>
        <div class="le-circle-progress-inner">
          <div class="le-circle-progress-value" style="width: <?= esc_attr($progress_percent) ?>">
            <div class="le-circle-progress-number" style="<?= esc_attr($progress_number_style) ?>">
              <?= $this->get_progress_text() ?>
            </div>
          </div>
        </div>
      </div>
    <div><?php
  }     
      

  /**
   * Render in the lifter shortcode style
   */
  protected function render_lifter() {
    
    $settings = $this->get_settings_for_display(); ?>
    
    <?php if( !empty(lift\course_progress()) || $settings['show_for_visitor'] === 'yes' ): ?>
        
      <?php if( utils\is_builder() && utils\is_builder() && lift\course_progress_percent() === "0%" ): ?>
        <?= utils\preview_message( 'progress' ); ?>
      <?php endif; ?>

      <?php if( utils\is_builder() && $settings['is_button'] === 'yes' ): ?>
        <?= utils\preview_message( 'continue_button' ) ?> 
      <?php endif; ?>
      
      <div class="course le-course-progress-wrapper">
        <div class="llms-course-progress">
          <div class="llms-progress">
            <?php if( $settings['show_progress_number'] === 'percent' ): ?>
              <div class="progress__indicator">
                <?= lift\course_progress_percent() ?>
              </div>
            <?php elseif( $settings['show_progress_number'] === 'steps' ): ?>
              <div class="progress__indicator">
                <?= lift\course_lessons_completed_number() . ' / ' . lift\course_lessons_number() ?>
              </div>
            <?php endif; ?>
            <?= lift\course_progress_bar() ?>
          </div>
          <?php if( $settings['is_button'] === 'yes' ): ?>
            <?= lift\continue_button() ?>
            <?php if( empty(lift\continue_button()) && utils\is_builder() ): ?>
              <a class="llms-button-primary llms-course-continue-button" href="">Get Started</a>
            <?php endif; ?>
          <?php endif; ?>
        </div>
      </div>
      
    <?php elseif( utils\is_builder() ): ?>
        
        <!-- Preview for the builder -->
        <?= utils\preview_message() ?>
        
      <div class="course le-course-progress-wrapper">
        <div class="llms-course-progress">
          <div class="llms-progress">
            <?php if( $settings['show_progress_number'] === 'percent' ): ?>
              <div class="progress__indicator">0%</div>               
            <?php elseif( $settings['show_progress_number'] === 'steps' ): ?>
              <div class="progress__indicator">
                <?= lift\course_lessons_completed_number() . ' / ' . lift\course_lessons_number() ?>
              </div>      
            <?php endif; ?>
            <div class="llms-progress-bar">
              <div class="progress-bar-complete" data-progress="0%" style="width:0%"></div>
            </div>      
          </div>
          <?php if( $settings['is_button'] === 'yes' ): ?>
            <a class="llms-button-primary llms-course-continue-button" href="">Get Started</a>
          <?php endif; ?>
        </div>
      </div>
      
    <?php endif; ?>
    <?php

  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {}


  /*
   * Enqueue scripts for the current widget
   */
  public function get_script_depends() {
    return ['conic-gradient'];
  }

}


