<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Reviews extends \Elementor\Widget_Base {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-reviews';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Reviews', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-review";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {

		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		
		$this->add_control(
			'is_custom_reviews_text',
			[
				'label' 		  => __( 'Custom reviews text', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default'     => 'no',
			]
		);

		$this->add_control(
			'custom_reviews_text', [
				'label' 		  => __( 'Reviews text', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::TEXT,
				'label_block'	=> true,
				'default'		  =>  __( 'What Others Have Said', 'lifterlms' ),
				'conditions'  => [
					'terms' => [
						[
							'name' 		=> 'is_custom_reviews_text',
							'operator'=> 'in',
							'value' 	=> ['yes'],
						],
					]
				]		
			]
		);

		$this->add_control(
			'is_custom_post_text',
			[
				'label' 		  => __( 'Custom post a reviews text', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'no',
			]
		);

		$this->add_control(
			'custom_post_text', [
				'label' 		  => __( 'After review text', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::TEXT,
				'label_block'	=> true,
				'default'		  =>  __( 'Thank you for your review!', 'lifterlms' ),
				'conditions' 	=> [
					'terms' => [
						[
							'name' 		=> 'is_custom_post_text',
							'operator'=> 'in',
							'value' 	=> ['yes'],
						],
					]
				]		
			]
		);

		$this->end_controls_section();
	}


	/**
	 * Front-end render
	 *
	 * @see  lifterlms/includes/class.llms.review.php
	 * 
	 * @return string
	 */
	protected function render() {

		$settings = $this->get_settings_for_display();

		$is_custom_reviews_text = $settings['is_custom_reviews_text'] === 'yes';
		$is_custom_post_text 	  = $settings['is_custom_post_text'] === 'yes';
		
		$custom_post_text     = $settings['custom_post_text'] !== '' ? esc_html($settings['custom_post_text']) : false;
		$custom_reviews_text  = $settings['custom_reviews_text'] !== '' ? esc_html($settings['custom_reviews_text']) : false;
		
		if( $is_custom_reviews_text && $custom_reviews_text ) {
			add_filter( 'lifterlms_reviews_section_title', function( $value ) use( $custom_reviews_text ) {
				return $custom_reviews_text;
			}, 10, 1 );
		}

		if( $is_custom_post_text && $custom_post_text ) {
			add_filter( 'llms_review_thank_you_text', function( $value ) use( $custom_post_text ) {
				return $custom_post_text;
			}, 10, 1 );
		}

		echo do_shortcode("[lifterlms_course_reviews]");
	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}

}


