<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Syllabus extends \Elementor\Widget_Base {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-syllabus';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Syllabus', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-sitemap";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {
		
		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		
		$this->add_control(
			'is_sections',
			[
				'label' 		  => __( 'Display section title', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->end_controls_section();
	}


	/**
	 * Front-end render
	 *
	 * @see  lifterlms/templates/course/syllabus.php
	 * 
	 * @return string
	 */
	protected function render() {
		
		$settings = $this->get_settings_for_display();

		$is_sections = $settings['is_sections'] === 'yes';

		if( !$is_sections ) {
			add_filter( 'llms_display_outline_section_titles', function( $value ) {
				return false;
			}, 10, 1 );
		}

		// remove dripped lesson view restriction
    $this->remove_lesson_drip_restriction();

		echo do_shortcode("[lifterlms_course_syllabus ]");
	}

  /**
   * Removes drip restriction in the lesson-preview template part
   *
   * @return void
   */
  protected function remove_lesson_drip_restriction() {

    /**
     *  Fires before lifterlms template part
     *
     *  @see in lifterlms\includes\functions\llms.functions.template.php
     *
     *  do_action( 'lifterlms_before_template_part', $template_name, $template_path, $located, $args );
     */
    add_action( 'lifterlms_before_template_part', function( $template_name, $template_path, $located, $args ){

      $lesson = isset( $args[ 'lesson' ] ) ? $args[ 'lesson' ] : null;
      if ( $lesson && $lesson instanceof \LLMS_Lesson ) {

        /**
         * Adds action to remove drip restriction
         *
         * @see dripped_lesson_not_restricted( $lesson = null ) method in inc/Elementor.php
         */
        do_action( 'remove_lesson_preview_drip_restriction', $lesson );
      }
    }, 10, 4);
  }

	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}

}


