<?php

namespace Tangible\LifterElements\Widgets\Courses;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;
use Tangible\WordpressVideo\Video as WordpressVideo;

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Video extends \Elementor\Widget_Base {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-course-video';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Course Video', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-youtube";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return ['lifter-elements'];
	}


	/**
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	protected function register_controls() {
		
		$this->start_controls_section(
			'content_section',
			[
				'label' => __( 'Dimensions', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		
		$this->add_control(
			'is_auto_size',
			[
				'label' 		  => __( 'Dimension auto', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->add_control(
			'ratio', [
				'label' 	=> __( 'Video ratio', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> '16:9',
				'options'	=> [
					'3:2'		  => 'Classic Film 3:2',
					'4:3'		  => 'Standard, Monitor 4:3',
					'16:9'	  => 'HD Video 16:9',
					'21:9'	  => 'Cinemascope 21:9',
				],
				'conditions' 	=> [
					'terms' 	=> [
						[
							'name' 		=> 'is_auto_size',
							'operator'=> '!in',
							'value' 	=> [
								'yes'
							],
						],
					]
				],			
			]
		);

		$this->add_control(
			'width', [
				'label' 		=> __( 'Width', 'lifter-elements' ),
				'type' 			=> \Elementor\Controls_Manager::NUMBER,
				'conditions'=> [
					'terms' 	=> [
						[
							'name' 		=> 'is_auto_size',
							'operator'=> '!in',
							'value' 	=> ['yes'],
						],
					]
				],
				'default' => 640,		
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'content_behavior',
			[
				'label' => __( 'Options', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);
		  
    /**
     * This setting is only aivalable when the advanced video add-on
     * is enable
     */
    if( lift\is_extension( 'advanced-videos' ) ) {
      $this->add_control(
        'is_advanced_video_priority',
        [
          'label'       => __( 'Prioritize the options set in the advanced video plugin', 'lifter-elements' ),
          'type'        => \Elementor\Controls_Manager::SWITCHER,
          'label_on'    => __( 'Yes', 'lifter-elements' ),
          'label_off'   => __( 'No', 'lifter-elements' ),
          'return_value'=> 'yes',
          'default'     => 'yes',
        ]
      ); 
    }
    else {
      $this->add_control(
        'is_advanced_video_priority',
        [
          'type'        => \Elementor\Controls_Manager::HIDDEN,
          'default'     => 'yes',
        ]
      );
    }

		$this->add_control(
			'important_note',
			[
				'type'	=> \Elementor\Controls_Manager::RAW_HTML,
				'raw' 	=> '<i>' . __( 'The options in this tab are currently only compatible with Youtube, Vimeo, and self hosted videos.', 'lifter-elements' ) . '</i>',
			]
		);

		$this->add_control(
			'important_note_about_vimeo',
			[
				'type' 	=> \Elementor\Controls_Manager::RAW_HTML,
				'raw' 	=> '<i>' .  __( 'With Vimeo, a Plus account (or higher) is required for being able to remove the controls.' , 'lifter-elements' ) . '<a target="_blank" href="https://vimeo.zendesk.com/hc/en-us/articles/360001494447-Using-Player-Parameters">' . __( ' Learn more', 'lifter-elements' ) . '</a></i>',
			]
		);

		$this->add_control(
			'is_controls',
			[
				'label' 		  => __( 'Enable controls', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->add_control(
			'is_muted',
			[
				'label' 		  => __( 'Mute audio', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->add_control(
			'is_autoplay',
			[
				'label' 		  => __( 'Enable autoplay', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$this->end_controls_section();
	}


	/**
	 * Front-end render
	 *
	 * @see  lifterlms/templates/course/syllabus.php
	 * 
	 * @return string
	 */
	protected function render() {

		$video_url = lift\course_video_url();

		$this->render_video( $video_url );
	}


	/**
	 * Render the video (separate method for giving access to classes which extends this one)
	 *
	 * @param      url  $audio_url  The video url
	 */
	protected function render_video( $video_url ) {
		
		$settings = $this->get_settings_for_display();

		$is_auto_size	= $settings['is_auto_size'] === 'yes';
		$is_controls	= $settings['is_controls'] === 'yes';
		$is_muted		  = $settings['is_muted'] === 'yes';
		$is_autoplay	= $settings['is_autoplay'] === 'yes';
		$ratio			  = $settings['ratio'];
		
		$width = (int) $settings['width'];

		switch( $ratio ) {
			case '3:2':
				$height = ( $width / 3 ) * 2;  
				break;
			case '4:3':
				$height = ( $width / 4 ) * 3;  
				break;
			case '16:9':
				$height = ( $width / 16 ) * 9;  
				break;
			case '21:9':
				$height = ( $width / 21 ) * 9;  
				break;
		}
		
		$container_width = $is_auto_size ? '' : 'style="width:' . $width . 'px;height:' . $height . 'px;';
		
		$args = [ 
			'controls' 	=> $is_controls, 
			'autoplay' 	=> $is_autoplay, 
			'muted' 	  => $is_muted, 
		];

		if( !$is_auto_size ) {
			$args['width'] 	= $width;
			$args['height'] = $height;
		}
    
    /**
     * The advanced videos plugin has options similar to this module's options so, if set in the 
     * settings, the plugin's settings will overide the module's settings
     */
    if( 
      lift\is_extension( 'advanced-videos' ) && 
      $settings['is_advanced_video_priority'] === 'yes' &&
      lift\is_video_integration( $video_url ) 
    ) {
      $video_html = lift\advanced_video( $video_url, $args );  
    }
    else {
  		
      // If provider is not supported, will just use the width/height param
  		$video = new WordpressVideo( $video_url, $args ); 
      $video_html = $video->get_output();
    }

    $data_autosize = $is_auto_size ? 'data-autosize="true"' : 'data-autosize="false"';
      
    $class = $this->get_name();
    $class = !$is_controls ? $class . " le-video-no-controls" : $class;

		if( !empty($video_html) ) {
			echo '<div ' . $data_autosize . ' class="' . $class . '" ' . $container_width . '">';
			echo $video_html;
			echo '</div>'; 
		}

		if( empty($video_html) && utils\is_builder() ) {
			
			// Demo video
			$video = new WordpressVideo( 'https://www.youtube.com/watch?v=ScMzIvxBSi4', $args );

			echo utils\preview_message( 'content', 'course' );
			echo '<div ' . $data_autosize . ' class="' . $this->get_name() . '" ' . $container_width . '">';
			echo $video->get_output();
			echo '</div>';
		}

		
		// We need this script for performing the resize when we refresh the view in the builder
    // TODO: move to a separte file
		if( $is_auto_size && utils\is_builder() ): ?>
			<script>
				elementorFrontend.hooks.addAction( 'frontend/element_ready/widget', function( $scope ) {
					
					let d = document
					let videosContainer = d.getElementsByClassName('<?= $this->get_name() ?>')[0]
					
					function lifterElementResizeVideo() {
					 	
					 	let isAutosize = videosContainer.getAttribute('data-autosize')
					 	if(isAutosize === 'true') {
						 	let video = videosContainer.getElementsByTagName('video')[0]
						 	if(video) {
							 	video.removeAttribute('width')
							 	video.removeAttribute('height')	 	
							 	videosContainer.setAttribute('style','height:' + video.offsetHeight + 'px')
						 	}
					 	}
					}
					lifterElementResizeVideo()
				});
			</script>
		<?php endif;
	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}

}


