<?php

namespace Tangible\LifterElements\Widgets\Lessons;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils;
use BetterLifter as lift;

/**
 * 
 * Extends the default Elementor Button widget 
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Content extends \Tangible\LifterElements\Widgets\Courses\Content {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-lesson-content';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Lesson Content', 'lifter-elements' );
	}

  /**
   * Add control to show content for lesson with drip time in the future
   */
  public static function add_controls( $element, $section_id, $args ) {

    // The controls will be added before this section
    if( $section_id !== 'section_style' ) return;

    $element -> start_controls_section(
      'drip_show_content',
      [
        'label' => __( 'Lesson Drip Restriction', 'lifter-elements' ),
        'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $element -> add_control(
      'drip_lesson_content',
      [
        'label'       => __( 'Content To Show', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::WYSIWYG,
        'rows'        => 10,
        'dynamic'     => [ 'active' => true ],
        'default'     => __( 'Lesson content is restricted by Drip time in future.', 'lifter-elements' ),
        'description' => __( 'If the Lesson is restricted by drip date this content will appear instead of the Lesson Content', 'lifter-elements' ),
      ]
    );

    $element->end_controls_section();
  }


	/**
	 * Gets the post content for the render method (@see parent)
	 *
	 * @return     string  The post content.
	 */
	protected function get_post_content() {

		$settings = $this -> get_settings_for_display();
		$lesson = lift\lesson_lifter();

    if( !lift\is_lesson_drip_past( $lesson ) ){
			echo '<div class="le-drip-content-show">' . do_shortcode($settings['drip_lesson_content']) . '</div>';
		} else {
			return lift\lesson_content();
		}
	}

}


