<?php

namespace Tangible\LifterElements\Widgets\Lessons;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;

/**
 * TODO: Extend the elementor navigation widget and give the choice
 * between the LifterLMS render and the Elementor for more custom option
 * 
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Navigation extends \Elementor\Widget_Button {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-lessons-navigation';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Lesson Navigation', 'lifter-elements' );
	}


	/**
	 * Class name of an eicon or font-awesome icon
	 *
	 * @see https://elementor.github.io/elementor-icons/
	 * 
	 * @return string
	 */
	public function get_icon() {
		return "eicon-post-navigation";
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * Add control to the exidting one (from the parent)
	 * 
	 * @see https://developers.elementor.com/elementor-controls/
	 */
	public static function add_controls( $element, $section_id, $args ) {
		
		// The controls will be added before this section
		if( $section_id !== 'section_button' ) return;
		
		$element->start_controls_section(
			'link_type',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$element->add_control(
			'render_type', [
				'label' 	=> __( 'Content', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'custom_button',
				'options'	=> [
					'lifter'			  => __( 'LifterLMS render', 'lifter-elements' ),
					'custom_button'	=> __( 'Custom buttons', 'lifter-elements' ),
				],
			]
		);

		$element->add_control(
			'direction', [
				'label' 	=> __( 'Direction', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'both',
				'options'	=> [
					'previous'=> __( 'Previous', 'lifter-elements' ),
					'next'		=> __( 'Next', 'lifter-elements' ),
					'both'		=> __( 'Both', 'lifter-elements' ),
				]			
			]
		);

		$element->add_control(
			'previous_label', [
				'label' 	  => __( 'Previous label', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::TEXT,
				'default'	  => __( 'Previous Lesson', 'lifterlms' ),
				'dynamic'   => ['active' => true],
				'conditions'=> [
					'terms' => [
						[
							'name' 		=> 'direction',
							'operator'=> '!in',
							'value' 	=> ['next'],
						],
					]
				]			
			]
		);

		$element->add_control(
			'next_label', [
				'label' 	  => __( 'Next label', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::TEXT,
				'default'	  => __( 'Next Lesson', 'lifterlms' ),
				'dynamic' 	=> ['active' => true],
				'conditions'=> [
					'terms' => [
						[
							'name' 		=> 'direction',
							'operator'=> '!in',
							'value' 	=> ['previous'],
						],
					]
				]			
			]
		);

		$element->add_control(
			'is_course_link', [
				'label' 		  => __( 'Link to parent course if no item?', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
			]
		);

		$element->add_control(
			'course_label', [
				'label' 	  => __( 'Course label', 'lifter-elements' ),
				'type' 		  => \Elementor\Controls_Manager::TEXT,
				'default'	  => __( 'Back to course', 'lifterlms' ),
				'dynamic' 	=> ['active' => true],
				'conditions'=> [
					'terms' => [
						[
							'name' 		=> 'is_course_link',
							'operator'=> 'in',
							'value' 	=> ['yes'],
						],
					]
				]			
			]
		);

		$element->add_control(
			'is_lesson_title', [
				'label' 		  => __( 'Show the lesson name?', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on' 		=> __( 'Yes', 'lifter-elements' ),
				'label_off' 	=> __( 'No', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default' 		=> 'yes',
				'conditions'  => [
					'terms' => [
						[
							'name' 		=> 'render_type',
							'operator'=> 'in',
							'value' 	=> ['lifter'],
						],
					]
				]	
			]
		);

		$element->end_controls_section();
	}
	

	/**
	 * Modify control defined in the parent class
	 * 
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 * 
	 * @param      obj     $element     The element
	 * @param      string  $section_id  The section identifier
	 * @param      array   $args        The arguments
	 */
	public static function modify_controls( $element, $section_id, $args ) {

		// Hide the controls if it's not a custom button 
		if( !in_array($section_id, ['button_content', 'link_type']) ) {

			$control = $element->get_controls( $section_id );
			$control['conditions'] = [
				'terms' => [
					[
						'name' 		=> 'render_type',
						'operator'=> 'in',
						'value' 	=> ['custom_button'],
					],
				]
			];

			$element->update_control( $section_id, $control );
		}

		if( $section_id !== 'section_button' ) return;

		// We hide the link control, we will set it according to the access plan ID 
		utils\hide_control( $element, 'link' );
		utils\hide_control( $element, 'text' );
		utils\hide_control( $element, 'button_css_id' );
	}


	/**
	 * Front-end render
	 * 
	 * @return string
	 */
	protected function render() {
		
		$settings = $this->get_settings_for_display();

		if( $settings['render_type'] === 'lifter' ) {
			$this->lifter_render();
    }
		else if( $settings['render_type'] === 'custom_button' ) {
			$this->button_render();
		}
	}


	/**
	 * Render if we use Elementor button for the navigation
	 * We use the render method from the parent (button widget)
	 * for both previous and next button 
	 */
	protected function button_render() {

		$settings = $this->get_settings_for_display();
		$strict = $settings['is_course_link'] === 'yes';

		// Eleemntor widget button HTML
		ob_start();	
		parent::render();
		$button = ob_get_clean();
		
		$prev_id      = lift\prev_lesson_id();
		$prev_button 	= utils\change_button_text( $button, $settings['previous_label'] );
		$prev_button 	= utils\modify_link( $prev_button, lift\lesson_previous_url() );
		
		$next_id      = lift\next_lesson_id();
		$next_button 	= utils\change_button_text( $button, $settings['next_label'] );
		$next_button 	= utils\modify_link( $next_button, lift\lesson_next_url() );

		if( !$next_id || !$prev_id  ) {
			$course_id     = lift\course_id();
			$course_button = utils\change_button_text( $button, $settings['course_label'] );
			$course_button = utils\modify_link( $course_button, lift\course_url() );
		}

		
		if( $settings['direction'] === 'both' ): ?>
			<nav class="llms-course-navigation le-course-navigation">
		<?php endif; ?>
			
			<?php if( in_array($settings['direction'], ['previous', 'both']) && $prev_id ): ?>
				<div class="llms-course-nav llms-prev-lesson le-course-nav le-prev-lesson">
					<?php if( lift\can_access_lesson( $prev_id ) ): ?>
						<?= isset($prev_button) ? $prev_button : '' ?>
					<?php endif; ?>
				</div>
			<?php endif; ?>
			
			<?php if( (!$prev_id || !$next_id) && !$strict ): ?>
				<?php if( $settings['direction'] === 'both' || (!$prev_id && $settings['direction'] === 'previous') || (!$next_id && $settings['direction'] === 'next') ): ?>
					<div class="llms-course-nav llms-back-to-course le-course-nav le-back-to-course">
						<?= isset($course_button) ? $course_button : '' ?>
					</div>
				<?php endif; ?>
			<?php endif; ?>

			<?php if( in_array($settings['direction'], ['next', 'both']) && $next_id ): ?>
				<div class="llms-course-nav llms-next-lesson le-course-nav le-next-lesson">
					<?php if( lift\can_access_lesson( $next_id ) ): ?>
						<?= isset($next_button) ? $next_button : '' ?>
					<?php endif; ?>
				</div>
			<?php endif; ?>
	

		<?php if( $settings['direction'] === 'both' ): ?>
			</nav>
		<?php endif; ?>

		<?php
	}


	/**
	 * Render if we use the LifterLMS default render for the navigation
	 */
	protected function lifter_render() {

		$settings = $this->get_settings_for_display();
		$strict = $settings['is_course_link'] === 'yes';

		if( $settings['direction'] === 'both' ) {
			?><nav class="llms-course-navigation"><?php
		}

		$args = array(
			'previous_label' 	=> $settings['previous_label'], 
			'is_lesson_title'	=> $settings['is_lesson_title'] === "yes",
			'next_label'		  => $settings['next_label'],
			'course_label'		=> $settings['course_label']
		);

		if( in_array($settings['direction'], ['previous', 'both']) ) {
			echo lift\lesson_navigation_previous( $strict, $args );
		}
		
		if( in_array($settings['direction'], ['next', 'both']) ) {
			echo lift\lesson_navigation_next( $strict, $args );
		}

		if( $settings['direction'] === 'both' ) {
			?></nav><?php
		}
	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {}
  
}



