<?php

namespace Tangible\LifterElements\Widgets\Memberships;

defined( 'ABSPATH' ) or die();

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Memberships extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-membership-memberships';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Memberships List', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return " eicon-product-rating";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * Return an array with all the categories for the select control
   * 
   * @return array
   */
  protected function get_membership_categories() {

    $terms = get_terms([
      'taxonomy'    => 'membership_cat',
      'hide_empty'  => false,
    ]);

    $response = ['' => __( 'None', 'lifter-elements' )];

    foreach( $terms as $key => $term ) $response[$term->slug] = $term->name;      

    return $response;
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {
    
    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'category', [
        'label'   => __( 'Membership category', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => '',
        'options' => $this->get_membership_categories()     
      ]
    );

    $this->add_control(
      'order', [
        'label'   => __( 'Order', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'ASC',
        'options' => [
          'ASC'   => __( 'Ascending order', 'lifter-elements' ),
          'DESC'  => __( 'Descending order', 'lifter-elements' ),
        ]     
      ]
    );

    $this->add_control(
      'orderby', [
        'label'   => __( 'Order by', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'title',
        'options' => [
          'ID'          => __( 'ID', 'lifter-elements' ),
          'author'      => __( 'Author', 'lifter-elements' ),
          'title'       => __( 'Title', 'lifter-elements' ),
          'name'        => __( 'Name', 'lifter-elements' ),
          'date'        => __( 'Date', 'lifter-elements' ),
          'modified'    => __( 'Last modification', 'lifter-elements' ),
          'rand'        => __( 'Random', 'lifter-elements' ),
          'menu_order'  => __( 'Menu order', 'lifter-elements' ),
        ]     
      ]
    );

    $this->end_controls_section();
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {

    $settings = $this->get_settings_for_display();

    $category       = empty($settings['category']) ? '' : 'category="' . esc_attr($settings['category']) . '"';
    $order          = esc_attr($settings['order']);
    $orderby        = esc_attr($settings['orderby']);
    $posts_per_page = isset($settings['posts_per_page']) ? (int) $settings['posts_per_page'] : '';

    echo do_shortcode('[lifterlms_memberships ' . $category . ' "order="' . $order . '" orderby="' . $orderby . '" posts_per_page="' . $posts_per_page . '"]');
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {}

}


