<?php

namespace Tangible\LifterElements\Widgets\Plans;

defined( 'ABSPATH' ) or die();

use BetterLifter as lift;
use Tangible\LifterElements\Utils as utils;

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class AccessPlanButton extends \Elementor\Widget_Button {


	/**
	 * Widget slug
	 * 
	 * @return string
	 */
	public function get_name() {
		return 'le-plan-access-plan-button';
	}


	/**
	 * Widget name
	 * 
	 * @return string
	 */
	public function get_title() {
		return __( 'Access Plan Button', 'lifter-elements' );
	}


	/**
	 * @see https://developers.elementor.com/widget-categories/
	 * 
	 * @return array
	 */
	public function get_categories() {
		return array( 'lifter-elements' );
	}


	/**
	 * Return an array with all the categories for the select control
	 * 
	 * @return array
	 */
	public function get_access_plans() {

		$access_plans = lift\plans();

		$response = [];
		if( empty($access_plans) ) return $response;

    foreach( $access_plans as $key => $access_plan ) {
      $response[$access_plan->ID] = $access_plan->post_title;
    }

		return $response;
	}


	/**
	 * Adds before the existing controls 
	 *
	 * @see https://code.elementor.com/php-hooks/#elementorelementbefore_section_start
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 * 
	 * @param      obj     $element     The element
	 * @param      string  $section_id  The section identifier
	 * @param      array   $args        The arguments
	 */
	public static function add_controls( $element, $section_id, $args ) {
		
		if( $section_id !== 'section_button' ) return;
		
		$element->start_controls_section(
			'link_type',
			[
				'label' => __( 'Content', 'lifter-elements' ),
				'tab' 	=> \Elementor\Controls_Manager::TAB_CONTENT,
			]
		);

		$element->add_control(
			'access_plan', [
				'label' 	=> __( 'Access plan', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> '',
				'options'	=> $element->get_access_plans()			
			]
		);

		$element->add_control(
			'render_type', [
				'label' 	=> __( 'Render', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'custom_button',
				'options'	=> [
					'lifter'			    => __( 'LifterLMS render', 'lifter-elements' ),
					'custom_button'		=> __( 'Custom button', 'lifter-elements' ),
				],
			]
		);

		$element->end_controls_section();

		$element->start_controls_section(
			'button_lifter',
			[
				'label'     => __( 'Button', 'lifter-elements' ),
				'tab'       => \Elementor\Controls_Manager::TAB_CONTENT,
				'conditions'=> [
					'terms' 	=> [
						[
							'name'     => 'render_type',
							'operator' => 'in',
							'value'    => ['lifter'],
						],
					]
				]
			]
		);

		$element->add_control(
			'is_custom_text',
			[
				'label'       => __( 'Use custom text', 'lifter-elements' ),
				'type' 			  => \Elementor\Controls_Manager::SWITCHER,
				'label_on'    => __( 'On', 'lifter-elements' ),
				'label_off'   => __( 'Off', 'lifter-elements' ),
				'return_value'=> 'yes',
				'default'     => 'no'
			]
		);

		$element->add_control(
			'custom_text', [
				'label'     => __( 'Custom text', 'lifter-elements' ),
				'type'      => \Elementor\Controls_Manager::TEXT,
				'default'	  => '',
				'conditions'=> [
					'terms' => [
						[
							'name'     => 'is_custom_text',
							'operator' => 'in',
							'value'    => ['yes'],
						],
					]
				]		
			]
		);

		$element->add_control(
			'lifter_classes', [
				'label'       => __( 'CSS Classes', 'lifter-elements' ), 
				'type'        => \Elementor\Controls_Manager::TEXT,
				'default'     => 'Join',
				'placeholder'	=> 'my-class1 my-class2...',
				'description'	=> __( 'Add your custom class WITHOUT the dot. e.g: my-class. You can add multiple classes by by adding a spaced between them: my-class1 myclass2.', 'lifter-elements' )					
			]
		);

		$element->add_control(
			'lifter_size', [
				'label' 	=> __( 'Size', 'lifter-elements' ),
				'type' 		=> \Elementor\Controls_Manager::SELECT,
				'default'	=> 'default',
				'options'	=> [
					'default'  => __( 'Default', 'lifter-elements' ),
					'small'    => __( 'Small', 'lifter-elements' ),
					'large'    => __( 'Large', 'lifter-elements' ),
				]
			]
		);

		$element->add_control(
			'lifter_type', [
				'label'       => __( 'Type', 'lifter-elements' ),
				'type'        => \Elementor\Controls_Manager::SELECT,
				'description' => __( 'Your theme and/or custom CSS may alter the colors defined by these styles.', 'lifter-elements' ),
				'default'     => 'primary',
				'options'	    => [
					'primary'  => __( 'Primary', 'lifter-elements' ),
					'secondary'=> __( 'Secondary', 'lifter-elements' ),
					'action' 	 => __( 'Action', 'lifter-elements' ),
					'danger'   => __( 'Danger', 'lifter-elements' ),
				]		
			]
		);

		$element->end_controls_section();
	}


	/**
	 * Modify control defined in the parent class
	 * 
	 * @see inc/Elementor.php in the update_controls_widgets() method
	 * 
	 * @param      object  $element     The element
	 * @param      string  $section_id  The section identifier
	 * @param      array   $args        The arguments
	 */
	public static function modify_controls( $element, $section_id, $args ) {

		// Hide the controls if it's not a custom button 
		if( !in_array($section_id, ['button_lifter', 'link_type']) ) {

			$control = $element->get_controls( $section_id );
			$control['conditions'] = [
				'terms' => [
					[
						'name'    => 'render_type',
						'operator'=> 'in',
						'value'   => ['custom_button'],
					],
				]
			];

			$element->update_control( $section_id, $control );
		}

		if( $section_id !== 'section_button' ) return;

		// We hide the link control, we will set it according to the access plan ID 
		utils\hide_control( $element, 'link' );
 		
		$element->set_render();
	}


	/**
	 * Front-end render
	 * 
	 * @return string
	 */
	protected function lifter_render() {

		$settings = $this->get_settings_for_display();

		$access_plan 	= (int) $settings['access_plan'];
		$classes		  = esc_attr($settings['lifter_classes']);
		$size			    = esc_attr($settings['lifter_size']);
		$type			    = esc_attr($settings['lifter_type']);

		$shortcode = '[lifterlms_access_plan_button id="' . $access_plan . '" size="' . $size . '" classes="' . $classes . '" type="' . $type . '"]';
 		
		if( $settings['is_custom_text'] === 'yes' && $settings['custom_text'] !== '' ) {
			$shortcode .= esc_html($settings['custom_text']);
			$shortcode .= '[/lifterlms_access_plan_button]';
		}

		ob_start();
		echo do_shortcode( $shortcode );
		return ob_get_clean();
	}


	/**
	 * Sets the render.
	 *
	 * @param      obj  $widget  The widget
	 */
	public function set_render() {
		
		$widget = &$this;
		
		add_action('elementor/widget/render_content', function( $content, $element ) use( $widget ) {
	  	
  		if( $element->get_name() !== $widget->get_name() ) return $content;
  			
  		$settings = $element->get_settings_for_display();

			/**
			 * If outline_type is set on lifter_element 
			 * we bypass the render defiened in the parent class
			 */
			if( $settings['render_type'] === 'lifter' ) return $element->lifter_render();

			/**
			 * Re-run render with the new URL
			 */
			return "<div class='elementor-widget-button' >" . utils\modify_link( $content, lift\plan_url($settings['access_plan']) ) . '</div>';

		}, 999, 2 );
	}


	/**
	 * Editor output for generating the live preview
	 */
	protected function content_template() {
		$this->set_render();	
	}

}


