<?php

namespace Tangible\LifterElements\Widgets\Plans;

defined( 'ABSPATH' ) or die();

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class PricingTable extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-plan-pricing-table';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Pricing Table', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-price-table";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * Return an array with all the categories for the select control
   * 
   * @return array
   */
  protected function get_membership_categories() {

    $terms = get_terms([
      'taxonomy'    => 'membership_cat',
      'hide_empty'  => false,
    ]);

    $response = ['' => __( 'None', 'lifter-elements' )];

    foreach( $terms as $key => $term ) $response[$term->slug] = $term->name;      

    return $response;
  }


  /**
   * Return an array with all the courses
   * 
   * @return array
   */
  protected function get_courses() {

    $courses = get_posts([
      'post_type'       => 'course',
      'posts_per_page'  => -1,
      'sort_order'      => 'asc',
      'orderby'         => 'title'
    ]);

    $response = [];

    foreach( $courses as $key => $course ) {
      $response[$course->ID] = $course->post_title;
    }

    return $response;
  }


  /**
   * Return an array with all the courses
   * 
   * @return array
   */
  protected function get_memberships() {

    $memberships = get_posts([
      'post_type'       => 'llms_membership',
      'posts_per_page'  => -1,
      'sort_order'      => 'asc',
      'orderby'         => 'title'
    ]);

    $response = [];
    foreach( $memberships as $key => $membership ) {
      $response[$membership->ID] = $membership->post_title;
    }

    return $response;
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {
    
    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'current_course',
      [
        'label'       => __( 'Current course pricing', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'Yes', 'lifter-elements' ),
        'label_off'   => __( 'No', 'lifter-elements' ),
        'return_value'=> 'yes',
        'default'     => 'yes'
      ]
    );

    $this->add_control(
      'type', [
        'label'   => __( 'Type of product', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'memberships',
        'options' => [
          'memberships' => __( 'Memberships', 'lifter-elements' ),
          'courses'     => __( 'Course', 'lifter-elements' ),
        ],
        'conditions' => [
          'terms' => [
            [
              'name'      => 'current_course',
              'operator'  => '!in',
              'value'     => ['yes'],
            ],
          ]
        ]   
      ]
    );

    $this->add_control(
      'memberships', [
        'label'   => __( 'Memberships', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => '',
        'options' => $this->get_memberships(),
        'conditions'  => [
          'terms' => [
            [
              'name'      => 'type',
              'operator'  => 'in',
              'value'     => ['memberships'],
            ],
            [
              'name'      => 'current_course',
              'operator'  => '!in',
              'value'     => ['yes'],
            ]
          ]
        ]   
      ]
    );

    $this->add_control(
      'courses', [
        'label'   => __( 'Courses', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => '',
        'options' => $this->get_courses(),
        'conditions' => [
          'terms' => [
            [
              'name'      => 'type',
              'operator'  => 'in',
              'value'     => ['courses'],
            ],
            [
              'name'      => 'current_course',
              'operator'  => '!in',
              'value'     => ['yes'],
            ]
          ]
        ]   
      ]
    );

    $this->end_controls_section();
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {

    $settings = $this->get_settings_for_display();

    $type = $settings['type'];
    $current_course = $settings['current_course'] === 'yes' ? true : false;

    $membership = (int) $settings['memberships'];
    $course     = (int) $settings['courses'];
    
    if( $current_course === true ) {
      $shortcode = '[lifterlms_pricing_table]';
    }
    else if( $type === 'memberships' ) {
      $shortcode = '[lifterlms_pricing_table product="' . $membership . '"]';
    }
    else{
      $shortcode = '[lifterlms_pricing_table product="' . $course . '"]';
    }

    echo do_shortcode( $shortcode );
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {}

}


