<?php

namespace Tangible\LifterElements\Widgets\Users;

defined( 'ABSPATH' ) or die();

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Achievements extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-user-achievement';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'User Achievements', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-rating";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {
    
    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'count',
      [
        'label'       => __( 'Number of achievements', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::NUMBER,
        'description' => __( 'Leave empty to show all achievements', 'lifter-elements' ),
        'min'         => 1,
        'step'        => 1,
        'default'     => ''
      ]
    );

    $this->add_control(
      'column',
      [
        'label'       => __( 'Number of columns used when listing achievements', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::NUMBER,
        'label_block' => true,
        'min'         => 1,
        'step'        => 1,
        'default'     => 5
      ]
    );

    $this->end_controls_section();  
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {

    $settings = $this->get_settings_for_display();

    $count  = $settings['count'] === '' ? '' : 'count="' . (int) $settings['count'] . '"';
    $column = (int) $settings['column'];

    echo do_shortcode('[lifterlms_my_achievements ' . $count . ' columns="' . $column . '"]');
  }



}


