<?php

namespace Tangible\LifterElements\Widgets\Users;

defined( 'ABSPATH' ) or die();

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Dashboard extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-user-dashboard';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'User Dashboard', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-person";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {
    
    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'is_custom_redirect',
      [
        'label'       => __( 'Define a custom login redirect URL', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'On', 'lifter-elements' ),
        'label_off'   => __( 'Off', 'lifter-elements' ),
        'return_value'=> 'yes',
        'default'     => 'no'
      ]
    );

    $this->add_control(
      'redirect_url', [
        'label'     => __( 'Custom login redirect URL', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::TEXT,
        'default'   => '',
        'conditions'=> [
          'terms' => [
            [
              'name'    => 'is_custom_redirect',
              'operator'=> 'in',
              'value'   => ['yes'],
            ],
          ]
        ]   
      ]
    );

    $this->end_controls_section();  
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {

    $settings = $this->get_settings_for_display();

    $is_custom_redirect = $settings['is_custom_redirect'];
    $redirect_url = esc_url($settings['redirect_url']);

    if( $is_custom_redirect === 'yes') {
      $shortcode = '[lifterlms_my_account login_redirect="' . $redirect_url . '"]';
    }
    else{
      $shortcode = '[lifterlms_my_account]';
    }

    echo do_shortcode( $shortcode );
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {}

}


