<?php

namespace Tangible\LifterElements\Widgets\Users;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils; 

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Login extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-user-login';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Login form', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-lock-user";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {
    
    $this->start_controls_section(
      'content_section',
      [
        'label' => __( 'Content', 'lifter-elements' ),
        'tab'   => \Elementor\Controls_Manager::TAB_CONTENT,
      ]
    );

    $this->add_control(
      'layout', [
        'label'   => __( 'Layout', 'lifter-elements' ),
        'type'    => \Elementor\Controls_Manager::SELECT,
        'default' => 'columns',
        'options' => [
          'columns'   => __( 'Columns', 'lifter-elements' ),
          'stacked'   => __( 'Stacked', 'lifter-elements' ),
        ]   
      ]
    );

    $this->add_control(
      'is_custom_redirect',
      [
        'label'       => __( 'Define a custom redirect URL', 'lifter-elements' ),
        'type'        => \Elementor\Controls_Manager::SWITCHER,
        'label_on'    => __( 'On', 'lifter-elements' ),
        'label_off'   => __( 'Off', 'lifter-elements' ),
        'return_value'=> 'yes',
        'default'     => 'no'
      ]
    );

    $this->add_control(
      'redirect_url', [
        'label'     => __( 'Custom redirect URL', 'lifter-elements' ),
        'type'      => \Elementor\Controls_Manager::TEXT,
        'default'   => '',
        'conditions'=> [
          'terms' => [
            [
              'name'    => 'is_custom_redirect',
              'operator'=> 'in',
              'value'   => ['yes'],
            ],
          ]
        ]   
      ]
    );

    $this->end_controls_section();  
  }


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {

    $settings = $this->get_settings_for_display();

    $layout = $settings['layout'] === 'stacked' ? $settings['layout'] : 'columns';
    $is_custom_redirect = $settings['is_custom_redirect'];
    $redirect_url = esc_url($settings['redirect_url']);
    
    // Final render
    if( !utils\is_builder() ) {
      
      if( $is_custom_redirect === 'yes' ) {
        $shortcode = '[lifterlms_login layout="' . $layout . '" redirect="' . $redirect_url . '"]';
      }
      else{
        $shortcode = '[lifterlms_login layout="' . $layout . '"]';
      }

      echo do_shortcode( $shortcode );
    }
    
    /**
      *  Render in the builder mode
      *  
      *  @see lifterlms/template/global/form-login.php 
      */
    else if( utils\is_builder() ) {

      ?>
      
      <?= utils\preview_message( 'need_unlogged_user' ) ?>
      <div class="col-1 llms-person-login-form-wrapper">
        <form action="" class="llms-login" method="POST">
          <h4 class="llms-form-heading"><?php _e( 'Login', 'lifterlms' ); ?></h4>
          <div class="llms-form-fields">
            <?php do_action( 'lifterlms_login_form_start' ); ?>
            <?php foreach ( \LLMS_Person_Handler::get_login_fields($layout) as $field ): ?>
              <?php llms_form_field( $field ); ?>
            <?php endforeach; ?>      
            <?php do_action( 'lifterlms_login_form_end' ); ?>
          </div>
        </form>
      </div>
      <?php
    }
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {}

}


