<?php

namespace Tangible\LifterElements\Widgets\Users;

defined( 'ABSPATH' ) or die();

use Tangible\LifterElements\Utils as utils; 

/**
 * @see https://developers.elementor.com/creating-a-new-widget/
 */
class Registration extends \Elementor\Widget_Base {


  /**
   * Widget slug
   * 
   * @return string
   */
  public function get_name() {
    return 'le-user-registration';
  }


  /**
   * Widget name
   * 
   * @return string
   */
  public function get_title() {
    return __( 'Registration form', 'lifter-elements' );
  }


  /**
   * Class name of an eicon or font-awesome icon
   *
   * @see https://elementor.github.io/elementor-icons/
   * 
   * @return string
   */
  public function get_icon() {
    return "eicon-lock-user";
  }


  /**
   * @see https://developers.elementor.com/widget-categories/
   * 
   * @return array
   */
  public function get_categories() {
    return array( 'lifter-elements' );
  }


  /**
   * @see https://developers.elementor.com/elementor-controls/
   */
  protected function register_controls() {}


  /**
   * Front-end render
   * 
   * @return string
   */
  protected function render() {
    
    ob_start();
    echo do_shortcode( '[lifterlms_registration]' );
    $content = ob_get_clean();

    if( !empty($content) ) {
      echo $content;
    }
    // Preview is the builder even if user connected
    elseif( utils\is_builder() ){

      echo utils\preview_message( 'need_unlogged_user' );
      
      /**
       * We need to display the template manually (can't bypass the get_current_user_id)
       * 
       * @see lifterlms/templates/global/form-registration.php
       */
      $field_data = isset( $_POST ) ? $_POST : array();
      do_action( 'lifterlms_before_person_register_form' ); ?>
      <div class="col-2 llms-new-person-form-wrapper">
        <h4 class="llms-form-heading"><?php _e( 'Register', 'lifterlms' ); ?></h4>
        <form method="post" class="llms-new-person-form register">
          <?php do_action( 'lifterlms_register_form_start' ); ?>
          <div class="llms-form-fields">
            <?php do_action( 'lifterlms_before_registration_fields' ); ?>
            <?php foreach ( \LLMS_Person_Handler::get_available_fields('registration', $field_data) as $field ): ?>
              <?php llms_form_field( $field ); ?>
            <?php endforeach; ?>
            <?php
              /**
               * llms_registration_privacy
               * @hooked llms_privacy_policy_form_field - 10
               * @hooked llms_agree_to_terms_form_field - 20
               */
              do_action( 'llms_registration_privacy' );
            ?>
            <?php do_action( 'lifterlms_after_registration_fields' ); ?>
          </div>
          <footer class="llms-form-fields">
            <?php do_action( 'lifterlms_before_registration_button' ); ?>
            <?php llms_form_field( array(
              'columns'     => 3,
              'classes'     => 'llms-button-action',
              'id'          => 'llms_register_person',
              'value'       => apply_filters( 'lifterlms_registration_button_text', __( 'Register', 'lifterlms' ) ),
              'last_column' => true,
              'required'    => false,
              'type'        => 'submit',
            ) ); ?>
            <?php do_action( 'lifterlms_after_registration_button' ); ?>
          </footer>
          <?php do_action( 'lifterlms_register_form_end' ); ?>
        </form>
      </div>
      <?php do_action( 'lifterlms_after_person_register_form' ); 
    }
  }


  /**
   * Editor output for generating the live preview
   */
  protected function content_template() {
    
  }

}


