<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * Return the instance of the memeberships associate with the
 * current course 
 * 
 * @see plugins/lifterlms/includes/models/model.llms.access.plan.php
 * @see plugins/lifterlms/includes/models/model.llms.membership.php
 * @see plugins/lifterlms/includes/models/model.llms.product.php
 * 
 * @param      integer  $course_id  The course identifier
 *
 * @return     array    
 */
function course_memberships( $course_id = 0 ) {

  $course = lift\course_lifter( $course_id );
  
  if( empty($course) || !is_object($course) ) return [];
  
  $memberships = lift\memberships();

  if( empty($memberships) ) return [];

  $response = [];
  
  foreach( $memberships as $membership ) {
    $membership_lifter = new \LLMS_Membership( $membership->ID ); 
    $membership_courses = $membership_lifter->get_auto_enroll_courses();
    
    if( in_array( lift\course_id( $course_id ), $membership_courses ) )
      array_push( $response, $membership_lifter );
  }

  return $response;
}


/**
 * Return the instance of the plans associate with a memebership 
 * associate with the current course 
 * 
 * @see plugins/lifterlms/includes/models/model.llms.access.plan.php
 * @see plugins/lifterlms/includes/models/model.llms.product.php
 * @see plugins/lifterlms/includes/models/model.llms.membership.php
 * 
 * @param      integer  $course_id  The course identifier
 *
 * @return     array    
 */
function course_membership_plans( $course_id = 0 ) {
  
  $memberships = lift\course_memberships( $course_id );

  $response = [];

  foreach( $memberships as $membership ) {
    $plans = membership_plans( $membership->get( 'id' ) );
    $response = array_merge( $response, $plans );
  }

  return $response;
}
