<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * Return all the enable integrations instance 
 *
 * @return     array  
 */
function video_intergations() {

  // Get the enable intergations' ids
  $integrations = LLMS_AV()->get_available_providers();
  $response =  array();

  foreach( $integrations as $integration_id ) {
    $integration = LLMS_AV()->get_integration( $integration_id );
    array_push($response, $integration);
  }

  return $response;
}


/**
 * Determines if there is video a integration set in the addon for
 * a given video url.
 *
 * @param      string   $url    The url
 *
 * @return     boolean  True if the is a video integration, False otherwise.
 */
function is_video_integration( $url ) {
    
  // Get the default wordpress embed 
  $wp_html = \wp_oembed_get( $url );

  foreach( lift\video_intergations() as $integration ) {
    $av_html = $integration->modify_embed_html( $wp_html );
    if( $wp_html !== $av_html ) return true; 
  }

  return false;
}


/**
 * Return the advanced video intergarion html
 *
 * @param      string  $url    The url
 * @param      array   $args   The arguments
 *
 * @return     string
 */
function advanced_video( $url, $args = [] ) {

  if( !lift\is_video_integration($url) ) return '';

  // Get the default wordpress embed 
  $wp_html = \wp_oembed_get( $url, $args );

  foreach( lift\video_intergations() as $integration ) {
    
    if( method_exists($integration, 'modify_oembed_fetch_url') ) {
      
      add_filter( 'llms_av_allow_oembed_filtering', '__return_true', 10 );
      $modified_url = $integration->modify_oembed_fetch_url( $url );

      $av_html = \wp_oembed_get( $modified_url, $args );
      $av_html = $integration->modify_embed_html( $av_html );
      
      remove_filter( 'llms_av_allow_oembed_filtering', '__return_true', 10 );
      
      if( $wp_html !== $av_html ) return $av_html; 
      continue;
    }
    
    $av_html = $integration->modify_embed_html( $wp_html );
    if( $wp_html !== $av_html ) return $av_html; 
  }

  return '';
}
