<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift; 


/**
 * Determines if extension is activated.
 *
 * @param      string   $name  The extension name
 *
 * @return     boolean  True if extension active, False otherwise.
 */
function is_extension( $name ) {

  if( empty($name) ) return false;

  /**
   * First index = addon's id in the lifterlms system, 
   * Second = constant for determine if the plugin is activated or not
   * Third = ID (or IDs) defined into the Integration classes
   */
  $extensions = [
    'woocommerce'     => ['lifterlms-com-woocommerce-extension','LLMS_WC_PLUGIN_FILE', 'woocommerce'],
    'advanced-videos' => ['lifterlms-com-advanced-videos', 'LLMS_ADVANCED_VIDEOS_VERSION', ['av_wistia', 'av_vimeo', 'av_youtube']],
    'assignments'     => ['lifterlms-com-lifterlms-assignments', 'LLMS_ASSIGNMENTS_PLUGIN_FILE', false],
    'private-area'    => ['lifterlms-com-private-areas', 'LLMS_PRIVATE_AREAS_PLUGIN_FILE', 'private_areas', ],
    'gravity-forms'   => ['lifterlms-com-gravity-forms', 'LLMS_GF_PLUGIN_FILE', 'gravityforms', ],
  ];

  if( ! isset($extensions[$name]) ) return false;
  if( ! defined($extensions[$name][1]) ) return false;

  $add_ons = \llms_get_add_ons();
  
  if( ! is_array($add_ons) || ! isset($add_ons['items']) ) {
    return false;
  } 
  
  foreach( $add_ons['items'] as $item ) {
    if( $item['id'] === $extensions[$name][0] ) $extension = \llms_get_add_on( $item['id'] );
  }

  $status = $extension->is_installed() && $extension->is_installable() && $extension->is_active(); 
  if( !$status || $extensions[$name][2] === false ) return $status;

  $status = false;
  $active_integrations = lift\llms()->integrations()->get_available_integrations();

  if( empty($active_integrations) ) return $status;

  foreach( $active_integrations as $key => $integration ) {
    if( $key === $extensions[$name][2] ) {
      return $integration->is_enabled();
    }
    if( is_array($extensions[$name][2]) && in_array($key, $extensions[$name][2]) ) {
      return $integration->is_enabled();
    } 
  }

  return $status;
} 
 