<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * On a private area single page, the post will be equal to
 * the parent course, so we need to get the post using another 
 * method
 *
 * @see lifterlms-private-areas/includes/class-llms-pa-endpoints.php do_area_content() 
 * 
 * @return     object
 */
function private_area_post() {
  $posts = lift\private_area_posts();
  if( empty($posts) ) return false;
  return isset($posts->posts[0]) ? $posts->posts[0] : false;
}


/**
 * Get current private areas posts
 * 
 * @return object 
 */
function private_area_posts() {
  if( \is_llms_private_area() ) {

    global $wp_query;
    $slug = $wp_query->query_vars[ 'llms_private_area' ];

    // single post.
    if ( $slug && 0 === strpos( $slug, '/' ) ) {
      return \llms_pa_get_posts(
        null,
        null,
        array(
          'pa_slug' => sanitize_text_field( ltrim( $slug, '/' ) ),
        )
      );
    }
  }
  return false;
}


/**
 * Return the ID for the current single private area post type
 * 
 * @return     integer
 */
function private_area_id() {
  $post = lift\private_area_post();
  if( empty($post) ) return 0;
  return $post->ID;
}


/**
 * Return the title for the current single private area post type
 *
 * @return     string  
 */
function private_area_title() {
  $post = lift\private_area_post();
  if( empty($post) ) return '';
  return $post->post_title; 
}


/**
 * Return the content for the current single private area post type
 *
 * @return     string  
 */
function private_area_content() {
  $post = lift\private_area_post();
  if( empty($post) ) return '';
  return $post->post_content; 
}


/**
 * Return the navigation HTML for the current single private area post type
 * 
 * @return     string  (HTML)
 */
function private_area_navigation() {
  $posts = lift\private_area_posts();

  if( empty($posts) ) return '';
  
  // Not the cleanest way but we need to change the value of get_the_ID() in the template
  global $post;
  $save_post = $post;
  $post = lift\course();

  ob_start();
  \llms_pa_template_private_area_title( $posts );
  \llms_pa_template_private_area_return( $posts );
  $content = ob_get_clean();

  // Restaure post value
  $post = $save_post;

  return $content;
}


/**
 * Return the date of the private area single
 *
 * @param      string  $format  The format
 *
 * @return     string
 */
function private_area_date( $format = '' ) {
  $post = lift\private_area_post();
  if( empty($post) ) return '';
  return \get_the_date( $format, $post );
}


/**
 * Return the author ID for the current single private area post type
 *
 * @return     integer  
 */
function private_area_author_id() {
  $post = lift\private_area_post();
  if( empty($post) ) return '';
  return $post->post_author;
}


/**
 * Return the author HTML for the current single private area post type
 *
 * @return     string  (HTML)
 */
function private_area_author( $args = [] ) {

  $author_id = lift\private_area_author_id();
  if( empty($author_id) ) return '';

  $default_args = array(
    'user_id' => $author_id
  );

  $args = array_merge( $default_args, $args );
  
  return \llms_get_author( $args );
}


/**
 * Return the comments HTML for the current single private area post type
 *
 * @return     string  (HTML)
 */
function private_area_comments() {

  $private_post_id = lift\private_area_id();

  ob_start(); ?>
    <article class="llms-pa-post" >
      <?php \llms_pa_template_private_area_discussion_list( $private_post_id ) ?>
    </article>
  <?php $content = ob_get_clean();

  return $content;
}


/**
 * Return the comments form HTML for the current single private area post type
 *
 * @return     string  (HTML)
 */
function private_area_comment_form() {

  // Not the cleanest way but we need to change the value of get_the_ID() in the template
  global $post;
  $save_post = $post;
  $post = lift\private_area_post();

  ob_start(); ?>
    <article class="llms-pa-post" >
      <?php \llms_pa_template_private_area_discussion_form() ?>
    </article>
  <?php $content = ob_get_clean();

  // Restaure post value
  $post = $save_post;

  return $content;
}


/**
 * Get the course ID related to a private area post ID
 *
 * @see lifterlms-private-areas/includes/admin/class-llms-pa.metabox-pa-student.php
 * 
 * @param      integer   $post_id  The post identifier
 *
 * @return     integer  
 */
function private_area_course_id( $post_id ) {
  return (int) \get_post_meta( $post_id, '_llms_pa_post_id', true );
}
