<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

function lesson_post_type() {
  return 'lesson';
}

function lesson($post_id = 0) {
  $lesson_id = lift\lesson_id($post_id);
  if (!empty($lesson_id)) return get_post($lesson_id);
}

function lesson_lifter($post_id = 0) {
  $lesson_id = lift\lesson_id($post_id);
  if (!empty($lesson_id)) return new \LLMS_Lesson($lesson_id);
}

function is_lesson($post_id = 0) {
  if (empty($post_id)) $post_id = get_the_id();
  $post = get_post($post_id);
  return !empty($post) && $post->post_type === lift\lesson_post_type();
}

function lesson_id($post_id = 0) {

  if ($post_id instanceof WP_Post) $post_id = $post_id->ID;
  if (empty($post_id)) $post_id = get_the_id();
  
  if (lift\is_lesson($post_id)) return $post_id;

  return 0;
}


/**
 * Extract the text paragraph from the content
 *
 * @see https://github.com/WordPress/gutenberg/issues/6358 
 * 
 * @param      integer  $post_id  The post identifier
 *
 * @return     string
 */
function lesson_content( $post_id = 0 ) {
  
  $lesson_id = lift\lesson_id( $post_id ); 
  if( empty($lesson_id) ) return ''; 

  $lesson = lift\lesson_lifter( $lesson_id );
  if( !lift\is_lesson_drip_past( $lesson ) ) return '';

  // Get the Lifter related Gutenberg block names
  $lifter_blocks = lift\lifter_block_names();
  $post = get_post( $lesson_id ); 
  
  if( has_blocks($post->post_content) ) {
    
    $blocks = parse_blocks( $post->post_content );
    $content = '';

    // Echo only blocks not related to LifterLMS
    foreach( $blocks as $block ) {
      if( !in_array($block['blockName'], $lifter_blocks) ) {
        $content .= \render_block($block);
      }
    }

    /**
     * LifterLMS adds HTML we don't want in 'the_content' filter
     * 
     * @see lifterlms/includes/functions/llms-functions-content.php 
     */
    remove_filter( 'the_content', 'llms_get_post_content' );

    /**
     * We have to use 'the_content' filter for completly display embed-core blocks (youtube, vimeo ...)
     *
     * @see https://github.com/WordPress/gutenberg/issues/14080#issuecomment-567429414
     */
    $content = apply_filters( 'the_content', $content );
    
    add_filter( 'the_content', 'llms_get_post_content' );
  }
  // If there is not block, the content come from the classic editor and we can display as it is
  else{
    $content = $post->post_content;
  }

  return do_shortcode($content);
}
