<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * Return the complete navigation block (lifter output)
 * 
 * @return string 	HTML
 */
function lesson_navigation() {
	
	ob_start();
	\llms_get_template( 'course/lesson-navigation.php' );
	$content = ob_get_clean();
	
	return $content;
}


/**
 * Return the previous lesson ID
 *
 * @param      int  $lesson_id  The lesson identifier
 * 
 * @return     int  $prev_id  The lesson identifier
 */
function prev_lesson_id( $lesson_id = 0 ) {

	$lesson_id =  empty($lesson_id) ? lift\lesson_id() : (int) $lesson_id;
	$lesson = new \LLMS_Lesson( $lesson_id );

	$prev_id = $lesson->get_previous_lesson();
	
	return $prev_id;
}


/**
 * Return the previous lesson ID
 *
 * @param      int  $lesson_id  The lesson identifier
 * 
 * @return     int  $prev_id  The lesson identifier
 */
function next_lesson_id( $lesson_id = 0 ) {

	$lesson_id =  empty($lesson_id) ? lift\lesson_id() : (int) $lesson_id;
	$lesson = new \LLMS_Lesson( $lesson_id );

	$next_id = $lesson->get_next_lesson();

	return $next_id;
}


/**
 * Return the next part of the lifterLMS navigation output
 *
 * @param      	boolean 	$strict (optional) If true, will no show course link
 * @param       array 		$args 	(optional) Output options
 * 
 * @return string 	HTML
 */
function lesson_navigation_next( $strict = false, $args = array() ) {
	
	$content = '';
	$next_id = lift\next_lesson_id();

	$default_args = array(
		'next_label'      => __( 'Next Lesson', 'lifterlms' ),
		'course_label'    => __( 'Back to Course', 'lifterlms' ),
		'is_lesson_title' => true,
	);

	$args = array_merge( $default_args, $args );

	// If there is a next lesson
	if( $next_id ) {
		$can_access = lift\can_access_lesson( $next_id );
		if( !$can_access ) return '';

		ob_start(); ?>
		<div class="llms-course-nav llms-next-lesson">
			<div class="llms-lesson-preview">
				<a class="llms-lesson-link" href="<?= lift\lesson_next_url() ?>">
					<section class="llms-main">
						<h6 class="llms-pre-text"><?= $args['next_label'] ?></h6>
						<?php if( $args['is_lesson_title'] ): ?>
							<h5 class="llms-h5 llms-lesson-title"><?= get_the_title( $next_id ) ?></h5>
						<?php endif; ?>
					</section>
				</a>
			</div>
		</div>
		<?php $content = ob_get_clean();
	}
	else if( !$strict ){
		ob_start(); ?>
		<div class="llms-course-nav llms-back-to-course">
			<div class="llms-lesson-preview">
				<a class="llms-lesson-link" href="<?= lift\course_url() ?>">
					<section class="llms-main">
						<h6 class="llms-pre-text"><?= $args['course_label'] ?></h6>
						<?php if( $args['is_lesson_title'] ): ?>
							<h5 class="llms-h5 llms-lesson-title"><?= lift\course_title() ?></h5>
						<?php endif; ?>
					</section>
				</a>
			</div>
		</div>
		<?php $content = ob_get_clean();
	}

	return $content;
}


/**
 * Return the next part of the lifterLMS navigation output
 *
 * @param      boolean 	$strict (optional) If true, will no show course link
 * @param      array 	$args 	(optional) Output options
 * 
 * @return 		string 	HTML
 */
function lesson_navigation_previous( $strict = false, $args = array() ) {
	
	$content = '';

	$prev_id = lift\prev_lesson_id();

	$default_args = array(
		'previous_label' 	=> __( 'Previous Lesson', 'lifterlms' ),
		'course_label' 		=> __( 'Back to Course', 'lifterlms' ),
		'is_lesson_title' 	=> true,
	);

	$args = array_merge( $default_args, $args );

	// If there is a next lesson
	if( $prev_id ) {
		$can_access = lift\can_access_lesson( $prev_id );
		if( !$can_access ) return '';

		ob_start(); ?>
		<div class="llms-course-nav llms-prev-lesson">
			<div class="llms-lesson-preview">
				<a class="llms-lesson-link" href="<?= lift\lesson_previous_url() ?>">
					<section class="llms-main">
						<h6 class="llms-pre-text"><?= $args['previous_label'] ?></h6>
						<?php if( $args['is_lesson_title'] === true ): ?>
							<h5 class="llms-h5 llms-lesson-title"><?= get_the_title( $prev_id ) ?></h5>
						<?php endif; ?>
					</section>
				</a>
			</div>
		</div>
		<?php $content = ob_get_clean();
	}
	else if( !$strict ){
		ob_start(); ?>
		<div class="llms-course-nav llms-back-to-course">
			<div class="llms-lesson-preview">
				<a class="llms-lesson-link" href="<?= lift\course_url() ?>">
					<section class="llms-main">
						<h6 class="llms-pre-text"><?= $args['course_label'] ?></h6>
						<?php if( $args['is_lesson_title'] ): ?>
							<h5 class="llms-h5 llms-lesson-title"><?= lift\course_title() ?></h5>
						<?php endif; ?>
					</section>
				</a>
			</div>
		</div>
		<?php $content = ob_get_clean();
	}

	return $content;
}


/**
 * Return the link to the previous item according to the lesson 
 * Will not return anything is user not allowed to acces the next lesson
 *
 * @param      integer  $post_id  The post identifier
 * 
 * @return     string URL
 */
function lesson_previous_url( $post_id = 0 ) {
	
	$lesson_id =  empty($post_id) ? lift\lesson_id() : (int) $post_id;
	$lesson = new \LLMS_Lesson( $lesson_id );

	$prev_id = $lesson->get_previous_lesson();

	if( $prev_id ){
		$can_access = lift\can_access_lesson( $prev_id );
		return $can_access ? get_permalink( $prev_id ) : '';
	}

	return get_permalink( $lesson->get('parent_course') );
}


/**
 * Return the link to the next item according to the lesson 
 * Will not return anything is user not allowed to acces the next lesson
 *
 * @param      integer  $post_id  The post identifier
 * 
 * @return     string URL
 */
function lesson_next_url( $post_id = 0 ) {
	
	$lesson_id =  empty($post_id) ? lift\lesson_id() : (int) $post_id;
	$lesson = new \LLMS_Lesson( $lesson_id );

	$next_id = $lesson->get_next_lesson();

	if( $next_id ){
		$can_access = lift\can_access_lesson( $next_id );
		return $can_access ? get_permalink( $next_id ) : '';
	}

	return get_permalink( $lesson->get('parent_course') );
}


/**
 * Return the <a> HTML element to the previous lesson
 *
 * @param      integer  $post_id  	The post identifier
 * @param      string  	$text  		(optional) Custom text
 * 
 * @return     string HTML
 */
function lesson_previous_link( $post_id = 0, $text = false ) {
	$url = lift\lesson_previous_url( $post_id ); 
	
	if( empty($url) ) return '';

	$text = $text ? $text : 'Previous Lesson';

	return '<a href="' . $url . '">' . $text . '</a>';
}


/**
 * Return the <a> HTML element to the next lesson
 *
 * @param      integer  $post_id  	The post identifier
 * @param      string  	$text  		(optional) Custom text
 * 
 * @return     string HTML
 */
function lesson_next_link( $post_id = 0, $text = false ) {
	$url = lift\lesson_next_url( $post_id );

	if( empty($url) ) return '';

	$text = $text ? $text : 'Next Lesson';

	return '<a href="' . $url . '">' . $text . '</a>';
}
