<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * Return the price of a given plan (with the currency)
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_price( $id = 0 ) {
  
  if( empty($id) ) return '';
  
  /**
   * If the WooCommerce integration is enable, we need to get the price from
   * the WC produt
   */
  if( lift\is_extension('woocommerce') ) {
    return lift\woocommerce_plan_price( $id );   
  }

  $plan = new \LLMS_Access_Plan( $id );

  return $plan->get_price( 'price' );
}


/**
 * Return the proceof a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_price_number( $id = 0 ) {
  
  if( empty($id) ) return '';

  /**
   * If the WooCommerce integration is enable, we need to get the price from
   * the WC produt
   */
  if( lift\is_extension('woocommerce') ) {
    return lift\woocommerce_plan_price_number( $id );
  }
  
  $plan = new \LLMS_Access_Plan( $id );

  return lift\is_plan_on_sale( $id ) ? $plan->get( 'sale_price' ) : $plan->get( 'price' ) ; 
}


/**
 * Return the int part of the price
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_price_number_int( $id = 0 ) {
  
  if( empty($id) ) return '';
  
  $price = (float) lift\plan_price_number( $id ); 
  return (int) floor($price); 
}


/**
 * Return the decimal part of the price
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_price_number_decimal( $id = 0 ) {
  
  if( empty($id) ) return '';
  
  $price      = (float) lift\plan_price_number( $id );
  $price_int  = (int) lift\plan_price_number_int( $id );
  
  return 100 * bcsub($price, $price_int, 2); 
}


/**
 * Return the price on sale of a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_price_sale( $id = 0 ) {
  
  if( empty($id) ) return '';
   
  /**
   * If the WooCommerce integration is enable, we need to get the price from
   * the WC produt
   */
  if( lift\is_extension('woocommerce') ) {
    return lift\woocommerce_plan_price_sale( $id );
  }
  
  $plan = new \LLMS_Access_Plan( $id );

  return $plan->get_price( 'sale_price' );
}


/**
 * Return the price if match the arguments
 *
 * @param      args     $array  Arguments
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_price_custom(  $id = 0, $args = array() ) {
  
  // All enable by default
  $default_args = array(
    'regular'     => true,
    'sale'        => true,
    'frequency'   => true,
    'expiration'  => true,
    'membership'  => true,
  );

  $args = array_merge( $default_args, $args );

  if( empty($id) ) return '';
  
  $price = '';

  if( $args['regular'] === true )
    $price = lift\plan_price( $id );

  if( $args['sale'] === true && lift\is_plan_on_sale( $id ) )
    $price = lift\plan_price_sale( $id );
  
  if( $args['frequency'] === true )
    $price .= ' ' . lift\plan_frequency( $id );

  if( $args['expiration'] === true && lift\is_plan_access_expiration( $id ) )
    $price .= ' ' . lift\plan_access_expiration( $id );

  
  return $price;
}


/**
 * Return the price on sale of a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function is_plan_on_sale( $id = 0 ) {
  
  if( empty($id) ) return '';

  /**
   * If the WooCommerce integration is enable, we need to get the price from
   * the WC produt
   */
  if( lift\is_extension('woocommerce') ) {
    return lift\woocommerce_is_plan_on_sale( $id );
  }

  $plan = new \LLMS_Access_Plan( $id );
  $is_sale = $plan->get( 'on_sale' ) === 'yes' ? true : false;

  return $is_sale;
}




