<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * Return the url to the checkout page for a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string   URL
 */
function plan_url( $id = 0 ) {
	if( empty($id) ) return '';
	
	$plan = new \LLMS_Access_Plan( $id );
	return esc_url( $plan->get_checkout_url() );
}


/**
 * Return the title of a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_title( $id = 0 ) {
	if( empty($id) ) return '';
	
	$plan = new \LLMS_Access_Plan( $id );
	return esc_attr( $plan->get( 'title' ) );
}


/**
 * Return the frequecy of a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_frequency( $id = 0 ) {
	
	if( empty($id) ) return '';
  
  /**
   * If the WooCommerce integration is enable, we need to get the price from
   * the WC produt
   */
  if( lift\is_extension('woocommerce') ) {
    return lift\woocommerce_plan_frequency( $id );
  }

	$plan = new \LLMS_Access_Plan( $id );
	
  return $plan->get_schedule_details();
}


/**
 * Return the enroll text of a given plan
 *
 * @param      integer  $id     The identifier
 *
 * @return     string   
 */
function plan_enroll_text( $id = 0 ) {
	
	if( empty($id) ) return '';
	
	$plan = new \LLMS_Access_Plan( $id );
	return $plan->get_enroll_text();
}


/**
 * If there is a time limitation
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function is_plan_access_expiration( $id = 0 ) {
  
  if( empty($id) ) return '';
  
  $plan = new \LLMS_Access_Plan( $id );
  $is_expiration = $plan->get( 'access_expiration' ) !== 'lifetime' ? true : false;

  return $is_expiration;
}


/**
 * Access expiration ext
 *
 * @param      integer  $id     The identifier
 *
 * @return     string
 */
function plan_access_expiration( $id = 0 ) {
  
  if( empty($id) ) return '';
  
  if( !lift\is_plan_access_expiration( $id ) ) return '';

  $plan = new \LLMS_Access_Plan( $id );

  return $plan->get_expiration_details();    
}
