<?php

namespace BetterLifter;

defined('ABSPATH') or die();

use BetterLifter as lift;

/**
 * Determines ability to access  lesson.
 *
 * @param      integer  $lesson_id  The lesson identifier
 * @param      integer  $user_id    The user identifier
 *
 * @return     boolean  True if able to access lesson, False otherwise.
 */
function can_access_lesson( $lesson_id = 0, $user_id = 0 ) {
  
  $lesson_id  = empty($lesson_id) ? lift\lesson_id() : (int) $lesson_id; 
  $user_id  = empty($user_id) ? get_current_user_id() : (int) $user_id; 
  
  $restrictions = \llms_page_restricted( $lesson_id, $user_id );

  return !$restrictions['is_restricted'];
}

/**
 * If the user has complete the lesson.
 *
 * @param      integer  $lesson_id  The lesson identifier
 * @param      integer  $user_id    The user identifier
 *
 * @return     boolean  True if able to access lesson, False otherwise.
 */
function has_complete_lesson( $lesson_id = 0, $user_id = 0 ) {
  
  $user_id  = empty($user_id) ? get_current_user_id() : (int) $user_id; 
  
  $lesson = lift\lesson_lifter( $lesson_id );
  $student = llms_get_student( $user_id );

  if( empty($student) || empty($lesson) || !is_object($lesson) ) return false;
  
  return $student->is_complete( $lesson->get( 'id' ), 'lesson' );
}
