<?php

namespace Tangible\ElementLogic;

defined( 'ABSPATH' ) or die( 'Nothing to see here' );

use \Tangible\Logic as logic;


/**
 * Define the conditional logic field
 * 
 * @see: https://developers.elementor.com/creating-a-new-control/
 */
if( !class_exists( '\Tangible\ElementLogic\Control' ) && class_exists( '\Elementor\Base_Data_Control' ) ) {
  class Control extends \Elementor\Base_Data_Control {

    /**
     * Slug of the field
     *
     * @var        string
     */
    const FIELD_NAME = 'logic_control';


    /**
     * Initializes the given control.
     */
    public function init() {}


    /**
     * Gets the type.
     *
     * @return     string  The type.
     */
    public function get_type() {
      return self::FIELD_NAME;
    }


    /**
     * Enqueue the control's dependencies
     */
    public function enqueue() {

        logic\enqueue();

        wp_enqueue_script('elementor-logic-control',
            Controller::$instance->state['url'] . 'control/script.js',
            ['jquery'],
            '1.0.0'
        );

        wp_enqueue_style('elementor-logic-control',
            Controller::$instance->state['url'] . 'control/style.css',
            [],
            '1.0.0'
        );
    }


    /**
     * Return the fields for the conditional logic form
     * 
     * @return array
     */
    public function get_logic_config() {

      $fields_enable = array(
        'title'       => 'Tangible Logic',
        'description' => 'Show if following conditions match:',
        'fields'      => Controller::$instance->state['group']->config['fields'] 
      );

      return $fields_enable;
    }


    /**
     * Add the Conditional Logic button in the settings
     */
    public function content_template() {
      
      ?>
        <span class="tangible-logic-control-button" data-tangible-logic="open">
          Open Conditional Logic Settings
        </span>

        <input type="hidden"
          id="<?= esc_attr( $this->get_control_uid() ) ?>"
          data-setting="{{{ data.name }}}"
          data-tangible-logic="input"
          data-tangible-logic-config='<?= esc_attr( json_encode($this->get_logic_config()) ) ?>'
        />
      <?php

    }

  }
}


